
C
C
C     PROGRAM   BBLTM
C
C     + + + PURPOSE + + +
C     Build the input file ("bltm.in") for "bltm"
C     Reads the terminal, "flow.in" or "old" for information.
C     Reads boundary values from either the terminal, "old", or "bc".
C     "bltm" allows DQQ and decay to vary reach to reach
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOBR   - Maximum number of branches allowed in model
C     NOSC   - Maximum number of grids allowed in a branch
C     NOCO   - Maximum number of constituents allowed
C
C     + + + LOCAL VARIABLES + + +
      INTEGER I,IBC,IED,IENG,IOUT(NOBR,NOSC),IPPR,ITDDS,ITRM,J,JJ,JGO,
     #        JNCD,JNCU,JPO,JTS,K,L,LL,LR,N,NBB,NBC,NBRCH,NEQ,NGB,NHR,
     #        NJNCT,NXSEC,LUIN,LUOUT,LUBC,M
      REAL AA,DDQQ,DQQ(NOBR,NOSC),DQV,DT,OPT(NOBR,NOCO,NOSC),
     #        GPT(NOBR,NOCO,NOSC),TGP(NOCO),X(NOSC),A1
      CHARACTER*4 LABEL(NOCO)
      CHARACTER*64 VERSN
      CHARACTER*80 TITLE
C
C     + + + LOCAL DEFINITIONS + + +
C     DDQQ   - Default value of DQQ`
C     IBC    - Code 1 to read file "bc", 0 otherwise
C     IED    - Code 0 to create a new file 1 to read "old" and "flow.in"
C     ITRM   - Code 1 for input from terminal, 0 otherwise
C     NBB    - Branch of new input for "bc" boundary condition
C     NGB    - Grid of new input for boundary conditions
C     OPT(N,L,K)- Old value of conc of const L, in parcel K, branch N
C     GPT(N,L,K)- Concentration of const L in parcel K, branch N
C     TGP(L) - Default value of initial concentraions
C
C     + + + INTRINSICS + + +
      INTRINSIC   ABS
C
C     + + + INPUT FORMATS + + +
 1500 FORMAT(A)
 1510 FORMAT(20X,I10)
 1530 FORMAT(20X,F10.3)
 1540 FORMAT (13X,I3,16X,F5.2,16X,I3,8X,I3)
 1550 FORMAT (3X,G11.4,I2,G11.4,4G10.3,F7.1,F6.3)
 1560 FORMAT(2X,I3,2X,I3,10F7.2)
C
 1700    FORMAT(7X,10F7.2)
C
C     + + + OUTPUT FORMATS + + +
 1600 FORMAT (1X,A)
 1610 FORMAT ('HEADER 1  ',10I7)
 1611 FORMAT (10X,10I7)
 1620 FORMAT ('HEADER 2  ',10F7.2)
 1621 FORMAT (10X,10F7.2)
 1630 FORMAT ('LABEL     ',I7,3X,A4,I7)
 1631 FORMAT (10X,I7,3X,A4,I7)
 1640 FORMAT('Branch',I4,5I7)
 1641 FORMAT (10X,5I7)
 1650 FORMAT(' Grid',I5,F7.3,I7,F7.3,7F7.2)
 1651 FORMAT (10X,F7.3,I7,8F7.3)
 1660 FORMAT(10X,10F7.2)
 1670 FORMAT('Time',I6,I7)
 1680 FORMAT(' B',I3,' G',I3,10F7.2)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)BBLTM - last modified MAY  3, 1996, kmf'
C
C     set unit numbers and open required data files
      LUIN  = 15
      LUOUT = 16
      LUBC  = 17
      OPEN (LUOUT, FILE='bltm.in')
C
      WRITE(*,*)' Enter 0 to create new file, 1 to edit existing file- '
      READ (*,*) IED
      IF (IED.EQ.0) THEN
        OPEN (LUIN,FILE='flow.in')
        READ (LUIN,1500)TITLE
        WRITE(*,1600)TITLE
        WRITE(*,*)'Enter new title (do not exceed 80) characters'
        WRITE(*,*)' '
        READ(*,1500)TITLE
        WRITE(LUOUT,1600)TITLE
        READ(LUIN,1510)NBRCH
        READ(LUIN,1510)NJNCT
        READ(LUIN,1510)NHR
        READ(LUIN,1510)JTS
        READ(LUIN,1510)JGO
        JPO=1
        WRITE(*,*) ' Enter number of time steps between outputs in',
     $             ' "parcel.out"-  '
        WRITE(*,*) ' To select default enter any letter except d or e,',
     $             ' Default=1, JPO=  '                                  HEJ02/92
        READ(*,'(I10)',ERR=10)JJ                                        meb01/92
          JPO=JJ
   10   CONTINUE
        READ(LUIN,1510)IENG
        WRITE(*,*) ' Enter number of constituents to be modeled-  '
        READ(*,*)NEQ
        ITDDS=0
        WRITE(LUOUT,1610)NBRCH,NJNCT,NHR,NEQ,JTS,JGO,JPO,ITDDS,IENG
        READ(LUIN,1530)DT
        READ(LUIN,1530)AA
        DQV=0.0
        WRITE(*,*) ' Enter minimum dispersive velocity,',
     $             ' Default=0.0, DQV= '
        READ(*,'(F10.0)',ERR=20)AA
          DQV=AA
   20   CONTINUE
        WRITE(LUOUT,1620)DT,DQV
        DO 40 L=1,NEQ
          TGP(L)=0.0
          WRITE(*,*) ' Enter label for constituent',L,' (4 char max)-  '
          READ(*,1500)LABEL(L)
          LR=L
          WRITE(*,*) ' Enter constituent no. to be tracked by PDC,',
     $               ' Default=' ,L,' LR= '
          READ(*,'(I10)',ERR=30)LL
            LR=LL
   30     CONTINUE
          WRITE(LUOUT,1630)L,LABEL(L),LR
   40   CONTINUE
        DDQQ=0.2
        IPPR=1
        DO 130 N=1,NBRCH
          READ(LUIN,1540)NXSEC,AA,JNCU,JNCD
          READ(LUIN,1500)TITLE
          WRITE(*,*) ' For branch',N,' enter the following:'
          WRITE(*,*) ' Initial number of parcels in each subreach,',
     #               ' Default= ',IPPR,' IPPR = '
          READ(*,'(I10)',ERR=50) JJ
            IPPR=JJ
   50     CONTINUE
          WRITE(LUOUT,1640)N,NXSEC,JNCU,JNCD,IPPR
          WRITE(*,*) ' For each cross section (grid)',
     #               ' enter the following:'
          DO 90 I=1,NXSEC
            READ(LUIN,1550)X(I)
            IOUT(N,I)=0
            IF(I.EQ.NXSEC) GO TO 90
              WRITE(*,*) ' Enter the following for branch',N,' Grid ',I
              WRITE(*,*) ' Dispersion factor = dis coef/(U*U*DT), ',
     #                   '    Default=',DDQQ,' DQQ = '
              READ(*,'(F10.0)',ERR=60)AA
                DDQQ=AA
   60         CONTINUE
              DQQ(N,I)=DDQQ
              WRITE(*,*) ' Initial concentration for subreach ',I
              DO 80 L=1,NEQ
                WRITE(*,*)(LABEL(L)),' Default=',TGP(L),',  Value = '
                READ(*,'(F10.0)',ERR=70)AA                              meb01/92
                  TGP(L)=AA
   70           CONTINUE
                GPT(N,L,I)=TGP(L)
   80         CONTINUE
   90     CONTINUE
          WRITE(*,*) ' For branch',N
  100     CONTINUE
            WRITE(*,*) ' Enter an output GRID (enter q to quit) '
            READ(*,'(I10)',ERR=110)I                                    meb01/92
              IOUT(N,I)=1
          GO TO 100
  110     CONTINUE
          JJ=NXSEC-1
          DO 120 I=1,JJ
            LL=7
            IF(NEQ.LE.7)LL=NEQ
            WRITE(LUOUT,1650) I,X(I),IOUT(N,I),DQQ(N,I),
     #                        (GPT(N,L,I),L=1,LL)
            IF(NEQ.GT.7)WRITE(LUOUT,1660)(GPT(N,L,I),L=8,NEQ)
  120     CONTINUE
          I=NXSEC
          WRITE(LUOUT,1650)I,X(I),IOUT(N,I)
  130   CONTINUE
      ELSE
C       ******  read data from "bltm.in" *********
        OPEN (LUIN, FILE='old')
        READ(LUIN,1500)TITLE
        WRITE(LUOUT,1600)TITLE
        READ(LUIN,1611) NBRCH,NJNCT,NHR,NEQ,JTS,JGO,JPO,ITDDS,IENG
        WRITE(LUOUT,1610) NBRCH,NJNCT,NHR,NEQ,JTS,JGO,JPO,ITDDS,IENG
        READ(LUIN,1621) DT,DQV
        WRITE(LUOUT,1620)DT,DQV
        DO 200 L=1,NEQ
          READ(LUIN,1631)M,LABEL(L),LR
          WRITE(LUOUT,1630)M,LABEL(L),LR
  200   CONTINUE
        DO 220 N=1,NBRCH
          READ(LUIN,1641) NXSEC,JNCU,JNCD,IPPR
          WRITE(LUOUT,1640)N,NXSEC,JNCU,JNCD,IPPR
          JJ=NXSEC-1
          DO 210 I=1,JJ
            LL=7
            IF (NEQ.LE.7) LL=NEQ
            READ(LUIN,1651) X(I),IOUT(N,I),DQQ(N,I),(GPT(N,L,I),L=1,LL)
            WRITE(LUOUT,1650)I,X(I),IOUT(N,I),DQQ(N,I),
     #                       (GPT(N,L,I),L=1,LL)
            IF(NEQ.GT.7) READ(LUIN,1621) (GPT(N,L,I),L=8,NEQ)
            IF(NEQ.GT.7)WRITE(LUOUT,1660)(GPT(N,L,I),L=8,NEQ)
 210      CONTINUE
          I=NXSEC
          READ(LUIN,1651) X(I),IOUT(N,I)
          WRITE(LUOUT,1650)I,X(I),IOUT(N,I)
 220    CONTINUE
      END IF
C
C     Build boundary conditions
      DO 300 N=1,NBRCH
        DO 300 I=1,NOSC
          DO 300 L=1,NEQ
            GPT(N,L,I)=0.0
  300 CONTINUE
      IBC=0
      WRITE(*,*) ' Enter 1 to read a boundary condition from',
     #           ' file "bc"- '
      WRITE(*,*) ' Enter 0 or n otherwise  '
      READ(*,'(I10)',ERR=310)JJ                                         meb01/92
        IBC=JJ
        IF(IBC.EQ.0)GO TO 310
          OPEN(LUBC,FILE='bc')
          WRITE(*,*) ' Enter BRANCH of boundary condition in',
     #               ' file "bc"- '
          READ(*,*)NBB
          WRITE(*,*) ' Enter  GRID  of boundary condition in',
     #               ' file "bc"- '
          READ(*,*)NGB
          READ(LUBC,1500)TITLE
          READ(LUBC,1600)TITLE
  310 CONTINUE
      ITRM=0
      WRITE(*,*)'Enter 1 to read boundary conditions from the terminal'
      WRITE(*,*)'Enter 0 or n otherwise  '
      READ(*,'(I10)',ERR=320)JJ                                         meb01/92
        ITRM=JJ
  320 CONTINUE
      DO 600 J=1,NHR
        DO 330 N=1,NBRCH
          DO 330 I=1,NOSC
            IOUT(N,I)=0
            DO 330 L=1,NEQ
              OPT(N,L,I)=GPT(N,L,I)
  330   CONTINUE
C       Read boundary conditions from file "bc" 
        IF(IED.EQ.0) GO TO 350
          READ(LUIN,1611,END=350)NBC
            IF(NBC.EQ.0)GO TO 350
              DO 340 K=1,NBC
                READ(LUIN,1560)N,I,(GPT(N,L,I),L=1,NEQ)
                IOUT(N,I)=1
  340         CONTINUE
  350   CONTINUE
        IF(IBC.NE.1)GO TO 360
C         Read boundary condition from file "bc" 
          READ(LUBC,*)A1,(GPT(NBB,L,NGB),L=1,NEQ)
          IOUT(NBB,NGB)=1
  360   CONTINUE
        IF(ITRM.NE.1)GO TO 390
C         Read boundary conditions from the terminal
          WRITE(*,*) ' For time step',J,' enter number of new',
     #               ' boundary conditions-  '
          READ(*,*)NBC
          DO 380 K=1,NBC
            WRITE(*,*) ' Enter BRANCH of new boundary condition- '
            READ(*,*)N
            WRITE(*,*) ' Enter  GRID  of new boundary condition- '
            READ(*,*)I
            IOUT(N,I)=1
            DO 370 L=1,NEQ
              GPT(N,L,I)=OPT(N,L,I)
              WRITE(*,*)LABEL(L),' Default=',OPT(N,L,I),' Value= '
              READ(*,'(F10.0)',ERR=370)AA                               meb01/92
              GPT(N,L,I)=AA
  370       CONTINUE
  380     CONTINUE
C         Find number of new boundary conditions
  390   CONTINUE
        NBC=0
        DO 500 N=1,NBRCH
          DO 500 I=1,NOSC
           JJ=0
           IF (IOUT(N,I).EQ.0)GO TO 500
             DO 510 L=1,NEQ
               AA=ABS(GPT(N,L,I)-OPT(N,L,I))
               IF(AA.GT.0.0001)JJ=1
  510        CONTINUE
             IF(JJ.EQ.0)IOUT(N,I)=0
             NBC=NBC+JJ
  500   CONTINUE
C       Write boundary conditions
        WRITE(LUOUT,1670)J,NBC
        IF (NBC.EQ.0)GO TO 600
          DO 520 N=1,NBRCH
            DO 520 I=1,NOSC
              IF(IOUT(N,I).EQ.0)GO TO 520
              WRITE(LUOUT,1680)N,I,(GPT(N,L,I),L=1,NEQ)
  520     CONTINUE
  600 CONTINUE
C
C     close files
      ENDFILE ( LUOUT )
      CLOSE   ( LUOUT )
      CLOSE   ( LUIN  )
      IF (IBC .EQ. 1) CLOSE ( LUBC )
C
      STOP
      END
