C     ********** MULTIPLE BRANCH ESTUARY TRANSPORT MODEL (DG version)*****
C                This version does not need subroutine TDDS
C
      PROGRAM   MBLTM
C
C     + + + PURPOSE + + +
C     All boundry conditions (BC) represent the ave during the time step.
C     The first BC represents cond. from time 0 to dt, for example.
C     The Q are at the grid point with QT occuring just upstream of grid
C     The A and W's are reach averaged values
C     Nxxx or Gxxx variables defined for all branches.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOBR   - Maximum number of branches allowed in model
C     NOSC   - Maximum number of cross sections (grids) allowed in branch
C     NOPR   - Maximum number of parcels allowed in branch
C              (NOPR should be at least 20 + 2 times NOSC)
C     NOCO   - Maximum number of constituents allowed
C
C     + + + COMMONS + + +
      INTEGER   NXSEC(NOBR), NBRCH, IENG
      COMMON  /CMBLTM/  NXSEC, NBRCH, IENG
C
C     + + + COMMON DEFINTIONS + + +
C     IENG   - Input units: 0=metric (except river miles), 1=English
C     NBRCH  - Number of branches
C     NXSEC(N) Number of subreaches in branch N
C
C     + + + LOCAL VARIABLES + + +
      INTEGER I,I1,I2,IDAY,IDV,II,INX,IOUT(NOBR,NOSC),IPX(NOPR),IRC,J,
     #        JCD(NOBR),JN,JNCD(NOBR),JNCU(NOBR),JGO,JPO,JTS,K,K1,
     #        KAI(NOSC),L,LR(NOCO),M,N,NEQ,NHR,NIPX(NOBR,NOPR),
     #        NJNCT,NKAI(NOBR,NOSC),NN,NS(NOBR),ITDDS,NBC,
     #        IPPR(NOBR),KK,LUIN,LUOUT,LUPO,LUFLW
      REAL    A(NOSC),AA,CJ(NOCO),DQQ(NOBR,NOSC),DT,DTT,DVD(NOBR),
     #        DVU(NOBR),FLOW(NOBR,4,NOSC),DX(NOBR,NOSC),
     #        GPDC(NOBR,NOCO,NOPR),GPDF(NOBR,NOCO,NOPR),PDQQ(NOSC),
     #        GPH(NOBR,NOPR),GPT(NOBR,NOCO,NOPR),GPTD(NOBR,NOCO),
     #        GPTI(NOBR,NOCO,NOPR),GPTR(NOBR,NOCO,NOPR),GPTU(NOBR,NOCO),
     #        GPV(NOBR,NOPR),GTRIB(NOBR,NOCO,NOSC),GVU(NOBR,NOSC),HR,
     #        PDC(NOCO,NOPR),PDF(NOCO,NOPR),PH(NOPR),PT(NOCO,NOPR),
     #        PTD(NOCO),PTI(NOCO,NOPR),PTR(NOCO,NOPR),PTU(NOCO),
     #        PV(NOPR),PX(NOPR),Q(NOSC),QD(NOBR),QI,QT(NOSC),
     #        QU(NOBR),SVOL,TIME,TRIB(NOCO,NOSC),VJ,VOL,
     #        VU(NOSC),W(NOSC),X(NOBR,NOSC),DQV,RPT(NOCO),XFACT
      CHARACTER*4  LABEL(NOCO)
      CHARACTER*80 TITLE
      CHARACTER*64 VERSN
C
C     + + + LOCAL DEFINITIONS + + +
C     A(I)      Average area in each subreach I (sq m)
C     CJ(L)     Conc. of L at junction M
C     DQQ(N,I)  Dispersion factor for branch N, gird I (D/U*U*DT)
C     DQV       Minimum dispersive velocity (m/s or ft/s)
C     DT        Time step size (hours)
C     DVD(N)    Unknown volume in outflow at d/s end
C     DVU(N)    Unknown volume in outflow at u/s end
C     FLOW(N,M,I) Flow field at grid or subreach I
C             M=1 Discharge at grid I (cu m/s or ft/s), during DT
C             M=2 Cross sectional area, subreach I (sq m or ft) end of DT
C             M=3 Top width (m or ft), Subreach I, at end of DT
C             M=4 Trib. flow (cu m/s or ft/s) us grid I, during DT
C     DX(N,I)     Length of subreach I in branch N
C     GPDC(N,L,K) Increase of L in parcel N,K due to reaction LR(L)
C     GPDF(N,L,K) Increase of L in parcel N,K due to dispersion
C     GPH(N,K)    Time in hours when parcel K entered branch N
C     GPT(N,L,K)  Concentration of constituent L in parcel N,K
C     GPTD(N,L)   Flux of L at ds end of branch N
C     GPTI(N,L,K) Conc. of L as parcel K entered branch N
C     GPTR(N,L,K) Increase of L in parcel N,K due to tribs.
C     GPTU(N,L)   Flux of L at us end of branch N
C     GPV(N,K)    Volume of parcel K in branch N
C     GTRIB(N,L,I)Conc. of L in trib. at grid I of branch N
C                 (tribs can not occur at first or last grid of branch)
C     GVU(N,I)    Volume of parcel u/s of grid I in branch N
C     HR          Hour of the day
C     IDAY        Days since model started
C     INX         Number of subreaches
C     IOUT(N,I)   Flag (1 = output) for grid I in branch N
C     IPPR(N)     Initial number of parcels per reach in branch N
C     IPX(K)      Subreach where u/s boundary of parcel K is located
C     IRC         Code for reading data in FINK (1=read data, 0=no read)
C     ITDDS       Code 0=read DAFLOW output, 1=BRANCH output
C     J           Time step
C     JCD(M)      Code for junction mixing (0=mixed, 2=not mixed)
C     JNCD(N)     D/s junction no. for branch N (number interior first)
C     JNCU(N)     U/s junction no. for branch N (number interior first)
C     JGO         Number of time steps between output prints for grids
C     JPO         Number of time steps between output prints for parcel
C     JTS         Number of time steps from midnight to start of model
C     K           Parcel number
C     KAI(I)      Parcel at grid I
C     L           Constituent number
C     LABEL(NOCO) Name of constituents (4 letters max)
C     LR(L)       Index denoting that the decay of constituent L
C                   due to the presence of constituent LR(L) is tracked
C     NBC         Number of boundary conditions,
C     NEQ         Number of equations (constituents)
C     NHR         Number of time steps to be modeled
C     NIPX(N,K)   Subreach in which the upstream boundary of parcel N,K is
C     NJNCT       Number of interior junctions
C     NKAI(N,I)   Parcel at branch N, grid I
C     NS(N)       Number of parcels in branch N
C     PDC(L,K)    Change in initial concentration L due to reaction K
C     PDF(L,K)    Change in initial concentration due to dispersion
C     PDQQ(I)     DQQ for single reach
C     PH(K)       Time parcel K entered reach in hours from day 0
C     PT(L,K)     Conc. of constituent L in parcel K
C     PTD(L)      Conc. of const. L in parcel d/s, avg'd over time step
C     PTI(L,K)    Initial concentration of constituent L in parcel K
C     PTR(L,K)    Change in initial concentration due to tributary inflow
C     PTU(L)      Conc. of const. L in parcel u/s, avg'd over time step
C     PV(K)       Volume of parcel K
C     PX(K)       Location of upstream boundry of parcel in grid units
C     Q(I)        River flow at grid I (cu m/s or ft/s),
C                   trib. inflow occurs just u/s of grid
C     QD(N)       Average flow at downstream end of branch N
C     QI          Minimum flow of interest (flows<QI are considered zero).
C     QT(I)       Trib inflow (cu m/s or ft/s) enter just u/s of grid I
C     QU(N)       Average flow at upstream end of branch N
C     RPT(L)      Initial concentration of L in reach
C     TITLE       Title up to 80 characters
C     TRIB(L,I)   Concentration of constituent L in trib at grid I
C                   (tribs can not occur at first or last grid)
C     VJ          Inflow volume to junction
C     VU(I)       Volume of parcel u/s of grid I
C     W(I)        Average top width in subreach I (m or ft)
C     X(N,I)      Distance of grid I from u/s of branch N (input in miles)
C     XFACT       Conversion from miles to feet or meters,
C
C     + + + INTRINSICS + + +
      INTRINSIC  FLOAT, IFIX, ABS, MOD
C
C     + + + EXTERNALS + + +
      EXTERNAL   ROUTE
C
C     + + + INPUT FORMATS + + +
1000  FORMAT (A)
1010  FORMAT (10X,10I7)
1020  FORMAT (10X,10F7.2)
1030  FORMAT (10X,I7,3X,A4,I7)
1050  FORMAT (10X,F7.3,I7,8F7.3)
1060  FORMAT (3X,I2,3X,I2,10F7.3)
1100  FORMAT (15X,4E18.5)
1110  FORMAT (3I5)
1120  FORMAT (8G10.4)
C
C     + + + OUTPUT FORMATS + + +
2000  FORMAT (20X,A,/)
2010  FORMAT (' The',I3,' Branch model has ',I3,' Internal junctions.')
2011  FORMAT (I5,' Time steps are run, each',F5.2,' Hours long.')
2020  FORMAT (' The model starts at',F6.2,' Hours past midnight.')
2021  FORMAT (' The grid   output is given in "bltm.out"    every',I4,
     #        ' Time steps.')
2022  FORMAT (' The parcel output is given in "parcel.out" every',I4,
     #        ' Time steps.')
2030  FORMAT (I4,' Constituents are routed and the Minimum Dispersive',
     #        ' velocity is',F7.3,'  m/s.')
2040  FORMAT (I4,' Constituents are routed and the Minimum Dispersive',
     #        ' velocity is',F7.3,'  ft/s.')
2050  FORMAT (' Flow conditions from DAFLOW used')
2060  FORMAT (' Flow conditions from BRANCH used')
2080  FORMAT (' Input units are (meters) except river miles')
2090  FORMAT (' Input units are English (feet and river miles)')
2100  FORMAT (/,30X,'INITIAL CONDITIONS')
2110  FORMAT (/,3X,'Grid  River Discharge    Area    Top Width',
     #        3X,'Trib. Flow',22X,'Initial Concentration')
2120  FORMAT (10X,'Mile    cu m/s    sq m       meters',
     #        7X,'cu m/s',10(3X,A4))
2121  FORMAT (10X,'Mile   cu ft/s   sq ft       feet  ',
     #        6X,'cu ft/s',10(3X,A4))
2130  FORMAT (/,10X,'Branch',I4,'  Extends from JUCT',
     #        I3,'  to junction',I3,/)
2140  FORMAT (I7,F7.2,F10.2,F8.2,F13.2,F13.2,10F7.2)
2300  FORMAT (1H ,'  Day',I4,'  Hour',F4.1,6X,'VU',8X,'PV',8X,'PH',
     #        10(4X,A4))
2310  FORMAT (1H ,' Br',I4,' Grid',I3,'  K=',I3,2G10.3,F7.2,10F8.3)
2320  FORMAT (1H ,38X,'PTI',7X,10F8.3)
2330  FORMAT (1H ,38X,'PDF',7X,10F8.3)
2340  FORMAT (1H ,38X,'PTR',7X,10F8.3)
2350  FORMAT (1H ,38X,'PDC',7X,10F8.3)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)CBLTM - last modified MAY  2, 1996, kmf'

C     open external files
C
      LUIN=15
      LUOUT=16
      LUPO=17
      LUFLW=18
      OPEN (LUIN,FILE='bltm.in')
      OPEN (LUOUT,FILE='bltm.out')
      OPEN (LUPO,FILE='parcel.out')
      OPEN (LUFLW,FILE='bltm.flw')
C
C     zero arrays and priliminaries
C
      DO 60 N=1,NOBR
        JNCU(N)=0
        JNCD(N)=0
        NS(N)=0
        NXSEC(N)=0
        IPPR(N)=1
        JCD(N)=0
        DO 30 I=1,NOSC
          GVU(N,I)=0.0
          DQQ(N,I)=0
          NKAI(N,I)=0
          DO 10 M=1,4
            FLOW(N,M,I)=0.0
 10       CONTINUE
          IOUT(N,I)=0
          DX(N,I)=0.0
          X(N,I)=0.0
          DO 20 L=1,NOCO
            GTRIB(N,L,I)=0.0
 20       CONTINUE
 30     CONTINUE
        DO 50 K=1,NOPR
          NIPX(N,K)=0
          GPV(N,K)=0.0
          GPH(N,K)=0.0
          DO 40 L=1,NOCO
            GPT(N,L,K)=0.0
            GPTI(N,L,K)=0.0
            GPDF(N,L,K)=0.0
            GPTR(N,L,K)=0.0
            GPDC(N,L,K)=0.0
 40       CONTINUE
 50     CONTINUE
 60   CONTINUE
      DO 70 L=1,NOCO
        LR(L)=0
 70   CONTINUE
C
C     read common input
C
      READ(LUIN,1000) TITLE
      WRITE(LUOUT,2000) TITLE
      READ(LUIN,1010) NBRCH,NJNCT,NHR,NEQ,JTS,JGO,JPO,ITDDS,IENG
      READ(LUIN,1020) DT,DQV
      WRITE(LUOUT,2010) NBRCH,NJNCT
      WRITE(LUOUT,2011)NHR,DT
      AA=DT*FLOAT(JTS)
      WRITE(LUOUT,2020) AA
      WRITE(LUOUT,2021) JGO
      WRITE(LUOUT,2022) JPO
      IF (IENG .EQ. 0) THEN
        WRITE(LUOUT,2030) NEQ,DQV
      ELSE
        WRITE(LUOUT,2040) NEQ,DQV
      END IF
C
C     read data for each branch
C
      DO 100 M=1,NEQ
        READ(LUIN,1030)L,LABEL(M),LR(M)
 100  CONTINUE
      DO 120 N=1,NBRCH
        READ(LUIN,1010) NXSEC(N),JNCU(N),JNCD(N),IPPR(N)
        IF (IPPR(N) .LE. 0) IPPR(N)=1
        IF (IPPR(N)*(NXSEC(N)-1) .GT. (NOPR-2)) IPPR(N)=(NOPR-2)/
     #   (NXSEC(N)-1)
        I1=NXSEC(N)
        DO 110 I=1,I1
          I2=7
          IF (NEQ.LT.7) I2=NEQ
          READ(LUIN,1050) X(N,I),IOUT(N,I),DQQ(N,I),(RPT(L),L=1,I2)
          IF (NEQ.GT.7 .AND. I .LT. I1) READ(LUIN,1020) (RPT(L),L=8,NEQ)
          IF (I .LT. I1) THEN
            DO 105 II=(I-1)*IPPR(N)+1,I*IPPR(N)
              DO 104 L=1,NEQ
                GPT(N,L,II)=RPT(L)
 104          CONTINUE
 105        CONTINUE
          END IF
 110    CONTINUE
 120  CONTINUE
      IF(ITDDS.EQ.0)THEN
        WRITE(LUOUT,2050)
      ELSE
        WRITE(LUOUT,2060)
      END IF
      DO 130 N=1,NBRCH
        DO 130 I=1,NXSEC(N)
          READ(LUFLW,1100) (FLOW(N,M,I),M=1,4)
 130  CONTINUE
C
C     make preliminary compt. and write init.conditions
C
      IF (IENG .EQ. 0) THEN
        WRITE(LUOUT,2080)
        XFACT=1609.34
      ELSE
        WRITE(LUOUT,2090)
        XFACT=5280.00
      END IF
      WRITE(LUOUT,2100)
      WRITE(LUOUT,2110)
      IF (IENG .EQ. 0) THEN
        WRITE(LUOUT,2120)(LABEL(L),L=1,NEQ)
      ELSE
        WRITE(LUOUT,2121)(LABEL(L),L=1,NEQ)
      END IF
      DO 250 N=1,NBRCH
        INX=NXSEC(N)-1
        DO 220 I=1,INX
          DX(N,I)=(X(N,I+1)-X(N,I))*XFACT
          DO 210 KK=1,IPPR(N)
            K=(I-1)*IPPR(N)+KK
            NIPX(N,K)=I
            IF (KK.EQ.1) NIPX(N,K)=I-1
            PX(K)=FLOAT(I)+FLOAT(KK-1)/FLOAT(IPPR(N))
            IF(ITDDS.EQ.0)THEN
              A(I)=FLOW(N,2,I)
              W(I)=FLOW(N,3,I)
            ELSE
              A(I)=0.5*(FLOW(N,2,I)+FLOW(N,2,I+1))
              W(I)=0.5*(FLOW(N,3,I)+FLOW(N,3,I+1))
            END IF
            GPV(N,K)=A(I)*DX(N,I)/FLOAT(IPPR(N))
            GPH(N,K)=-K
            DO 200 L=1,NEQ
              GPTI(N,L,K)=GPT(N,L,K)
 200        CONTINUE
 210      CONTINUE
          NKAI(N,I)=1+(I-1)*IPPR(N)
          GVU(N,I)=0.0
 220    CONTINUE
        NIPX(N,1)=1
        NS(N)=INX*IPPR(N)
        NKAI(N,NXSEC(N))=NS(N)
        GVU(N,NXSEC(N))=GPV(N,NS(N))
        WRITE(LUOUT,2130) N,JNCU(N),JNCD(N)
        I1=NXSEC(N)-1
        DO 230 I=1,I1
          II=1+(I-1)*IPPR(N)
          WRITE(LUOUT,2140)I,X(N,I),FLOW(N,1,I),A(I),W(I),FLOW(N,4,I),
     #                     (GPT(N,L,II),L=1,NEQ)
          WRITE(LUOUT,*)'DQQ = ',DQQ(N,I)
  230   CONTINUE
        I=NXSEC(N)
        WRITE(LUOUT,2140) I,X(N,I),FLOW(N,1,I),A(I),W(I),FLOW(N,4,I)
        IF(JPO.EQ.0)GO TO 250
          J=0
          WRITE(LUPO,1110)J,N,NS(N)
          WRITE(LUPO,1120)(PX(K),K=1,NS(N))
          DO 240 L=1,NEQ
            WRITE(LUPO,1120)(GPT(N,L,K),K=1,NS(N))
  240     CONTINUE
  250 CONTINUE
      QI=1.0
      WRITE(*,*)'Enter accuracy criteria, that is, the minimum flow '
      WRITE(*,*)'of interest.  Default = 1.0   '
      READ(*,'(F10.0)',ERR=260)AA
        QI=AA
  260 CONTINUE
C
C     start time loop
C
      DO 760 J=1,NHR
        WRITE(*,*)'STARTING TIME STEP',J
        IRC=1
C
C       set boundary and junction values
C
        DO 320 N=1,NBRCH
          DVU(N)=0.0
          DVD(N)=0.0
          DO 310 L=1,NEQ
            GPTU(N,L)=0.0
            GPTD(N,L)=0.0
 310      CONTINUE
 320    CONTINUE
        DO 330 N=1,NJNCT
          JCD(N)=2
 330    CONTINUE
C       READ BOUNDRY CONDITIONS
        READ(LUIN,1010)NBC
        IF(NBC.GT.0)THEN
          DO 350 M=1,NBC
            READ(LUIN,1060)N,I,(GTRIB(N,L,I),L=1,NEQ)
  350     CONTINUE
        END IF
        DO 360 N=1,NBRCH
          DO 360 L=1,NEQ
            GPTU(N,L)=GTRIB(N,L,1)
            GPTD(N,L)=GTRIB(N,L,NXSEC(N))
  360   CONTINUE
C
C       route branches
C
        DO 490 N=1,NBRCH
          I1=NXSEC(N)
          DO 405 I=1,I1
            Q(I)=FLOW(N,1,I)
            QT(I)=FLOW(N,4,I)
  405     CONTINUE
          READ(LUFLW,1100) ((FLOW(N,M,I),M=1,4),I=1,I1)
          DO 410 I=1,I1
            PDQQ(I)=DQQ(N,I)
            KAI(I)=NKAI(N,I)
            VU(I)=GVU(N,I)
            DO 400 L=1,NEQ
              TRIB(L,I)=GTRIB(N,L,I)
 400        CONTINUE
            IF (ITDDS.GT.0) THEN
              Q(I)=0.5*(Q(I)+FLOW(N,1,I))
              QT(I)=0.5*(QT(I)+FLOW(N,4,I))
              IF(I.LT.I1)A(I)=0.5*(FLOW(N,2,I)+FLOW(N,2,I+1))
              IF(I.LT.I1)W(I)=0.5*(FLOW(N,3,I)+FLOW(N,3,I+1))
            ELSE
              Q(I)=FLOW(N,1,I)
              QT(I)=FLOW(N,4,I)
              A(I)=FLOW(N,2,I)
              W(I)=FLOW(N,3,I)
            END IF
 410      CONTINUE
          QU(N)=Q(1)
          QD(N)=Q(NXSEC(N))
          DO 420 L=1,NEQ
            PTU(L)=GPTU(N,L)
            PTD(L)=GPTD(N,L)
 420      CONTINUE
          K1=NS(N)
          DO 440 K=1,K1
            IPX(K)=NIPX(N,K)
            PV(K)=GPV(N,K)
            PH(K)=GPH(N,K)
            DO 430 L=1,NEQ
              PT(L,K)=GPT(N,L,K)
              PTI(L,K)=GPTI(N,L,K)
              PDF(L,K)=GPDF(N,L,K)
              PTR(L,K)=GPTR(N,L,K)
              PDC(L,K)=GPDC(N,L,K)
 430        CONTINUE
 440      CONTINUE
          CALL ROUTE
     I      (A,PDQQ,DT,J,JTS,LR,NEQ,N,NXSEC(N),Q,QI,QT,TRIB,
     M      W,IRC,KAI,NS(N),PDC,PDF,PH,PT,PTD,PTI,PTR,PTU,PV,
     O      DVD(N),DVU(N),IPX,VU,DQV,LUOUT)
          I1=NXSEC(N)
          DO 450 I=1,I1
            NKAI(N,I)=KAI(I)
            GVU(N,I)=VU(I)
 450      CONTINUE
          K1=NS(N)
          DO 470 K=1,K1
            NIPX(N,K)=IPX(K)
            GPV(N,K)=PV(K)
            GPH(N,K)=PH(K)
            DO 460 L=1,NEQ
              GPT(N,L,K)=PT(L,K)
              GPTI(N,L,K)=PTI(L,K)
              GPDF(N,L,K)=PDF(L,K)
              GPTR(N,L,K)=PTR(L,K)
              GPDC(N,L,K)=PDC(L,K)
 460        CONTINUE
 470      CONTINUE
          DO 480 L=1,NEQ
            GPTU(N,L)=PTU(L)
            GPTD(N,L)=PTD(L)
 480      CONTINUE
 490    CONTINUE
C
C       update junction concentrations and codes
C
C       compute inflow flux at known junction
        DTT=3600.0*DT
        DO 640 M=1,NJNCT
          JN=0
 500      CONTINUE
            JN=JN+1
          IF (JCD(JN).EQ.0) GO TO 500
          VJ=0.0
          DO 510 L=1,NEQ
            CJ(L)=0.0
 510      CONTINUE
          DO 550 N=1,NBRCH
            IF (JNCU(N).EQ.JN)THEN
              IF (QU(N).LT.0.0) THEN
                IDV=IFIX(DVU(N)*1.0+0.5)
                IF(IDV.NE.0.AND.JNCD(N).LE.NJNCT) GO TO 500
                VJ=VJ-QU(N)*DTT
                DO 520 L=1,NEQ
                  CJ(L)=CJ(L)+GPTU(N,L)
 520            CONTINUE
              END IF
            END IF
            IF (JNCD(N).EQ.JN)THEN
              IF (QD(N).GT.0.0) THEN
                IDV=IFIX(DVD(N)*1.0+0.5)
                IF (IDV.NE.0.AND.JNCU(N).LE.NJNCT) GO TO 500
                VJ=VJ+QD(N)*DTT
                DO 540 L=1,NEQ
                  CJ(L)=CJ(L)+GPTD(N,L)
 540            CONTINUE
              END IF
            END IF
 550      CONTINUE
          DO 560 L=1,NEQ
            IF(VJ.GT.0.0) CJ(L)=CJ(L)/VJ
 560      CONTINUE
          JCD(JN)=0
C
C         update GPTU,GPTD,DVU,DVD,PT,PTI,PTR
C
          DO 630 N=1,NBRCH
            IF (JNCU(N).EQ.JN)THEN
              IF (QU(N).GT.0.0) THEN
                VOL=GPV(N,1)+DVD(N)
                DO 600 L=1,NEQ
                  GPTU(N,L)=CJ(L)*DTT*QU(N)
                  GPT(N,L,1)=(CJ(L)*QU(N)*DTT+GPT(N,L,1)*VOL)/VOL
                  GPTI(N,L,1)=CJ(L)
                  GPTR(N,L,1)=GPT(N,L,1)-CJ(L)
                  GPTD(N,L)=GPTD(N,L)+CJ(L)*DVD(N)
 600            CONTINUE
              END IF
              DVD(N)=0.0
            END IF
            IF (JNCD(N).EQ.JN)THEN
              IF (QD(N).LT.0.0) THEN
                VOL=GPV(N,NS(N))+DVU(N)
                DO 620 L=1,NEQ
                  GPTD(N,L)=CJ(L)*DTT*QD(N)
                  GPT(N,L,NS(N))=(-CJ(L)*QD(N)*DTT+GPT(N,L,NS(N))*VOL)
     #                           /VOL
                  GPTI(N,L,NS(N))=CJ(L)
                  GPTR(N,L,NS(N))=GPT(N,L,NS(N))-CJ(L)
                  GPTU(N,L)=GPTU(N,L)+CJ(L)*DVU(N)
 620            CONTINUE
              END IF
              DVU(N)=0.0
            END IF
 630      CONTINUE
 640    CONTINUE
C
C       write results
C
C       *********************** THIS STATEMENT SKIPS PARCEL OUT
        DO 730 N=1,NBRCH
          PX(1)=1.0
          INX=NXSEC(N)-1
          DO 720 I=1,INX
            VOL=-GVU(N,I)
            K=NKAI(N,I)-1
 700        CONTINUE
              K=K+1
              VOL=VOL+GPV(N,K)
            IF (K.LT.NKAI(N,I+1)) GO TO 700
            VOL=VOL-GPV(N,K)+GVU(N,I+1)
            K=NKAI(N,I)
            SVOL=GPV(N,K)-GVU(N,I)
 710        CONTINUE
              K=K+1
              PX(K)=FLOAT(I)+1.0
              VJ=ABS(VOL)
              IF(VJ.GT.1.0E-6)PX(K)=FLOAT(I)+SVOL/VOL
              SVOL=SVOL+GPV(N,K)
            IF (K.LT.NKAI(N,I+1)) GO TO 710
 720      CONTINUE
          IF(JPO.EQ.0)GO TO 730
            IF(MOD(J,JPO).NE.0)GO TO 730
              WRITE(LUPO,1110)J,N,NS(N)
              WRITE(LUPO,1120)(PX(K),K=1,NS(N))
              DO 770 L=1,NEQ
                WRITE(LUPO,1120)(GPT(N,L,K),K=1,NS(N))
  770         CONTINUE
  730   CONTINUE
        TIME=FLOAT(J+JTS)*DT
        IDAY=IFIX(TIME/24.0)+1
        HR=TIME-FLOAT(IDAY-1)*24.0
        IF (MOD(J,JGO).NE.0)GO TO 760
          WRITE(LUOUT,2300) IDAY,HR,(LABEL(L),L=1,NEQ)
          DO 750 N=1,NBRCH
            NN=NXSEC(N)
            DO 740 I=1,NN
              IF (IOUT(N,I).NE.1) GO TO 740
                K=NKAI(N,I)
                WRITE(LUOUT,2310) N,I,K,GVU(N,I),GPV(N,K),GPH(N,K),
     #            (GPT(N,L,K),L=1,NEQ)
                WRITE(LUOUT,2320) (GPTI(N,L,K),L=1,NEQ)
                WRITE(LUOUT,2330) (GPDF(N,L,K),L=1,NEQ)
                WRITE(LUOUT,2340) (GPTR(N,L,K),L=1,NEQ)
                WRITE(LUOUT,2350) (GPDC(N,L,K),L=1,NEQ)
 740        CONTINUE
 750      CONTINUE
 760  CONTINUE
C
C     close files
      CLOSE   ( LUIN  )
      ENDFILE ( LUOUT )
      CLOSE   ( LUOUT )
      CLOSE   ( LUFLW )
      ENDFILE ( LUPO  )
      CLOSE   ( LUPO  )
C
      STOP
      END
C
C     **************************  ROUTE ********************************
C
      SUBROUTINE   ROUTE
     I                  (A,DQQ,DT,J,JTS,LR,NEQ,NNN,NXSEC,Q,QI,QT,TRIB,
     M                   W,IRC,KAI,NS,PDC,PDF,PH,PT,PTD,PTI,PTR,PTU,PV,
     O                   DVD,DVU,IPX,VU,DQV,LUOUT)
C
C     + + + PURPOSE + + +
C     Route through each branch.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'

C     + + + PARAMETER DEFINITIONS + + +
C     NOSC   - Maximum number of cross sections (grids) allowed in branch
C     NOPR   - Maximum number of parcels allowed in branch
C              (NOPR should be at least 20 + 2 times NOSC)
C     NOCO   - Maximum number of constituents allowed
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IRC,J,JTS,KAI(NOSC),LR(NOCO),NEQ,NNN,NS,IPX(NOPR),NXSEC,
     #        LUOUT
      REAL    A(NOSC),DQQ(NOSC),DQV,DT,DVD,DVU,PDC(NOCO,NOPR),DQMIN,
     #        PDF(NOCO,NOPR),PH(NOPR),PT(NOCO,NOPR),PTD(NOCO),
     #        PTI(NOCO,NOPR),PTR(NOCO,NOPR),PTU(NOCO),PV(NOPR),
     #        Q(NOSC),QI,QT(NOSC),TRIB(NOCO,NOSC),VU(NOSC),W(NOSC)
C
C      + + + ARGUMENT DEFINITIONS + + +
C     A(I)    - Average area in subreach I (sq m)
C     DQQ(I)  - Dispersion factor D/(U*U*DT)
C     DT      - Time step size in hours
C     J       - Time step
C     JTS     - Number of time steps from midnight to start of model
C     LR(L)   - Index denoting that the decay of constituent L due
C               to the presence of constituent LR(L) is tracked
C     NEQ     - Number of equations (constituents)
C     NNN     - Branch number
C     NXSEC   - Number of Eulerian grids (subreaches in a branch)
C     Q(I)    - River flow at grid I (cu m/s),
C               trib inflow occurs just u/s of grid
C     QI      - Minimum flow of interest (flows<QI are considered zero).
C     QT(I)   - Tributary inflow at grid I (cu m/s), enter just u/s of grid
C     TRIB(L,I)- Concentration of constituent L in trib at grid I
C               (tribs can not occur at first or last grid)
C     W(I)    - Average top width in subreach I (m)
C     IRC     - Code for reading data in FINK (1=read data, 0=no read)
C     KAI(I)  - Parcel at grid I
C     NS      - Number of parcels
C     PDC(L,K)- Change in initial concentration due to a specific reaction
C     PDF(L,K)- Change in initial concentration due to dispersion
C     PH(K)   - Time parcel K entered reach in hours from day 0
C     PT(L,K) - Conc. of constituent L in parcel K
C     PTD(K)  - Conc. of constituent L in parcel K, d/s, avg'd over time step
C     PTI(L,K)- Initial concentration of constituent L in parcel K
C     PTR(L,K)- Change in initial concentration due to tributary inflow
C     PTU(L)  - Conc. of constituent L in parcel K, u/s, avg'd over time step
C     PV(K)   - Volume of parcel K
C     DVD     - Unknown volume of outflow at d/s end
C     DVU     - Unknown volume of outflow at u/s end
C     IPX(K)  - Subreach in which the u/s boundary of parcel K is located
C     VU(I)   - Volume of parcel upstream of grid I
C     DQV     - Minimum dispersive velocity (m/s)
C     LUOUT   - Fortran unit number of output file
C
C     + + + LOCAL VARIABLES + + +
      INTEGER I,I1,IEXP,IFP,II,ILP,INX,K,K2,KK,KL,KS,KSML,L,MX,N,
     #        ND(NOPR),NDMAX,NSL,KR
      REAL    COF,COF1,DF(NOCO,NOPR),DMF(NOPR),DQ(NOPR),DTT,PRDT(NOPR),
     #        PVT,RATIO,RDT,RQ,VI
C
C     + + + LOCAL VARIABLE DEFINITIONS + + +`
C     DF(L,K) - Change in conc. of constituent L in parcel K due to dispersion
C     DMF(K)  - Mass flux into parcel K at its u/s boundary
C     DQ(K)   - Flow volume between parcel K-1 and K due to velocity gradients
C     IEXP    - Exponent of 2 used to determine ND(K)
C     IFP     - Parcel at upstream boundary
C     ILP     - Parcel at d/s boundary
C     INX     - Number of subreaches
C     K       - Parcel number
C     KSML    - Parcel number of smallest parcel
C     L       - Constituent number
C     ND(K)   - No sub-time steps needed at u/s bndry of parcel K for dispersion
C     NDMAX   - Maximum number of sub-time steps needed for dispersion
C     MX      - Subreach where parcel is located
C     NOPR    - Maximum number of parcels in net dimension statements
C     PRDT(K) - Remaining time for decay in parcel K
C     RDT     - Time remaining for movement
C     X(I)    - Dist of grid I from u/s boundary (m), input as river mile
C     VI      - Smallest volume of interest (VI=QI*DT)
C
C     + + + INTRINSICS + + +
      INTRINSIC  ABS, MIN, INT, LOG10, MAX, MOD, FLOAT
C
C     + + + EXTERNALS + + +
      EXTERNAL   DECAY, DLUT
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (' IPX=',10I12)
 2010 FORMAT (' PV=',10E12.4)
 2020 FORMAT (' VU=',10F12.0)
 2030 FORMAT (' PT=',10E12.4)
C
C     + + + END SPECIFICATIONS + + +
C
C     compute hydraulics statements
C
      VI=QI*DT*3600.0
      INX=NXSEC-1
      IPX(NS+1)=NXSEC
C
C     disperse constituents statements
C
C     compute DQ's
      DO 20 K=2,NS
        MX=IPX(K)
        DQ(K)=ABS(DQQ(MX)*Q(MX))
        DQMIN=DQV*A(MX)*0.5
        IF(DQ(K).LT.DQMIN)DQ(K)=DQMIN
        IF(PV(K-1).LT.VI.OR.PV(K).LT.VI)DQ(K)=0.0
 20   CONTINUE
      DQ(1)=0.0
      DQ(NS+1)=0.0
C
C     determine number of subdivisions needed for each parcel
C
      NDMAX=0
      ND(1)=1
      ND(NS+1)=1
      DO 30 K=2,NS
        ND(K)=1
        IF(DQ(K).GE.0.00001) THEN
          IEXP=0
          RATIO=DQ(K)*DT*3600.0/MIN(PV(K-1),PV(K))
          IF(RATIO.GT.0.0)IEXP=INT(LOG10(RATIO/0.4)/LOG10(2.0)+1)
          IF(IEXP.GT.0)ND(K)=2**IEXP
        END IF
        NDMAX=MAX(ND(K),NDMAX)
 30   CONTINUE
C
C     compute new mass fluxes if needed and update all concentrations
C
      DMF(1)=0.0
      DO 120 L=1,NEQ
        DF(L,1)=0.0
        DO 110 N=1,NDMAX
          DO 100 K=1,NS
            IF (N.EQ.1) DF(L,K+1)=0.0
            IF (MOD(N-1,NDMAX/ND(K+1)).EQ.0) DMF(K+1)=DQ(K+1)*DT*3600.0*
     #        (PT(L,K)+DF(L,K)-PT(L,K+1)-DF(L,K+1))/FLOAT(NDMAX)
            IF(PV(K).GT.VI) THEN
              DF(L,K)=(DMF(K)-DMF(K+1))/PV(K)+DF(L,K)
            ELSE
              DF(L,K)=0.0
            END IF
            IF (ABS(DF(L,K)).LT.1.0E-6) DF(L,K)=0.0
 100      CONTINUE
 110    CONTINUE
 120  CONTINUE
      DO 140 L=1,NEQ
        DO 130 K=1,NS
          PT(L,K)=PT(L,K)+DF(L,K)
          PDF(L,K)=PDF(L,K)+DF(L,K)
 130    CONTINUE
 140  CONTINUE
C
C     set inflow boundary values statements
C
      IF (Q(1).LE.0.0) THEN
C       flow at upstream boundary is 0 or negative
        PRDT(1)=DT
      ELSE
C       flow in upstream boundary
        NS=NS+1
        NSL=NS-1
        DO 210 K2=1,NSL
          K=NS+1-K2
          KK=K-1
          IPX(K)=IPX(KK)
          PV(K)=PV(KK)
          PH(K)=PH(KK)
          DO 200 L=1,NEQ
            PT(L,K)=PT(L,KK)
            PTI(L,K)=PTI(L,KK)
            PDF(L,K)=PDF(L,KK)
            PTR(L,K)=PTR(L,KK)
            PDC(L,K)=PDC(L,KK)
 200      CONTINUE
 210    CONTINUE
        DO 220 I=1,NXSEC
          KAI(I)=KAI(I)+1
 220    CONTINUE
        IPX(1)=1
        PV(1)=Q(1)*DT*3600.0
        VU(1)=0.0
        PH(1)=DT*FLOAT(J+JTS)
        KAI(1)=1
        PRDT(1)=0.0
        DO 230 L=1,NEQ
          PT(L,1)=PTU(L)
          PTI(L,1)=PTU(L)
          PDF(L,1)=0.0
          PTR(L,1)=0.0
          PDC(L,1)=0.0
 230    CONTINUE
      END IF
      IF (Q(NXSEC).GE.0.0) THEN
        PRDT(NS)=DT
      ELSE
C       flow into downstream boundary
        NS=NS+1
        IPX(NS)=NXSEC-1
        PV(NS)=-Q(NXSEC)*DT*3600.0
        KAI(NXSEC)=NS
        PRDT(NS)=0.0
        PH(NS)=DT*FLOAT(J+JTS)
        VU(NXSEC)=PV(NS)
        DO 260 L=1,NEQ
          PT(L,NS)=PTD(L)
          PTI(L,NS)=PTD(L)
          PDF(L,NS)=0.0
          PTR(L,NS)=0.0
          PDC(L,NS)=0.0
 260    CONTINUE
      END IF
      NSL=NS-1
      DO 290 K=2,NSL
        PRDT(K)=DT
 290  CONTINUE
C
C     move parcels statements
C
C     move parcels passing donwstream or stationary
C
      DO 320 I=2,NXSEC
        RDT=DT
        DTT=RDT
        K=KAI(I)
        MX=I-1
        RQ=Q(I)-QT(I)
        IF(Q(I).LT.0.0.AND.RQ.LE.0.0)GO TO 320
          IF(RQ.LE.0) GO TO 310
 300        CONTINUE
              DTT=VU(I)/(RQ*3600.0)
              IF(DTT.LT.0.0)DTT=0.0
              K=KAI(I)
              MX=I-1
              IF (DTT.GE.RDT) GO TO 310
C               parcel passed grid
                RDT=RDT-DTT
                IF(Q(I).LT.0.0)DTT=DTT*RQ/(RQ-Q(I))
                IF (ABS(QT(I)).GT.QI) CALL DLUT (
     I             DTT,I,K,KAI,NEQ,NXSEC,QT,TRIB,
     M             PT,PV,PTR,VU,LUOUT)
                IF (PRDT(K).LT.DTT) DTT=PRDT(K)
                IF (DTT.GT.0.0) CALL DECAY (
     I             A(MX),DTT,J,K,LR,MX,NEQ,NNN,PV(K),Q(MX),W(MX),
     M             IRC,PDC,PT,LUOUT)
                PRDT(K)=PRDT(K)-DTT
                IPX(K)=I
                KAI(I)=K-1
                IF (K.GT.1)THEN
                  VU(I)=PV(K-1)
                ELSE
                  VU(I)=0.0
                  RDT=0.0
                  KAI(I)=1
                END IF
C               WRITE(LUOUT,*)'PARCEL',K,' PASSED WITH RDT=',RDT,
C    $                        ' VU=',VU(I)
C               WRITE(LUOUT,*)'PV=',PV(K)
            GO TO 300
C         did not pass grid
 310      CONTINUE
          IF(Q(I).LT.0.0)DTT=RDT*RQ/(RQ-Q(I))
          IF(DTT.GT.RDT)DTT=RDT
          IF (ABS(QT(I)).GT.QI) CALL DLUT (
     I       DTT,I,K,KAI,NEQ,NXSEC,QT,TRIB,
     M       PT,PV,PTR,VU,LUOUT)
          IF (PRDT(K).LT.DTT) DTT=PRDT(K)
          IF (DTT.GT.0.0) CALL DECAY (
     I       A(MX),DTT,J,K,LR,MX,NEQ,NNN,PV(K),Q(MX),W(MX),
     M       IRC,PDC,PT,LUOUT)
          PRDT(K)=PRDT(K)-DTT
          VU(I)=VU(I)-RQ*RDT*3600.0
C         WRITE(LUOUT,*)'PARCEL',K,' NO PASS WITH VU=',VU(I),
C    $                  ' PV=',PV(K)
 320  CONTINUE
C     move parcels going upstream
      DO 350 II=2,NXSEC
        I=NXSEC+1-II
        IF (Q(I).GE.0.0) GO TO 350
          RDT=DT
          MX=I
 330      CONTINUE
            K=KAI(I)
            RQ=Q(I)-QT(I)
            DTT=(VU(I)-PV(K))/(Q(I)*3600.0)
            IF(DTT.LT.0.0)DTT=0.0
C           WRITE(LUOUT,*)'MOVE US AT I=',I,' DTT=',DTT,' Q=',Q(I)
            IF (DTT.GE.RDT) GO TO 340
C             parcel passed grid
              RDT=RDT-DTT
              IF(RQ.GT.0.0)DTT=DTT*Q(I)/QT(I)
              IF (ABS(QT(I)).GT.QI) CALL DLUT (
     I           DTT,I,K,KAI,NEQ,NXSEC,QT,TRIB,
     M           PT,PV,PTR,VU,LUOUT)
              IF (PRDT(K).LT.DTT) DTT=PRDT(K)
              IF (DTT.GT.0.0) CALL DECAY (
     I           A(MX),DTT,J,K,LR,MX,NEQ,NNN,PV(K),Q(MX),W(MX),
     M           IRC,PDC,PT,LUOUT)
              PRDT(K)=PRDT(K)-DTT
              IPX(K+1)=I-1
              IF (I.EQ.1) IPX(K+1)=1
              KAI(I)=K+1
              VU(I)=0.0
C             WRITE(LUOUT,*)' PASSED U/S WITH RDT=',RDT,' DTT=',DTT,
C    $                      ' PV=',PV(K)
          GO TO 330
C         did not pass grid
 340      CONTINUE
          K=KAI(I)
          DTT=RDT
          IF(RQ.GT.0.0)DTT=RDT*RQ/(RQ-Q(I))
          IF (ABS(QT(I)).GT.QI) CALL DLUT (
     I       DTT,I,K,KAI,NEQ,NXSEC,QT,TRIB,
     M       PT,PV,PTR,VU,LUOUT)
          IF (PRDT(K).LT.DTT) DTT=PRDT(K)
          IF (DTT.GT.0.0) CALL DECAY (
     I       A(MX),DTT,J,K,LR,MX,NEQ,NNN,PV(K),Q(MX),W(MX),
     M       IRC,PDC,PT,LUOUT)
          PRDT(K)=0.0
          IF(RQ.LE.0.0) VU(I)=VU(I)-RQ*RDT*3600.0
C         WRITE(LUOUT,*)' NO PASS WITH DTT=',DTT,' VU=',VU(I),' PV=',PV(K)
 350  CONTINUE
C     complete decay step and find smallest parcel
      KSML=1
      DO 360 K=1,NS
        IF (PV(K).LE.PV(KSML)) KSML=K
        MX=IPX(K)
        IF (MX.GT.NXSEC-1) MX=NXSEC-1
        IF (MX.LT.1) MX=1
        IF (PRDT(K).GT.0.0) CALL DECAY(
     I      A(MX),PRDT(K),J,K,LR,MX,NEQ,NNN,PV(K),Q(MX),W(MX),
     M      IRC,PDC,PT,LUOUT)
 360  CONTINUE
C
C     compute outflow flux statements
C
      DO 400 L=1,NEQ
        PTU(L)=0.0
        PTD(L)=0.0
 400  CONTINUE
      IF (Q(1).GE.0.0) GO TO 430
        K=0
 410    CONTINUE
          K=K+1
          DVU=PV(K)
          IF (K.EQ.KAI(1)) DVU=VU(1)
          DO 420 L=1,NEQ
            PTU(L)=PTU(L)+PT(L,K)*DVU
 420      CONTINUE
        IF (K.LT.KAI(1)) GO TO 410
 430  CONTINUE
      IF (Q(NXSEC).LE.0.0) GO TO 460
        K=NS+1
 440    CONTINUE
          K=K-1
          DVD=PV(K)
          IF (K.EQ.KAI(NXSEC)) DVD=PV(K)-VU(NXSEC)
          DO 450 L=1,NEQ
            PTD(L)=PTD(L)+PT(L,K)*DVD
 450      CONTINUE
        IF (K.GT.KAI(NXSEC)) GO TO 440
 460  CONTINUE
      IFP=KAI(1)
      PV(IFP)=PV(IFP)-VU(1)
      I=1
 470  CONTINUE
        I=I+1
        IF (KAI(I).EQ.IFP) VU(I)=VU(I)-VU(1)
      IF (KAI(I).EQ.IFP) GO TO 470
      VU(1)=0.0
      ILP=KAI(NXSEC)
      PV(ILP)=VU(NXSEC)
C     WRITE(LUOUT,*)'IFP',IFP,'  ILP=',ILP,'  NXSEC=',NXSEC
C
C     renumber parcels and combine statements
C
      NS=ILP-IFP+1
      IF(NS.EQ.1)THEN
        IF(Q(NXSEC).GT.0.0) DVU=0.0
        IF(Q(1).LT.0.0) DVD=0.0
      ELSE     
         DVU=0.0
         DVD=0.0
      END IF
      IF (IFP.EQ.1) GO TO 530
C     RENUMBER PARCELS
      DO 510 K=1,NS
        K2=K+IFP-1
        IPX(K)=IPX(K2)
        PV(K)=PV(K2)
        PH(K)=PH(K2)
        DO 500 L=1,NEQ
          PT(L,K)=PT(L,K2)
          PTI(L,K)=PTI(L,K2)
          PDF(L,K)=PDF(L,K2)
          PTR(L,K)=PTR(L,K2)
          PDC(L,K)=PDC(L,K2)
 500    CONTINUE
 510  CONTINUE
      DO 520 I=1,NXSEC
        KAI(I)=KAI(I)-(IFP-1)
 520  CONTINUE
 530  CONTINUE
C
C     combine parcel statements
C
 600  CONTINUE
        IF (NS.GT.3.AND.PV(KSML).LT.VI) GO TO 610
          KR=1
  710     CONTINUE
            KR=KR+1
            IF(KR.GT.NS)GO TO 720
              RATIO=PV(KR)/PV(KR-1)
          IF(RATIO.LT.25.0.AND.RATIO.GT.0.04)GO TO 710
          KSML=KR
          IF(RATIO.GT.1.0)KSML=KR-1
          GO TO 610
  720     KR=NOPR-2*(NXSEC-1)
          IF (NS.LT.KR) GO TO 700
 610    CONTINUE
        IF (KSML.NE.1) GO TO 620
          KS=1
         GO TO 640
 620    CONTINUE
        IF (KSML.NE.NS) GO TO 630
          KS=NS-1
          GO TO 640
 630    CONTINUE
        KS=KSML-1
        IF (PV(KS).GT.PV(KS+2)) KS=KSML
 640    CONTINUE
        KL=KS+1
C       WRITE(LUOUT,*)' COMBINE KS KL',KS,KL
        I1=IPX(KS)+1
        DO 650 I=I1,NXSEC
          IF (IPX(KL).GE.I) GO TO 650
            KAI(I)=KAI(I)-1
            IF (KAI(I).EQ.KS) VU(I)=VU(I)+PV(KS)
 650    CONTINUE
C       WRITE(LUOUT,*)' KAI',(KAI(I),I=1,NXSEC)
        PVT=PV(KS)+PV(KL)
        IF (PVT.LE.0.0) PVT=VI
        COF=PV(KS)/PVT
        COF1=1.0-COF
        PV(KS)=PV(KS)+PV(KL)
        PH(KS)=PH(KS)*COF+COF1*PH(KL)
        DO 660 L=1,NEQ
          PT(L,KS)=PT(L,KS)*COF+COF1*PT(L,KL)
          PTI(L,KS)=PTI(L,KS)*COF+COF1*PTI(L,KL)
          PDF(L,KS)=PDF(L,KS)*COF+COF1*PDF(L,KL)
          PTR(L,KS)=PTR(L,KS)*COF+COF1*PTR(L,KL)
          PDC(L,KS)=PDC(L,KS)*COF+COF1*PDC(L,KL)
 660    CONTINUE
C       renumber remaining parcels
        NS=NS-1
        DO 680 K=KL,NS
          IPX(K)=IPX(K+1)
          PV(K)=PV(K+1)
          PH(K)=PH(K+1)
          DO 670 L=1,NEQ
            PT(L,K)=PT(L,K+1)
            PTI(L,K)=PTI(L,K+1)
            PDF(L,K)=PDF(L,K+1)
            PTR(L,K)=PTR(L,K+1)
            PDC(L,K)=PDC(L,K+1)
 670      CONTINUE
 680    CONTINUE
        KR=NOPR-2*(NXSEC-1)
        IF (NS.LT.KR) GO TO 700
          KSML=1
          DO 690 K=2,NS
            IF(PV(K).LT.PV(KSML)) KSML=K
 690     CONTINUE
      GO TO 600
 700  CONTINUE
C     WRITE(LUOUT,2000) (IPX(K),K=1,NS)
C     WRITE(LUOUT,2010) (PV(K),K=1,NS)
C     WRITE(LUOUT,2020) (VU(I),I=1,NXSEC)
C     WRITE(LUOUT,2030) (PT(1,K),K=1,NS)
C
      RETURN
      END
C
C ******************************  DECAY ********************************
C
      SUBROUTINE   DECAY
     I                  ( A,DT,J,K,LR,MX,NEQ,NNN,PV,Q,W,IRC,PDC,PT,
     I                    LUOUT)
C
C     + + + PURPOSE + + +
C     Performsthe decay step for each parcel.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOPR   - Maximum number of parcels allowed in branch
C              (NOPR should be at least 20 + 2 times NOSC)
C     NOCO   - Maximum number of constituents allowed
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     IRC,J,K,LR(NOCO),LUOUT,MX,NEQ,NNN
      REAL        A,DT,PDC(NOCO,NOPR),PT(NOCO,NOPR),PV,Q,W
C
C     + + + ARGUMENT DEFINITIONS + + +
C     A      - Average area in subreach I (sq m)
C     DT     - Time step size in hours
C     J      - Time step
C     K      - Parcel number
C     LR(L)  - Index denoting that the decay of constituent L due to the
C              presence of constituent LR(L) is tracked
C     MX     - Subreach where parcel is located
C     NEQ    - Number of equations (constituents)
C     NNN    - Branch number
C     PV     - Volume of parcel K
C     Q      - River flow at grid I (cu m/s),
C              trib inflow occurs just u/s of grid
C     W      - Average top width in subreach I (m)
C     IRC    - Code for reading data in FINK (1=read data, 0=no read)
C     PDC(L,K)- Change in initial concentration due to a specific reaction
C     PT(L,K)- Concentration of constituent L in parcel K
C
C     + + + LOCAL VARIABLES + + +
      INTEGER ISUM,L,LL,IRDT
      REAL    COM,CR(NOCO,NOCO),DC(NOCO),DC2(NOCO),DTL,DTM,PT1(NOCO),
     #        RDT,REF,S(NOCO),SUM(NOCO),SUM2(NOCO),XK(NOCO,NOCO)
C
C     + + + LOCAL DEFINTIONS + + +
C     IRDT   - Code for time remaining, if IRDT=1 need another itteration
C     XK(L,M)- Production rate of constituent L due to constituent M
C     CR(L,M)- Equilibrium concentration of constituent L due
C              to constituent M.
C     S(L)   - Source term for constituent L (per hour).
C
C
C     + + + INTRINSICS + + +
      INTRINSIC  ABS, IFIX
C
C     + + + EXTERNALS + + +
      EXTERNAL   FINK
C
C     + + + END SPECIFICATIONS + + +
C
      IRDT = 1
      DO 120 L = 1, NOCO
        S(L) = 0.0
        DO 110 LL = 1, NOCO
          XK(L,LL) = 0.0
          CR(L,LL) = 0.0
  110   CONTINUE
  120 CONTINUE
      RDT = DT
C
  200 CONTINUE
        DTM=RDT
        CALL FINK (K,MX,J,NEQ,PV,PT,Q,A,W,XK,S,CR,IRC,NNN,LUOUT)
C
C       find maximum decay time step
        DO 220 L = 1, NEQ
          SUM(L) = S(L)
          DO 210 LL = 1, NEQ
            SUM(L) = SUM(L) + XK(L,LL) * (PT(LL,K) - CR(L,LL))
  210     CONTINUE
          LL    = LR(L)
          DC(L) = XK(L,LL) * (PT(LL,K) - CR(L,LL))
          DTL   = RDT
          REF   = PT(L,K) - CR(L,L)
          ISUM  = IFIX ( ABS ( SUM(L) ) )
          IF (ISUM .EQ. 0) THEN
            ISUM= IFIX ( ABS ( SUM(L) * 1.0E06 ) )
          END IF
          IF (ISUM .NE. 0.  AND.  ABS ( REF ) .GT. 0.3) THEN
            DTL = ABS ( 0.1 * REF / SUM(L) )
          END IF
          IF (DTL .LT. DTM) DTM = DTL
  220   CONTINUE
        COM = 0.999 * RDT
        IF (DTM .GT. COM) THEN
          DTM  = RDT
          IRDT = 0
        END IF
C
C       update temperature
        DO 230 L = 1, NEQ
          PT1(L)  = PT(L,K)
          PT(L,K) = PT(L,K) + SUM(L) * DTM
  230   CONTINUE
        CALL FINK (K,MX,J,NEQ,PV,PT,Q,A,W,XK,S,CR,IRC,NNN,LUOUT)
        DO 250 L = 1, NEQ
          SUM2(L) = S(L)
          DO 240 LL = 1, NEQ
            SUM2(L) = SUM2(L) + XK(L,LL) * (PT(LL,K) - CR(L,LL))
  240     CONTINUE
          LL     = LR(L)
          DC2(L) = XK(L,LL) * (PT(LL,K) - CR(L,LL))
  250   CONTINUE
        DO 260 L = 1, NEQ
          PDC(L,K) = PDC(L,K) + (DC(L) + DC2(L)) * 0.5 * DTM
          PT(L,K)  = PT1(L)+ 0.5* (SUM(L) + SUM2(L)) * DTM
  260   CONTINUE
        RDT = RDT - DTM
      IF (IRDT .EQ. 1) GO TO 200
C
      RETURN
      END
C
C ******************************  DLUT ************************************
C
      SUBROUTINE   DLUT
     I                 ( DT,I,K,KAI,NEQ,NXSEC,QT,TRIB,
     M                   PT,PV,PTR,VU,LUOUT)
C
C     + + + PURPOSE + + +
C     Mix the tributary inflow with the parcel.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOSC   - Maximum number of cross sections (grids) allowed in branch
C     NOPR   - Maximum number of parcels allowed in branch
C              (NOPR should be at least 20 + 2 times NOSC)
C     NOCO   - Maximum number of constituents allowed
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER I,K,KAI(NOSC),LUOUT,NEQ,NXSEC
      REAL    DT,PT(NOCO,NOPR),PTR(NOCO,NOPR),PV(NOPR),QT(NOSC),
     #        TRIB(NOCO,NOSC),VU(NOSC)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DT     - Time step size in hours
C     I      - Grid number
C     K      - Parcel number
C     KAI(I) - Parcel at grid I
C     NEQ    - Number of equations (constituents)
C     NXSEC  - Number of grids in branch N
C     QT(I)  - Tributary inflow at grid I (cu m/s),
C              enter just upstream of grid
C     TRIB(L,I)- Concentration of constituent L in trib at grid I
C              (tribs can not occur at first or last grid)
C     PT(L,K)- Concentration of constituent L in parcel K
C     PV(K)  - Volume of parcel K
C     PTR(L,K)- Change in initial concentration due to tributary inflow
C     VU(I)  - Volume of parcel upstream of grid I
C     LUOUT  - Not used (Fortran unit number of output file)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I1,L
      REAL        DEL,DELV
C
C     + + + END SPECIFICATIONS + + +
C
      DELV = QT(I) * 3600.0 * DT
      DEL  = DELV + PV(K)
      IF (DEL .LE. 0.0) THEN
        PV(K)=0.0
        DO 110 L = 1, NEQ
          PT(L,K)  = 0.0
          PTR(L,K) = 0.0
  110   CONTINUE
      ELSE
        DO 120 L = 1, NEQ
          IF (DELV .GT. 0.0) THEN
            DEL=(TRIB(L,I)*DELV+PT(L,K)*PV(K))/(PV(K)+DELV)-PT(L,K)
          ELSE
            DEL = 0.0
          END IF
          PT(L,K)  = PT(L,K) + DEL
          PTR(L,K) = PTR(L,K) + DEL
  120   CONTINUE
        PV(K) = PV(K) + DELV
        IF (PV(K) .LT. 0.0) PV(K) = 0.0
      END IF
C
C     update VU's
      I1 = I + 1
      DO 210 I1 = I1, NXSEC
        IF (KAI(I1) .EQ. K) VU(I1) = VU(I1) + DELV
        IF (VU(I1) .LT. 0.0) VU(I1) = 0.0
  210 CONTINUE
C
      RETURN
      END
