C
C
C
      PROGRAM   BQUAL2E
C
C     + + + PURPOSE + + +
C     Build the input file for QUAL2E
C
C     + + + LOCAL VARIABLES + + +
      INTEGER I,IGRO,INX,ITRM,J,K,K2O,LFO,LUBC,LUOUT,N,NBRCH
      INTEGER NHR,NXSEC(99)
      REAL    AA,ALGSET,ALPH0,ALPH1,ALPH2,ALPH3,ALPH4,ALPH5,ALPH6,A1,B1,
     #        BET1,BET2,BET3,BET4,CB,CK1,CK2,CK3,CK4,CKL,CKN,
     #        CKP,DSONET,DV,GROMAX,PN,RSPRT,SHAD0,SHAD1,SHAD2,SIG2,
     #        SIG3,SIG4,SIG5,SONET,TE,TIM,V
      CHARACTER*64 VERSN
      CHARACTER*80 BLK

C     + + + LOCAL DEFINITIONS + + +
C     AA     -  Local variable
C     ALGSET -  Local settling rate for algae, ft/day
C     ALPH0  -  Conversion factor, ug Chlorophyll-A/mg Algae
C     ALPH1  -  Fraction of algae biomass that is nitrogen, mg-N/mg-A
C     ALPH2  -  Fraction of algae biomass that is phosphorus, mg-P/mg-A
C     ALPH3  -  Oxygen production per unit of algae growth, mg-O/mg-A
C     ALPH4  -  Oxygen uptake per unit of algae expired, mg-O/mg-A
C     ALPH5  -  Oxygen uptake per unit of NH3 oxidation, mg-O/mg-N
C     ALPH6  -  Oxygen uptake per unit of NO2 oxidation, mg-O/mg-N
C     A1     -  Constant in wind function [mm/day kpa] suggest 3.01
C     B1     -  Mass transfer coeff [mm/d/kpa(m/s)] suggest 1.13
C     BLK    -  Alpha numeric character (80 characters)
C     BET1   -  Rate constant for biological oxidation of NH3-NO2, per day
C     BET2   -  Rate constant for biological oxidation of NO2-NO3, per day
C     BET3   -  Rate constant for hydrolysis ON-NH3, per day
C     BET4   -  Organic to disslved phorphorus decay rate, per day
C     CB     -  Exponent on flow for Option 7 computation of K2
C     CKL    -  Light constant, cal/(cm^2 min)
C                 for LFO =1, half saturation coefficient
C                 for LFO =2, intensity at 71% of the max growth rate
C                 for LFO =3, light intensity at max growth rate
C     CKN    -  Nitrogen half-saturation constant for algae, mg/L
C     CKP    -  Phosphorus half-saturation const for algae, mg/L
C     CK1    -  Carbonaceous BOD decay rate, per day
C     CK2    -  Reaeration rate, per day for option 1,Manning's for option
C                  5 or 8, coefficient on Q for option 7.
C     CK3    -  Carbonaceous BOD sink rate, 1/day
C     CK4    -  Benthos consumption of oxygen, mg-O/ft^2 day
C     DSONET -  Default Light intensity 
C     DV     -  Default wind speed (miles per hour)
C     GROMAX -  Maximum specific growth rate of algae, per day
C     IGRO   -  Growth rate option (1=multiplicative, 2=limiting nutrient,
C                                   3=Harmonic mean)
C     ITRM   -  Code for input ITRM=1 from file BC, ITRM=0 from terminal
C     J      -  Time step
C     K2O    -  Option for K2 (reaeration coefficient)
C                   1=read as input, 2=Churchill, 3=O'Connor, 4=Owens
C                   5=Thackston, 6=Langbien, 7= power fnt, 8=Tsivoglou
C     LFO    -  Light function option (1=half saturation, 
C                             2=Smith's function, 3=Steel's equation)
C     NBRCH  -  Number of branches N
C     NHR    -  Number of time steps to be modeled
C     NXSEC(N)- Number of grids in branch
C     PN     -  Preference factor for ammonia nitrogen (0-1.0)
C     RSPRT  -  Algae respiration rate, per day
C     SHAD0  -  Non-algal portion of light extintion coefficient, 1/ft
C     SHAD1  -  Linear algal self shading coefficient
C     SHAD2  -  Nonlinear algal self shading coefficient
C     SIG2   -  Benthos source rate for dissolved phosphorus, mg-P/ft^2 day
C     SIG3   -  Benthos source rate for NH3, mg-N/ft^2 day
C     SIG4   -  Organic nitrogen settling rate, per day
C     SIG5   -  Organic phorphorus settling rate, per day
C     SONET  -  Light intensity [langley/min = cal/(cm^2 min)]
C     TE     -  Equilibrium temperatue (Celsius)
C     TIM    -  Time or time step number
C     V      -  Wind speed (mph in, m/s out)
C
C     + + + INTRINSICS + + +
      INTRINSIC   FLOAT
C
C     + + + INPUT FORMATS
 1000 FORMAT(A)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT('  Enter title (no more than 80 characters)',/)
 2001 FORMAT('Wind function =  A1  +  B1 V [mm/(day kpa)],',
     #       ' Wind speed V in m/s')
 2002 FORMAT(15X,F5.2,F7.2)
 2003 FORMAT('ALPH0   ALPH1   ALPH2   ALPH3   ALPH4  ALPH5  ALPH6 ',
     #       'GROMAX IGRO RSPRT ')
 2004 FORMAT(F5.1,2F8.3,2F8.2,3F7.2,I4,F7.2)
 2005 FORMAT(' LFO  CKL    CKN     CKP    SHAD0   SHAD1  SHAD2    PN',
     #       '    K2O')
 2006 FORMAT(I3,2F7.3,F8.3,F9.3,F7.3,F8.3,F7.3,I4)
 2007 FORMAT('  Br  Gr  ALGSET  BET3    SIG4     BET1      SIG3       ',
     #       'BET2      BET4')
 2008 FORMAT(2I4,F7.2,F9.3,F7.3,F9.3,F11.3,F9.2,F11.3)
 2009 FORMAT('  Br  Gr    SIG5     SIG2     CK1      CK3      CK4  CK2')
 2010 FORMAT('  Br  Gr    SIG5     SIG2     CK1      CK3      CK4')
 2011 FORMAT('  Br  Gr    SIG5     SIG2     CK1      CK3      CK4  Man')
 2012 FORMAT('  Br  Gr    SIG5     SIG2     CK1      CK3      CK4  ',
     #       'cof a   cof b')
 2013 FORMAT(2I4,F8.3,F9.3,F8.3,2F9.3,F8.3,F8.2)
 2014 FORMAT(F6.1,2F9.2,F9.3)
 2015 FORMAT(1X,A)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)BQUAL2E - last modified MAY  3, 1996, kmf'
C
C     set unit numbers and open required data files
      LUBC=17
      LUOUT=16
      WRITE(*,*)' Meteorological data read either from terminal or "bc"'
      WRITE(*,*)' Output written to "qual2e.in"'
      WRITE(*,*)' Enter any letter to quit, 1 to continue '
      READ(*,'(I9.0)',ERR=99)I
        OPEN (LUOUT,FILE='qual2e.in')
C
        WRITE(*,2000)
        READ (*,1000)BLK
        WRITE(LUOUT,2015)BLK
        A1=3.01
        WRITE(*,*) ' Enter any letter to get the default value'
        WRITE(*,*) ' Enter constant in wind function, '
        WRITE(*,*) ' Default=',A1,' (mm/(day Kpa)) A1 = '
        READ(*,'(F9.0)',ERR=1)AA
          A1=AA
    1   CONTINUE
        B1=1.13
        WRITE(*,*) ' Enter mass transfer coefficient,'
        WRITE(*,*) ' Default=',B1,' (mm/(day Kpa(m/s))), B1 = '
        READ(*,'(F9.0)',ERR=2)AA
          B1=AA
    2   CONTINUE
        WRITE(LUOUT,2001)
        WRITE(LUOUT,2002)A1,B1
        WRITE(LUOUT,2003)
        ALPH0=50.0
        WRITE(*,*) ' Enter ratio of chlorophyl-a to algal biomass.'
        WRITE(*,*) ' Range; 10-100 ug-chla/mg-A, Default =',
     #               ALPH0,', ALPH0='
        READ(*,'(F9.0)',ERR=3)AA
          ALPH0=AA
    3   CONTINUE
        ALPH1=0.08
        WRITE(*,*) ' Enter fraction of algae biomass that is nitrogen.'
        WRITE(*,*) ' Range; 0.07-0.09 mg-N/mg-A, Default =',
     #               ALPH1,', ALPH1='
        READ(*,'(F9.0)',ERR=4)AA
          ALPH1=AA
    4   CONTINUE
        ALPH2=0.015
        WRITE(*,*) ' Enter fraction of algae biomass that is',
     #             ' phosphorus.'
        WRITE(*,*) ' Range; 0.01-0.02 mg-N/mg-A, Default =',
     #              ALPH2,', ALPH2='
        READ(*,'(F9.0)',ERR=5)AA
          ALPH2=AA
    5   CONTINUE
         ALPH3=1.6
         WRITE(*,*) ' Enter DO production per unit of algal growth.'
         WRITE(*,*) ' Range; 1.4-1.8 mg-O/mg-A, Default=',
     #              ALPH3,', ALPH3 = '
         READ(*,'(F9.0)',ERR=6)AA
           ALPH3=AA
    6    CONTINUE
         ALPH4=2.0
         WRITE(*,*) ' Enter DO uptake per unit of algae respired.'
         WRITE(*,*) ' Range; 1.6-2.3 mg-O/mg-A, Default=',
     #              ALPH4,', ALPH4 = '
         READ(*,'(F9.0)',ERR=7)AA
           ALPH4=AA
    7    CONTINUE
         ALPH5=3.5
         WRITE(*,*) ' Enter DO uptake per unit of NH3 oxidation.'
         WRITE(*,*) ' Range; 3.0-4.0 mg-O/mg-N, Default=',
     #              ALPH5,', ALPH5 = '
         READ(*,'(F9.0)',ERR=8)AA
           ALPH5=AA
    8    CONTINUE
         ALPH6=1.12
         WRITE(*,*) ' Enter DO uptake per unit of NO2 oxidation.'
         WRITE(*,*) ' Range; 1.0-1.14 mg-O/mg-N, Default=',
     #              ALPH6,', ALPH6= '
         READ(*,'(F9.0)',ERR=9)AA
           ALPH6=AA
    9    CONTINUE
         GROMAX=2.0
         WRITE(*,*) ' Enter maximum algal growth rate, 1/day'
         WRITE(*,*) ' Range; 1.0-3.0 1/day, Default=',
     #              GROMAX,', GROMAX = '
         READ(*,'(F9.0)',ERR=10)AA
           GROMAX=AA
   10    CONTINUE
         IGRO=2
         WRITE(*,*) ' Enter growth rate option (1=multiplicative,'
         WRITE(*,*) '        2=limiting nutrient, 3=Harmonic mean - '
         READ(*,'(I9)',ERR=11)I
           IGRO=I
   11    CONTINUE
         RSPRT=0.2
         WRITE(*,*) ' Enter algae respiration rate'
         WRITE(*,*) ' Range; 0.05-0.5, Default =',RSPRT,'/day, RSPRT=  '
         READ(*,'(F9.0)',ERR=12)AA
           RSPRT=AA
   12    CONTINUE
         WRITE(LUOUT,2004)ALPH0,ALPH1,ALPH2,ALPH3,ALPH4,ALPH5,ALPH6,
     #                    GROMAX,IGRO,RSPRT
         LFO=1
         WRITE(*,*) ' Enter light function option; 1= half saturation, '
         WRITE(*,*) '        2=Smith"s function, 3=Steel"s equation'
         WRITE(*,*) ' Default =',LFO,', LFO = '
         READ(*,'(I9)',ERR=13)I
           LFO=I
   13   CONTINUE
        IF (LFO .EQ. 1) THEN
          CKL=5.0
          WRITE(*,*) ' Enter half saturation coefficient, cal/cm^2 min'
          WRITE(*,*) ' Range; 1.2-6.0, Default =',CKL,
     #               ' cal/cm2 min, CKL = '
          READ(*,'(F9.0)',ERR=14)AA
            CKL=AA
   14     CONTINUE
        ELSE
          IF (LFO .EQ. 2) THEN
            CKL=8.6
            WRITE(*,*) ' Enter light intensity for 71% max growth rate'
            WRITE(*,*) ' Range; 2-10, Default =',CKL,
     #                 ' cal/cm^2 min, CKL = '
            READ(*,'(F9.0)',ERR=15)AA
              CKL=AA
   15       CONTINUE
          ELSE
            CKL=22.0
            WRITE(*,*) ' Enter light intensity for 50% max growth rate'
            WRITE(*,*) ' Range; 5-26, Default =',CKL,
     #                 ' cal/cm^2 min, CKL = '
            READ(*,'(F9.0)',ERR=16)AA
              CKL=AA
   16       CONTINUE
          END IF
        END IF
        CKN=0.3
        WRITE(*,*) ' Enter nitrogen half-saturation constant for algae'
        WRITE(*,*) ' Range; 0.01-0.30, Default =',CKN,' mg-N/L, CKN = '
        READ(*,'(F9.0)',ERR=17)AA
          CKN=AA
   17   CONTINUE
        CKP=0.04
        WRITE(*,*) ' Enter phosphorus half-saturation constant',
     #             ' for algae'
        WRITE(*,*) ' Range; 0.001-0.05, Default =',CKP,' mg-P/L, CKP = '
        READ(*,'(F9.0)',ERR=18)AA
          CKP=AA
   18   CONTINUE
        SHAD0=0.1
        WRITE(*,*) ' Enter non-algal light extinction coef, 1/ft'
        WRITE(*,*) ' Range= variable, Default =',SHAD0,', SHAD0 = '
        READ(*,'(F9.0)',ERR=19)AA
          SHAD0=AA
   19   CONTINUE
        SHAD1=0.00268
        WRITE(*,*) ' Enter linear algal self shading coefficient'
        WRITE(*,*) ' Default =',SHAD1,' 1/(ft ug-ChlA/L), SHAD1 = '
        READ(*,'(F9.0)',ERR=20)AA
          SHAD1=AA
   20   CONTINUE
        SHAD2=0.0
        WRITE(*,*) ' Enter non-linear algal self shading coef'
        WRITE(*,*) ' Default =',SHAD2,' 1/(ft (ug-ChlA/L)^.67),',
     #             ' SHAD2 = '
        READ(*,'(F9.0)',ERR=21)AA
          SHAD2=AA
   21   CONTINUE
        PN=0.5
        WRITE(*,*) ' Enter Algal preference factor for ammonia'
        WRITE(*,*) ' Range 0-1.0, Default =',PN,', PN = '
        READ(*,'(F9.0)',ERR=22)AA
          PN=AA
   22   CONTINUE
        K2O=1
        WRITE(*,*) ' Enter option for reaeration input, 1=read as input'
        WRITE(*,*) ' 2=Churchill, 3=O"Connor, 4=Owens, 5=Thackston'
        WRITE(*,*) ' 6=Langbien, 7=power function, 8=Tsivoglou'
        WRITE(*,*) ' Default =',K2O,', K2O = '
        READ(*,'(I9)',ERR=23)I
          K2O=I
   23   CONTINUE
        WRITE(LUOUT,2005)
        WRITE(LUOUT,2006)LFO,CKL,CKN,CKP,SHAD0,SHAD1,SHAD2,PN,K2O
        WRITE(*,*) ' Enter number of branches = '
        READ(*,*)NBRCH
        WRITE(LUOUT,2007)
        DO 60 N=1,NBRCH
          WRITE(*,*) ' Enter number of grids in branch',N,' NXSEC = '
          READ(*,*)NXSEC(N)
          INX=NXSEC(N)-1
          DO 60 I=1,INX
            WRITE(*,*) ' For branch',N,', grid',I,
     $                 ' enter the following:'
            ALGSET=1.5
            WRITE(*,*) ' Enter the local settling rate for algae,',
     $                 ' ft/day'
            WRITE(*,*) ' Range 0.5-6, Default=',ALGSET,
     #                 ' ft/day, ALGSET=  '
            READ(*,'(F9.0)',ERR=50)AA
              ALGSET=AA
   50       CONTINUE
            BET3=0.2
            WRITE(*,*) ' Enter rate constant for hydrolysis of',
     $                 ' ON to NH3'
            WRITE(*,*) ' Range 0.02-0.4, Default =',BET3,
     #                 ' 1/day, BET3 = '
            READ(*,'(F9.0)',ERR=51)AA
              BET3=AA
   51       CONTINUE
            SIG4=.01
            WRITE(*,*) ' Enter Organic nitrogen settling rate, per day'
            WRITE(*,*) ' Range 0.001-0.1, Default =',SIG4,
     $                 ' 1/day, SIG4 = '
            READ(*,'(F9.0)',ERR=52)AA
              SIG4=AA
   52       CONTINUE
            BET1=0.3
            WRITE(*,*) ' Enter rate constant; bio oxidation of',
     #                 ' NH3 to NO2'
            WRITE(*,*) ' Range 0.1-1, Default=',BET1,' 1/day, BET1=  '
            READ(*,'(F9.0)',ERR=53)AA
              BET1=AA
   53       CONTINUE
            SIG3=3.0
            WRITE(*,*) ' Enter benthos source rate for NH3,',
     $                 ' gm-N/ft2 day'
            WRITE(*,*) ' Range ??, Default=',SIG3,
     $                 ' mg-N/ft2 day, SIG3=  '
            READ(*,'(F9.0)',ERR=54)AA
              SIG3=AA
   54       CONTINUE
            BET2=1.5
            WRITE(*,*) ' Enter rate cnst for oxidation of NO2-NO3,',
     $                 ' 1/day'
            WRITE(*,*) ' Range 0.2-2, Default =',BET2,', BET2 = '
            READ(*,'(F9.0)',ERR=55)AA
              BET2=AA
   55       CONTINUE
            BET4=0.4
            WRITE(*,*) ' Enter organic-disslved phorphorus',
     #                 ' decay rte, 1/day'
            WRITE(*,*) ' Range 0.01-0.7, Default =',BET4,', BET4 = '
            READ(*,'(F9.0)',ERR=56)AA
              BET4=AA
   56       CONTINUE
            WRITE(LUOUT,2008)N,I,ALGSET,BET3,SIG4,BET1,SIG3,BET2,BET4
   60   CONTINUE
        IF (K2O .LT. 5) THEN
          IF (K2O .EQ. 1) THEN
            WRITE(LUOUT,2009)
          ELSE
            WRITE(LUOUT,2010)
          END IF
        ELSE
          IF (K2O .EQ. 5  .OR.  K2O .EQ. 8) WRITE(LUOUT,2011)
          IF (K2O .EQ. 6) WRITE(LUOUT,2010)
          IF (K2O .EQ. 7 )WRITE(LUOUT,2012)
        END IF
        DO 70 N=1,NBRCH
          INX=NXSEC(N)-1
          DO 70 I=1,INX
            WRITE(*,*) ' For branch',N,', grid',I,
     $                 ' enter the following:'
            SIG5=0.2
            WRITE(*,*) ' Enter organic phorphorus settling rate, 1/day'
            WRITE(*,*) ' Range 0.001-0.1, Default =',SIG5,', SIG5 = '
            READ(*,'(F9.0)',ERR=61)AA
              SIG5=AA
   61       CONTINUE
            SIG2=1.0
            WRITE(*,*) ' Enter benthos source rate for',
     $                 ' dissolved phosphorus'
            WRITE(*,*) ' Range ??, Default =',SIG2,
     $                 ', mg-P/ft2 day, SIG2 = '
            READ(*,'(F9.0)',ERR=62)AA
              SIG2=AA
   62       CONTINUE
            CK1=0.34
            WRITE(*,*) ' Enter carbonaceous BOD decay rate, 1/day'
            WRITE(*,*) ' Range 0.02-3.4, Default =',CK1,', CK1 = '
            READ(*,'(F9.0)',ERR=63)AA
              CK1=AA
   63       CONTINUE
            CK3=0.24
            WRITE(*,*) ' Enter carbonaceous BOD sink rate, 1/day'
            WRITE(*,*) ' Range -0.36 to +0.36, Default =',CK3,', CK3 = '
            READ(*,'(F9.0)',ERR=64)AA
              CK3=AA
   64       CONTINUE
            CK4=20.0
            WRITE(*,*) ' Enter benthos consumption of oxygen,',
     $                 ' mg-O/ft^2 day'
            WRITE(*,*) ' Range ??, Default =',CK4,', CK4 = '
            READ(*,'(F9.0)',ERR=65)AA
              CK4=AA
   65       CONTINUE
            CK2=0.0
            CB=0.0
            IF (K2O .LT. 5) THEN
              IF (K2O .EQ. 1) THEN
                CK2=2.20
                WRITE(*,*)'Enter reaeration rate constant, 1/day'
                WRITE(*,*)'Range 0-100, Default =',CK2,', CK2 = '
                READ(*,'(F9.0)',ERR=66)AA
                  CK2=AA
   66           CONTINUE
              END IF
            ELSE
              IF (K2O .EQ. 5  .OR.  K2O .EQ .8) THEN
                CK2=0.03
                WRITE(*,*) ' Enter Mannings n, Default =',CK2,', CK2 = '
                READ(*,'(F9.0)',ERR=67)AA
                  CK2=AA
   67           CONTINUE
              ELSE IF (K2O .EQ. 7) THEN
                WRITE(*,*) ' Enter coefficients in K2 = a Q^b, Q in cfs'
                WRITE(*,*) ' Enter coeff a, CK2 = '
                READ(*,*)CK2
                WRITE(*,*) ' Enter coeff b, CB = '
                READ(*,*)CB
              END IF
            END IF
            WRITE(LUOUT,2013)N,I,SIG5,SIG2,CK1,CK3,CK4,CK2,CB
   70   CONTINUE
        WRITE(LUOUT,*) ' Time      TE C    V, m/s   Solar cal/cm^2 min'
        WRITE(*,*) ' Enter number of time steps to be modeled  '
        READ(*,*)NHR
        WRITE(*,*) ' Enter 0 to input meteorologic data from terminal'
        WRITE(*,*) ' Enter 1 to input meteorologic data for file "bc"  '
        READ(*,*)ITRM
        TE=20.0
        DV=10.0
        DSONET=0.0
        WRITE(*,*) ' Wind speed is entered in mph and converted to m/s'
        WRITE(*,*) ' Radiation is entered in Cal/sq cm min,',
     $             ' (Langley/min)'
        IF (ITRM .NE. 0) THEN
          OPEN(LUBC,FILE='bc')
          READ(LUBC,1000)BLK
          READ(LUBC,1000)BLK
        END IF
        DO 90 J=1,NHR
          IF (ITRM .EQ. 1) THEN
           READ(LUBC,*)TIM,TE,V,SONET
          ELSE
            WRITE(*,*) ' For time step',J,' enter equilibrium',
     $                 ' temperature'
            WRITE(*,*) ' Default=',TE,' deg C, TE = '
            READ(*,'(F9.0)',ERR=81)AA
              TE=AA
   81       CONTINUE
            WRITE(*,*) ' For time step',J,' enter wind speed'
            WRITE(*,*) ' Default=',DV,' mph, V =  '
            READ(*,'(F9.0)',ERR=82)AA
              DV=AA
   82       CONTINUE
            V=DV*0.447
            WRITE(*,*) ' For time step',J,' enter light intensity'
            WRITE(*,*) ' Default=',DSONET,' langley/min, SONET=  '
            READ(*,'(F9.0)',ERR=83)AA
              DSONET=AA
   83       CONTINUE
            SONET=DSONET
          END IF
          TIM=FLOAT(J)
          WRITE(LUOUT,2014)TIM,TE,V,SONET
   90   CONTINUE
C
C       finished, close files
        ENDFILE ( LUOUT )
        CLOSE   ( LUOUT )
        IF (ITRM .EQ. 1) CLOSE(LUBC)
   99 CONTINUE
C
      STOP
      END
