C
C
C
      PROGRAM   CTPLT
C
C     + + + PURPOSE + + +
C     Plots concentration vs time and computes the RMS error.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOSC   - Maximum number of grids allowed in a branch
C     NOPR   - Maximum number of parcels allowed in branch
C     NOCP   - Maximum number of computed or observed points
C
C     + + + LOCAL VARIABLES + + +
      INTEGER I,IC,J,JTS,K,L,LP,M,N,NBRCH,NBRCHP,NN,NR,NS,NXSEC,
     #        LUIN, LUPC,LUOUT,LUDT,NEQ, NP
      REAL    AA,CMAX,CMIN,DT,DX,DY,ER,PT(NOPR),PX(NOPR),RM(NOSC),RMP,
     #        SR,SRR,X(NOCP),XMAX,XMIN,XR(NOCP),YC,Y(NOCP),YR(NOCP)
      CHARACTER*4  LABEL
      CHARACTER*20 LEGEND
      CHARACTER*40 MLAB, XLAB, YLAB
      CHARACTER*64 VERSN
      CHARACTER*80 TITLE
C
C     + + + LOCAL DEFINITIONS + + +
C     CMAX   - Maximum computed concentration or max of plot
C     CMIN   - Minimum concentration
C     DT     - Time step size in hours
C     DX     - Increment on tick marks for x axis
C     DY     - Increment on tick marks for y axis
C     ER     - Error in computed value
C     IC     - Code for plotting observed data (2 to plot observed)
C     JTS    - Number of time steps from midnight to start of model
C     K      - Parcel number
C     L      - Constituent number
C     LABEL  - Name of constituent L
C     LP     - Constituent number to be plotted
C     NBRCH  - Number of branches in model
C     NBRCHP - Branch where plot is disired
C     NEQ    - Number of equations in model
C     NP     - Number of points to be plotted
C     NR     - Number of observed concentrations
C     NS     - Number of parcels in branch N
C     NXSEC  - Number of grids in branch N
C     PT(K)  - Concentration of L in parcel K of branch N
C     PX(K)  - Grid unit location of u/s boundry of parcel K in branch N
C     RM(I)  - Miles downstream of grid I in branch N from reference point
C     RMP    - Miles downstream where plot is desired
C     SR     - Sum of errors
C     SRR    - Sum of squares of errors
C     TITLE  - Title of run
C     X(NP)  - X plotting position for point NP
C     XMAX   - Maximum time plotting position
C     XMIN   - Minimum time plotting position
C     XR(NR) - X plotting position for the observer conc point NR
C     YC     - Computed value of concentration
C     Y(NP)  - Y plotting position for point NP
C     YR(NR) - Y plotting position for the observer conc point NR
C
C     + + + INTRINSICS + + +
      INTRINSIC  IFIX, FLOAT, SQRT
C
C     + + + EXTERNALS + + +
      EXTERNAL   DRWCRV, DRWDON, DRWLIM, DRWSET
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT ( A )
 1010 FORMAT ( 10X,10I7 )
 1020 FORMAT ( 10X,10F7.3 )
 1030 FORMAT ( 10X,I7,A7,I7 )
 1040 FORMAT ( 10X,F7.3,I7 )
 1050 FORMAT ( 3I5 )
 1060 FORMAT ( 8G10.3 )
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT ( 1X,A80 )
 2010 FORMAT ( 3G12.4 )
 2020 FORMAT ( 'Concentration of  ', A4, '  in branch', I3,
     #        'at river mile', F7.3 )
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)CTPLT - modified May  5, 1994, kmf'
      VERSN = '@(#)CTPLT - modified Apr 30, 1996, kmf'
      VERSN = '@(#)CTPLT - modified Jul 24, 1996, hej'
      VERSN = '@(#)CTPLT - modified Sep 09, 1996, hej'
C
C     set unit numbers and open required data files
      LUIN  = 35
      LUPC  = 36
      LUOUT = 37
      LUDT  = 38
      OPEN ( LUIN,  FILE = 'bltm.in' )
      OPEN ( LUPC,  FILE = 'parcel.out' )
      OPEN ( LUOUT, FILE = 'plt.out' )
C
C     notice to user
      WRITE(*,*) ' This program reads parcel.out, bltm.in, and obs.'
      WRITE(*,*) ' It writes the file plt.out'
      WRITE(*,*) ' This program is dimensioned for:'
      WRITE(*,*) '      ', NOPR, ' Parcels per branch'
      WRITE(*,*) '      ', NOSC, ' cross sections per branch'
      WRITE(*,*) '      ', NOCP, ' computed or obs plotting points'
C
C     read data from bltm.in file
      READ (LUIN,1000) TITLE
      READ (LUIN,1010) NBRCH,I,L,NEQ,JTS
      READ (LUIN,1020) DT
      WRITE(*,*) ' Constituent number to be plotted = '
      READ (*,*) LP
      DO 110 L = 1, NEQ
        IF(L .EQ. LP) THEN
C         this is the constituent
          READ(LUIN,1030) M, LABEL
        ELSE
          READ(LUIN,1030) M
        END IF
  110 CONTINUE
      WRITE(*,*) ' Branch number where plot is desired = '
      READ (*,*) NBRCHP
      DO 130 N = 1, NBRCH
        READ (LUIN,1010) NXSEC
        DO 120 I = 1, NXSEC
          IF (NBRCHP .EQ. N) THEN
            READ (LUIN,1040) RM(I)
          ELSE
            READ(LUIN,1040) XR(I)
          END IF
          IF (NEQ.GT.7 .AND. I.LT.NXSEC) READ(LUIN,1000) TITLE(77:80)
  120   CONTINUE
  130 CONTINUE
C
C     read plot data
      WRITE(*,*) ' River mile of plot = '
      READ (*,*) RMP
      WRITE(LUOUT,2020) LABEL, NBRCHP, RMP
C
C     read parcel.out file and build plt.out file
      WRITE(LUOUT,*) '   TIME    COMPUTED'
      NP   = 0
      CMAX = 0.0
  200 CONTINUE
        DO 220 NN = 1, NBRCH
          READ (LUPC,1050,END=280) J, N, M
          IF (N .EQ. NBRCHP) THEN
            NS = M
            READ (LUPC,1060) (PX(K), K = 1, M)
          ELSE
           READ (LUPC,1060) (XR(K), K = 1, M)
          END IF
          DO 210 L = 1, NEQ
            IF (L .EQ. LP  .AND. N .EQ. NBRCHP) THEN
              READ (LUPC,1060) (PT(K), K = 1, M)
            ELSE
              READ (LUPC,1060) (XR(K), K = 1, M)
            END IF
  210     CONTINUE
  220   CONTINUE
C       WRITE(LUOUT,1050)J,NBRCHP,NS
C       WRITE(LUOUT,1060)(PX(K),K=1,NS)
C       WRITE(LUOUT,1060)(PT(K),K=1,NS)
        M = IFIX(1000.0*RMP)
        K = 0
  250   CONTINUE
          K = K + 1
          IF (K .LE. NS) THEN
            I = IFIX ( PX(K) )
            AA = PX(K) - FLOAT ( I )
            AA = RM(I) + AA * (RM(I+1) - RM(I))
            NN = IFIX ( 1000.0*AA )
            IF (NN .LE. M) GO TO 250
          END IF
        K     = K - 1
        NP    = NP + 1
        CMIN  = PT(K)
        X(NP) = DT * (FLOAT(J+JTS)-0.5)
        X(1)  = DT * FLOAT(JTS)
        IF (X(NP) .GT. XMAX) XMAX = X(NP)
        Y(NP) = PT(K)
        IF (Y(NP) .GT. CMAX) CMAX = Y(NP)
        IF (Y(NP) .LT. CMIN) CMIN = Y(NP)
        WRITE(LUOUT,2010) X(NP), Y(NP)
      GO TO 200
  280 CONTINUE
C
C     read observed data
      IC = 1
      WRITE(*,*)' Enter 2 to plot observed, n or 1 otherwise  '
      READ (*,*,ERR=380) I
        IC = I
        IF (IC .NE. 2) GO TO 380
          OPEN (LUDT,FILE='obs')
          I = 0
          READ (LUDT,1000)  TITLE
          WRITE(LUOUT,2000) TITLE
          READ (LUDT,1000)  TITLE
          WRITE(LUOUT,2000) TITLE
          WRITE(LUOUT,*)'   TIME    OBSERVED      Error'
  310     CONTINUE
            I = I + 1
            READ (LUDT,*,END=320) XR(I), YR(I)
C           WRITE(LUOUT,2010) XR(I), YR(I)
            IF (XR(I) .GT. XMAX) XMAX = XR(I)
            IF (YR(I) .GT. CMAX) CMAX = YR(I)
            IF (YR(I) .LT. CMIN) CMIN = YR(I)
          GO TO 310
  320     CONTINUE
        CLOSE ( LUDT )
        NR = I - 1
C       compute rms
        SR  = 0.0
        SRR = 0.0
        DO 340 I = 1, NR
          N = 0
  330     CONTINUE
            N=N+1
            IF (N .GT. NP) GO TO 340
          IF (X(N).LE.XR(I)) GO TO 330
          YC = Y(N-1)+(Y(N)-Y(N-1))*(XR(I)-X(N-1))/(X(N)-X(N-1))
          ER = YC - YR(I)
          WRITE(LUOUT,2010) XR(I), YR(I), ER
          SR  = SR + ER
          SRR = SRR + ER * ER
  340   CONTINUE
        SR  = SR / (FLOAT(NR))
        SRR = SQRT(SRR/(FLOAT(NR)))
        WRITE(LUOUT,*) 'MEAN ERROR=',SR,'  RMS=',SRR,
     #                 ' BASED ON',NR, ' POINTS'
  380 CONTINUE
C
C     get axes limits from user
      CALL DRWLIM ( XMIN, DX, XMAX, CMIN, DY, CMAX )

C     label the axes
      XLAB = 'Time, in hours'
      YLAB = 'Concentration'
      MLAB = 'Constituent'
      CALL DRWSET ( XLAB, XMIN, DX, XMAX,
     I              YLAB, CMIN, DY, CMAX,
     I              MLAB, LP, IC )
C
C     simulated curve (solid line, no symbol, black)
      LEGEND = 'Simulated'
      CALL DRWCRV ( X,XMAX,XMIN,Y,CMAX,CMIN,NP,LEGEND, 1, 1, 1 )
      IF (IC .EQ. 2) THEN
C       observed curve (no line, 0 symbol, black)
        LEGEND = 'Observed'
        CALL DRWCRV ( XR,XMAX,XMIN,YR,CMAX,CMIN,NR,LEGEND, 2, 2, 1 )
      END IF
C
C     finish plot
      CALL DRWDON
C
C     close output files
      CLOSE   ( LUPC  )
      CLOSE   ( LUIN  )
      ENDFILE ( LUOUT )
      CLOSE   ( LUOUT )
C
      STOP
      END
