C
C
C
      PROGRAM   CXPLT
C
C     + + + PURPOSE + + +
C     Plots concentration vs distance.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOBR   - Maximum number of branches allowed in model
C     NOSC   - Maximum number of grids allowed in a branch
C     NOPR   - Maximum number of parcels allowed in branch
C     NOCP   - Twice the maximum nunber of computed points to be plotted
C
C     + + + LOCAL VARIABLES + + +
      INTEGER I,I1,J,JPO,K,L,LP,M,N,NBP(NOBR),NBPM,NBRCH,NEQ,NPN,
     #        NS(NOBR),NXSEC(NOBR),LUIN, LUPC, LUOUT, NP
      REAL    A,CMAX,CMIN,DX,DY,PT(NOBR,NOPR),PX(NOBR,NOPR),
     #        RM(NOBR,NOSC),X(NOCP),XG(NOBR,NOSC),
     #        XMAX,XMIN,XS,Y(NOCP)
      CHARACTER*20 LEGEND
      CHARACTER*40 YLAB, XLAB, MLAB
      CHARACTER*64 VERSN
      CHARACTER*80 TITLE
C
C     + + + LOCAL DEFINITIONS + + +
C     CMAX   - Maximum value of concentration to be plotted
C     CMIN   - Minimum value of concentration to be plotted
C     DX     - X grid spacing on plot
C     DY     - Y grid spacing on plot
C     JPO    - Number of time steps between output prints for PARCEL.OUT
C     K      - Parcel number
C     L      - Constituent number
C     LP     - Constituent number to be plotted
C     NBP(NBPM) Branches to be plotted
C     NBPM   - Number of branches to be plotted
C     NBRCH  - Number of branches in model
C     NEQ    - Number of equations in model
C     NP     - Number of points to be plotted
C     NPN    - Output number in file PARCEL.OUT to be plotted
C     NS(N)  - Number of parcels in branch N
C     NXSEC(N)  Number of grids in branch N
C     PT(N,K)- Concentration of L in parcel K of branch N
C     PX(N,K)- Location of u/s boundry of parcel K in branch N, in grid units
C     RM(N,I)- Miles downstream for grid I in branch N
C     X(NP)  - X plotting position for point NP
C     XG(N,I)- Distance of grid I in branch N downstream of origin
C     XMAX   - Maximum value of X to be plotted
C     XMIN   - Minimum value of X to be plotted
C     XS     - Maximum distance to be plotted or distance to last point
C     Y(NP)  - Y plotting position for point NP
C     TITLE  - Title of run
C
C     + + + INTRINSICS + + +
      INTRINSIC  MOD, IFIX, FLOAT
C
C     + + + EXTERNALS + + +
      EXTERNAL   DRWCRV, DRWDON, DRWLIM, DRWSET
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT ( A )
 1010 FORMAT ( 10X,10I7 )
 1020 FORMAT ( 10X,10F7.3 )
 1050 FORMAT ( 3I5 )
 1060 FORMAT ( 8G10.3 )
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (1X,A80  )
 2010 FORMAT ('  Enter title of output (no more than 80 characters)',/)
 2020 FORMAT (2G12.4)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)CXPLT - modified May  5, 1994, kmf'
      VERSN = '@(#)CXPLT - modified May  3, 1996, kmf'
      VERSN = '@(#)CXPLT - modified Jul 24, 1996, hej'
C
C     set unit numbers and open required data files
      LUIN  = 35
      LUPC  = 36
      LUOUT = 37
      OPEN (LUIN,  FILE = 'bltm.in')
      OPEN (LUPC,  FILE = 'parcel.out')
      OPEN (LUOUT, FILE = 'plt.out')
C
C     notice to user
      J = NOCP / 2
      WRITE(*,*) ' This program reads "parcel.out" and "bltm.in".'
      WRITE(*,*) ' It writes the file "plt.out".'
      WRITE(*,*) ' This program is dimensioned for:'
      WRITE(*,*) '      ', NOBR, ' branches'
      WRITE(*,*) '      ', NOSC, ' cross sections per branch'
      WRITE(*,*) '      ', NOPR, ' parcels per branch'
      WRITE(*,*) '      ',    J, ' plotting points.'
C
C     read data from "bltm.in"
      READ (LUIN,1000) TITLE
      READ (LUIN,1010) NBRCH,I,K,NEQ,J,L,JPO
      IF (NBRCH .GT. NOBR) GO TO 910
      READ (LUIN,1020) A
      DO 110 L = 1, NEQ
        READ(LUIN,1010)M
  110 CONTINUE
      DO 130 N = 1, NBRCH
        READ (LUIN,1010) NXSEC(N)
        I1 = NXSEC(N)
        IF (I1 .GT. NOSC) GO TO 920
        DO 120 I = 1, I1
          READ (LUIN,1020) RM(N,I)
          IF (NEQ.GT.7 .AND. I.LT.I1) READ(LUIN,1000) TITLE(77:80)
  120   CONTINUE
  130 CONTINUE
C
C     read plot data
      WRITE(*,*) ' Enter time step number of profile to be plotted- '
      READ (*,*) NPN
      IF (MOD(NPN,JPO) .NE. 0) THEN
C       use adjusted time step
        WRITE(*,*) ' Desired time step not stored'
      END IF
      NPN = 1 + NPN / JPO
      J   = (NPN - 1) * JPO
      WRITE(*,*) ' Will plot time step',J
      WRITE(*,*) ' Enter total number of branches to be plotted- '
      READ (*,*) NBPM
      DO 150 M = 1, NBPM
        WRITE(*,*) ' Enter next branch to be plotted- '
        READ (*,*) NBP(M)
  150 CONTINUE
      WRITE(*,*) ' Enter constituent number to be plotted- '
      READ (*,*) LP
C
C     read "parcel.out"
      DO 230 I1 = 1, NPN
        DO 220 M = 1, NBRCH
          READ (LUPC,1050) J,N,NS(N)
          IF (NS(N) .GT. NOPR) GO TO 930
          READ (LUPC,1060) (PX(N,K), K = 1, NS(N))
          DO 210 L = 1, NEQ
            IF (L .EQ. LP) THEN
              READ (LUPC,1060) (PT(N,K), K = 1, NS(N))
            ELSE
              READ (LUPC,1060) (X(K), K= 1, NS(N))
            END IF
  210     CONTINUE
  220   CONTINUE
  230 CONTINUE
C
C     build plot file
      XS = 0.0
      DO 320 M = 1, NBPM
        N  = NBP(M)
        I1 = NXSEC(N)
        DO 310 I = 1, I1
          XG(N,I) = RM(N,I) - RM(N,1) + XS
  310   CONTINUE
        XS = XG(N,I1)
  320 CONTINUE
      X(1) = 0.0
      Y(1) = 0.0
      NP   = 2
      CMAX = Y(1)
      CMIN = Y(1)
      XMIN = X(1)
      XMAX = X(1)
      DO 350 M = 1, NBPM
        N = NBP(M)
        DO 340 K = 1, NS(N)
          X(NP) = X(NP-1)
          Y(NP) = PT(N,K)
          NP    = NP+1
          IF (NP .GT. NOCP) GO TO 940
          X(NP) = XG(N,NXSEC(N))
          Y(NP) = PT(N,K)
          IF (CMAX .LT. Y(NP)) CMAX = Y(NP)
          IF (CMIN .GT. Y(NP)) CMIN = Y(NP)
          IF (XMAX .LT. X(NP)) XMAX = X(NP)
          IF (XMIN .GT. X(NP)) XMIN = X(NP)
          IF(K .EQ. NS(N)) GO TO 330
            I1    = IFIX( PX(N,K+1) )
            A     = PX(N,K+1) - FLOAT( I1 )
            X(NP) = XG(N,I1) + A * (XG(N,I1+1) - XG(N,I1))
  330     CONTINUE
          NP = NP + 1
          IF (NP .GT. NOCP) GO TO 940
  340   CONTINUE
  350 CONTINUE
      NP = NP - 1
      WRITE(*,2010)
      READ (*,1000) TITLE
      WRITE(LUOUT,2000) TITLE
      WRITE(LUOUT,*) '     X          C     PROFILE PLOTTED=',NPN
      DO 360 I = 1, NP
        WRITE(LUOUT,2020) X(I), Y(I)
  360 CONTINUE
C
C     get axes limits from user
      CALL DRWLIM (  XMIN, DX, XMAX, CMIN, DY, CMAX )
C
C     label the axes
      XLAB = 'Distance, in miles'
      YLAB = 'Concentration'
      MLAB = 'Constituent'
      CALL DRWSET ( XLAB, XMIN, DX, XMAX,
     I              YLAB, CMIN, DY, CMAX,
     I              MLAB, LP, 2 )
C
C     first curve (solid line, no symbol, black)
      LEGEND = 'Simulated'
      CALL DRWCRV ( X,XMAX,XMIN,Y,CMAX,CMIN,NP,LEGEND, 1, 1, 1 )
C
C     set up second curve
      IF (3*NBPM+3 .GT. NOCP) THEN
C       too many plotted points
        GO TO 940
      END IF
      NP     = 1
      X(NP) = 0.0
      Y(NP) = CMAX
      DO 410 M = 1, NBPM
        IF (NP+5 .GT. NOCP) THEN
C         too many plotted points
          GO TO 940
        END IF
        NP = NP+1
        N      = NBP(M)
        X(NP) = XG(N,NXSEC(N))
        Y(NP) = CMAX
        NP     = NP+1
        X(NP) = X(NP-1)
        Y(NP) = 0.0
        NP     = NP+1
        X(NP) = X(NP-1)
        Y(NP) = CMAX
  410 CONTINUE
      NP = NP + 1
      X(NP) = XS
      Y(NP) = CMAX
      NP     = NP + 1
      X(NP) = XS
      Y(NP) = 0.0
C
C     second curve (line, no symbol, black)
      LEGEND = '   '
      CALL DRWCRV ( X,XMAX,XMIN,Y,CMAX,CMIN,NP,LEGEND, 1, 1, 1 )
C
C     finish plot
      CALL DRWDON
      GO TO 990
C
  910 CONTINUE
        WRITE(*,*) ' Too many branches, increase NOBR'
        GO TO 990
  920 CONTINUE
        WRITE(*,*) ' Too many cross sections, increase NOSC'
        GO TO 990
  930 CONTINUE
        WRITE(*,*) ' Too many parcels per branch, increase NOPR'
        GO TO 990
  940 CONTINUE
        WRITE(*,*) ' Too many plotted points, increase NOCP'
C
  990 CONTINUE
      CLOSE   ( LUPC  )
      CLOSE   ( LUIN  )
      ENDFILE ( LUOUT )
      CLOSE   ( LUOUT )
C
      STOP
      END
