C
C     ************ MULTIPLE BRANCH UPLAND FLOW MODEL ******************
C
      PROGRAM   DAFLOW
C
C     + + + PURPOSE + + +
C     ******This program uses a variable DF=Q^(1-W2)/(2 W1 S)********
C     This program assumes A=AO+A1(Q**A2), computes celerity from dQ/dA,
C     and conserves mass to compute the average flow at the grids.
C     All boundry conditions represent the average during the time step.
C     The first BC represents the flow from time 0 to Dt, for example.
C     Dispersion is moleled by mixing at shocks over a dispersion distance.
C     The Q is at the grid point with QT occuring just upstream of grid.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C     NOBR   - Maximum number of branches allowed in model
C     NOSC   - Maximum number of cross sections (grids) allowed in branch
C     NOSH   - Maximum number of shocks allowed in branch
C              (NOSH should be at least 4 times NOSC)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I,IDBG,IENG,IERR,II,I1,IOUT(NOSC,NOBR),J,JCD(NOBR),
     #            JN,JNCD(NOBR),JNCU(NOBR),JGO,JTS,K,LUIN,LUOUT,
     #            LUFLW,M,N,NBC,NBRCH,NCD(NOBR),NHR,NJNCT,NN,NS(NOBR),
     #            NXSEC(NOBR),N1                                            ****
      REAL        A,AA,AO(NOSC,NOBR),AQ(NOSC,NOBR),A1(NOSC,NOBR),
     #            A2(NOSC,NOBR),SL(NOSC,NOBR),DT,DTS,F(NOSH,NOBR),
     #            PF(NOBR),PX(NOSH,NOBR),Q,QI,QJ,QP,
     #            TRB(NOSC,NOBR),TF(NOSC,NOBR),V(NOSC,NOBR),VI,VO,
     #            W1(NOSC,NOBR),W2(NOSC,NOBR),X(NOSC,NOBR),
     #            XFACT,XL,XR,W
      CHARACTER*64 VERSN
      CHARACTER*80 TITLE
C
C     + + + LOCAL DEFINITIONS + + +
C     AO(I,N)  Cross sectional area at zero flow for subreach I, branch N
C     AQ(I,N)  Discharge at grid I, branch N (time step averaged)
C     A1(I,N)  Cross sectional area at a flow of 1.0.
C     A2(I,N)  Exponent of area equation A=AO+A1(Q**A2).
C     SL(I,N)  Slope, wave dispersion coefficient=Q/(2*S*W)
C     DT       Time step size (hours)
C     F(K,N)   Steady flow in shock K of branch N
C     IDBG     Debugger code (0 or N = no, 1 = write debug code)
C     IENG     Input units: 0=metric , 1=English
C     IERR     Error code IERR<20 for warning, IERR>20 fatal
C              (1=DL too large, 21=too many shocks)
C     IOUT(I,N) Flag (1 = output) for grid I in branch N
C     J        Time step
C     JCD(M)   Code for junction mixing (0=mixed, 2=not mixed,
C                  1=not all inflows known)
C     JN       junction in question
C     JNCD(N)  d/s junction no. for branch N (number interior first)
C     JNCU(N)  u/s junction no. for branch N (number interior first)
C     JGO      Number of time steps between output in "flow.out"
C     JTS      Number of time steps from midnight to start of model
C     K        Shock number
C     NBC      Number of boundary conditions
C     NBRCH    Number of branchs
C     NCD(N)   Branch code (0=routed, 1=not routed)
C     NHR      Number of time steps to be modeled
C     NJNCT    Number of interior junctions
C     NS(N)    Number of shocks in branch N
C     NXSEC(N) Number of grids in branch N
C     PF(N)    Portion of flow at junction to enter branch N
C     PX(K,N)  Location of u/s boundary of shock K in branch N
C     QP       Maximum discharge of interest
C     QI       Insignificant discharge (QP/100000.0)
C     QJ       Flow at junction M
C     TITLE    Title of program (80 characters max)
C     TF(I,N)  Flow  in tributary at grid I of branch N
C                (tributaries can not occur at first or last grids of branch)
C     TRB(I,N) New flow in tributary at grid I of branch N
C     V(I,N)   Volume of water in subreach I of branch N
C     VI       Insignificant volume (QI*DTS)
C     W        Average top width of subreach
C     W1(I,N)  Coefficient in W=W1(Q)**W2 for subreach I branch N
C     W2(I,N)  Coefficient in W=W1(Q)**W2 for subreach I branch N
C     X(I,N)   Distance of grid I in branch N downstream of reference point
C     XFACT    Conversion from miles to feet or meters, depending on IENG
C
C     + + + INTRINSICS + + +
      INTRINSIC  FLOAT, IFIX, MOD
C
C     + + + EXTERNALS + + +
      EXTERNAL   ROUTE, FKAI, FVOL
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (A)
 1010 FORMAT (20X,I10)
 1020 FORMAT (18X,I3)
 1030 FORMAT (20X,F10.3)
 1040 FORMAT (13X,I3,16X,F5.2,16X,I3,8X,I3)
 1050 FORMAT (3X,G11.4,I2,G11.4,4G10.3,F7.1,F6.3)
 1060 FORMAT (10X,I3,5X,I3,3X,G14.5)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (20X,A,/)
 2010 FORMAT (' The',I3,' Branch Model with',I3,' Internal Junctions',
     #        ' is run',I5,' Time Steps each',F5.2,' hours long.')
 2020 FORMAT ('The Model starts at',F6.2,' hours past midnight.')
 2030 FORMAT ('The grid output is given in "bltm.out" every',I4,
     #        ' time steps.')
 2040 FORMAT ('Input units are Metric (Meters) except river miles')
 2050 FORMAT ('Input units are English (feet and river miles)')
 2060 FORMAT (' Width=W1(Q)**W2')
 2070 FORMAT (' Cross sectional area = AO+A1(Q**A2)')
 2080 FORMAT (' AO = Cross sectional area at zero flow.')
 2090 FORMAT (/,28X,'* * *  INITIAL CONDITIONS  * * *')
 2100 FORMAT (/,' Grid    Mile  Disch       Area      Width  ',
     $          ' Slope           A1         A2         W1        ',
     $          ' W2         AO')
 2110 FORMAT (/,'Branch',I4,' Extends from JNCT',I3,' to JNCT',I3,
     $        ' and receives',F5.2,' of flow at JNCT',I3,/)
 2120 FORMAT (I5,F8.2,9G11.4)
 2130 FORMAT (' Day',I4,'  Hour',F6.1,' Branch  Grid  Discharge')
 2140 FORMAT (20X,I7,I6,G14.4)
 2150 FORMAT (/,25X,'* * *  OUTPUT  * * *',/)
 3000 FORMAT (3I5,4E18.5)
 4000 FORMAT ('TOO MANY WAVES IN BRANCH',I3,' AT J=',I5)
 4010 FORMAT ('Computed negative flow of',G14.3,' at',
     $         I4,' branch',I3, ' grid',I3)
 4020 FORMAT(' Warning DL>C DT in branch',I3,'. Increase DT to smooth.')
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)DAFLOW - last modified May  5, 1994, kmf'
      VERSN = '@(#)DAFLOW - last modified Feb  8, 1995, kmf'
      VERSN = '@(#)DAFLOW - last modified Sep 26, 1995, kmf'
      VERSN = '@(#)DAFLOW - last modified Apr 02, 1996, hej & kmf'
C     correction in computing range of possible mixed Qs - hej
C
C     set unit numbers and open required data files
      LUIN=15
      LUOUT=16
      LUFLW=17
      OPEN (LUIN,FILE='flow.in')
      OPEN (LUOUT,FILE='flow.out')
      OPEN (LUFLW,FILE='bltm.flw')
C
C     notice to user
      WRITE(*,*)' This program reads "flow.in"'
      WRITE(*,*)' It writes the files "flow.out" and "bltm.flw"'
      WRITE(*,*)' This program is dimensioned for:'
      WRITE(*,*)'      ',NOBR, ' branches'
      WRITE(*,*)'      ',NOSC, ' cross sections per branch'
      WRITE(*,*)'      ',NOSH, ' shocks per branch.'
C
C     zero arrays and preliminaries
      IERR=0
      DO 40 N=1,NOBR
        JCD(N)=0
        JNCU(N)=0
        JNCD(N)=0
        NS(N)=0
        NXSEC(N)=0
        DO 20 I=1,NOSC
          SL(I,N)=0.0
          IOUT(I,N)=0
          X(I,N)=0.0
          TF(I,N)=0.0
          TRB(I,N)=0.0
   20   CONTINUE
        DO 30 K=1,NOSH
          PX(K,N)=0.0
          F(K,N)=0.0
   30   CONTINUE
   40 CONTINUE
C
C     read common input
      READ(LUIN,1000)   TITLE
      WRITE(LUOUT,2000) TITLE
      READ(LUIN,1010) NBRCH
      READ(LUIN,1010) NJNCT
      READ(LUIN,1010) NHR
      READ(LUIN,1010) JTS
      READ(LUIN,1010) JGO
      READ(LUIN,1010) IENG
      READ(LUIN,1030) DT
      READ(LUIN,1030) QP
      WRITE(LUOUT,2010) NBRCH,NJNCT,NHR,DT
      AA=DT*FLOAT(JTS)
      DTS=DT*3600.0
      QI=QP/100000.0
      VI=QI*DTS
      WRITE(LUOUT,2020) AA
      WRITE(LUOUT,2030) JGO
C
C     read data for each branch
      DO 60 N=1,NBRCH
        READ(LUIN,1040) NXSEC(N),PF(N),JNCU(N),JNCD(N)
        READ(LUIN,1000)TITLE
        DO 50 I=1,NXSEC(N)
          IF(I.LT.NXSEC(N))THEN
            READ(LUIN,*)II,X(I,N),IOUT(I,N),F(I,N),A1(I,N),A2(I,N),
     #                  AO(I,N),SL(I,N),W1(I,N),W2(I,N)
          ELSE
            READ(LUIN,*)II,X(I,N),IOUT(I,N)
          END IF
   50   CONTINUE
   60 CONTINUE
C
C     make preliminary computation and write initial conditions
      IF (IENG .EQ. 0) THEN
C       metric units
        WRITE(LUOUT,2040)
        XFACT=1609.34
      ELSE
C       english units
        WRITE(LUOUT,2050)
        XFACT=5280.00
      END IF
      WRITE(LUOUT,2070)
      WRITE(LUOUT,2080)
      WRITE(LUOUT,2060)
      WRITE(LUOUT,2090)
      WRITE(LUOUT,2100)
      J=0
      DO 80 N=1,NBRCH
        NS(N)=NXSEC(N)-1
        WRITE(LUOUT,2110) N,JNCU(N),JNCD(N),PF(N),JNCU(N)
        DO 70 I=1,NXSEC(N)
          IF (I .LT. NXSEC(N)) THEN
            PX(I,N)=X(I,N)*XFACT
            A=AO(I,N)+A1(I,N)*(F(I,N)**A2(I,N))
            V(I,N)=A*XFACT*(X(I+1,N)-X(I,N))
            IF(F(I,N).GT.0.0) THEN
              W=W1(I,N)*(F(I,N)**W2(I,N))
            ELSE
              W=0.0
            END IF
            WRITE(LUOUT,2120) I,X(I,N),F(I,N),A,W,SL(I,N),
     $                        A1(I,N),A2(I,N),
     $                        W1(I,N),W2(I,N),AO(I,N)
            IF(I.GT.1)THEN
              TF(I,N)=F(I,N)-F(I-1,N)
            END IF
            WRITE(LUFLW,3000) J,N,I,F(I,N),A,W,TF(I,N)
          ELSE
            WRITE(LUOUT,2120) I,X(I,N),F(I-1,N)
            WRITE(LUFLW,3000) J,N,NXSEC(N),F(I-1,N)
          END IF
          X(I,N)=X(I,N)*XFACT
   70   CONTINUE
   80 CONTINUE
C
      IDBG=0
C     WRITE(*,*)' Do you want the debug statements, ',
C    $          '0 or N = no, 1 = yes'
C     READ(*,'(I10)',ERR=90)I
C     IDBG=I
C 90  CONTINUE
C
C     start time loop
      WRITE(LUOUT,2150)
      DO 100 J=1,NHR
        WRITE(*,*)' Starting time step',J
C
C       set boundary and junction values
        DO 110 N=1,NBRCH
C         branch code set to not routed
          NCD(N)=1
 110    CONTINUE
        DO 120 N=1,NJNCT
C         junction mixing code set to not mixed
          JCD(N)=2
 120    CONTINUE
C       read boundary conditions
        READ(LUIN,1020)NBC
        IF(NBC.GT.0)THEN
C         boundary conditons for this time with id for where
          DO 140 M=1,NBC
            READ(LUIN,1060)N,I,TRB(I,N)
  140     CONTINUE
        END IF
C
C       route branches
        DO 200 NN=1,NBRCH
          N=0
          IERR=0
  150     CONTINUE
C           looking for something to route?
            N=N+1
            IF(JCD(JNCU(N)).EQ.1)GO TO 150
            IF(NCD(N).EQ.0)GO TO 150
          CALL ROUTE
     I      (AO(1,N),A1(1,N),A2(1,N),SL(1,N),DTS,IDBG,IERR,J,LUOUT,N,
     M      NXSEC(N),NS(N),F(1,N),PX(1,N),QI,TRB(1,N),TF(1,N),
     N      VI,W1(1,N),W2(1,N),X(1,N))
C         an error causes a quick and nasty exit
          IF(IERR.GT.20)GO TO 440
          IF(IERR.EQ.1)THEN
            WRITE(LUOUT,4020)N
            WRITE(*,4020)N
          END IF
C
C         update junction flows and codes
          NCD(N)=0
          JN=JNCD(N)
          JCD(JN)=0
          QJ=0.0
          AQ(1,N)=TF(1,N)
          II=NXSEC(N)-1
          DO 160 I=1,II
            XL=X(I,N)
            XR=X(I+1,N)
            VO=V(I,N)
            CALL FKAI(K,NS(N),PX(1,N),X(I,N))
            CALL FVOL(AO(1,N),A1(1,N),A2(1,N),K,NS(N),NXSEC(N),F(1,N),
     #                PX(1,N),TF(1,N),V(I,N),X(1,N),XL,XR)
            AQ(I+1,N)=AQ(I,N)+TF(I+1,N)+(VO-V(I,N))/DTS
            IF (IDBG.EQ.1) THEN
C             debug output
              WRITE(LUOUT,*)'VO,V(I,N),AQ',VO,V(I,N),AQ(I+1,N)
            END IF
            IF (AQ(I+1,N).LT.0.0) THEN
C             can't have negative flow
              I1      = I+1
              WRITE(LUOUT,4010) AQ(I1,N),J,NN,I1
              AQ(I1,N)= 0.0
            END IF
  160     CONTINUE
          DO 170 N1=1,NBRCH
            IF (JNCD(N1).EQ.JN) THEN
              IF (NCD(N1).NE.0) JCD(JN)=1
              QJ=QJ+AQ(NXSEC(N1),N1)
            END IF
  170     CONTINUE
          IF(JCD(JN).NE.1) THEN
            DO 180 N1=1,NBRCH
              IF(JNCU(N1).EQ.JN) THEN
                TRB(1,N1)=QJ*PF(N1)
              END IF
  180       CONTINUE
          END IF
  200   CONTINUE
C
C       write results
        AA=(FLOAT(J+JTS-1)+0.5)*DT
        II=IFIX(AA/24.0)+1
        AA=AA-FLOAT(II-1)*24.0
        IF(MOD(J,JGO).EQ.0) THEN
          WRITE(LUOUT,2130) II,AA
        END IF
        DO 220 N=1,NBRCH
          DO 210 I=1,NXSEC(N)
            IF (MOD(J,JGO).EQ.0 .AND. IOUT(I,N).EQ.1) THEN
              WRITE(LUOUT,2140) N,I,AQ(I,N)
            END IF
            IF (I.LT.NXSEC(N))THEN
              IF (I.GT.1) THEN
                AA= TRB(I,N)
              ELSE
                AA= 0.0
              END IF
              A= V(I,N)/(X(I+1,N)-X(I,N))
              IF (A.GT.AO(I,N)) THEN
                Q= ((A-AO(I,N))/A1(I,N))**(1.0/A2(I,N))
              ELSE
                Q=0.0
              END IF
              IF (Q.GT.0.0) THEN
                W= W1(I,N)*(Q**W2(I,N))
              ELSE
                W= 0.0
              END IF
              WRITE(LUFLW,3000)J,N,I,AQ(I,N),A,W,AA
            END IF
  210     CONTINUE
          WRITE(LUFLW,3000)J,N,NXSEC(N),AQ(NXSEC(N),N)
  220   CONTINUE
  100 CONTINUE
C     end of time loop
      GO TO 240
C
C     nasty error comes to here
  440 CONTINUE
      IF (IERR.EQ.21) THEN
        WRITE(LUOUT,4000)N,J
      END IF
C
  240 CONTINUE
C     close files
      CLOSE(LUIN)
      ENDFILE(LUOUT)
      CLOSE(LUOUT)
      ENDFILE(LUFLW)
      CLOSE(LUFLW)
C
      STOP
      END
