
C     ************ EQUILIBRIUM TEMPERATURE MODEL ******************
C
      PROGRAM   EQULTMP
C
C     + + + PURPOSE + + +
C     Computes the equilibrium temperature from the times and
C     values of the extreme temperatures for the day.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      JE,JS,J,LUOUT
      REAL         AM,DAM,DT,DV,P,T,TA,TE,V,DTH,DTL,PH,TAH,TAL,
     $             TH,TL,TO,VO
      CHARACTER*64 VERSN
      CHARACTER*80 TITLE
C
C     + + + LOCAL DEFINITIONS + + +
C     AM     - Thermal mass of atmosphere
C     DAM    - Default value of AM
C     DT     - Time step size in model, in hours
C     P      - 3.14159
C     PH     - Phase angle
C     T      - Time, in hours
C     TA     - Air temperature in degrees F
C     TAH    - High air temperature for the day F
C     TAL    - Low air temperature for the day F
C     TITLE  - Title to placed on data file
C     TE     - Equilibrium temperature, in degrees F
C     TH     - Time of peak temperature for the day, in hours
C     TL     - Time of low temperature for the day, in hours
C     TR     - Temperature range for the day, in degrees F
C     V      - Wind speed, in miles per hour
C
C     + + + INTRINSICS + + +
      INTRINSIC   COS, FLOAT, IFIX, SIN
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (A)
 1001 FORMAT (1X,A)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT(4F7.2)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)TEMP  - last modified May  2, 1996, kmf'
C
C     zero arrays and priliminaries
      P=3.14159
C
C     read input data
      WRITE(*,*) ' Output written to "out"'
      WRITE(*,*) ' Enter any letter to quit, 1 to continue '
      READ(*,'(I10)',ERR=99) JE
C
C     set unit numbers and open required data files
      LUOUT = 16
      OPEN (LUOUT, FILE='out')
C
      WRITE(*,*) ' Enter time step size, in hours  '
      READ (*,*) DT
      WRITE(*,*) ' DT=',DT
      WRITE(*,*) ' Enter title for data file (80 character max)  '
      READ (*,1000) TITLE
      WRITE(LUOUT,1001) TITLE
      WRITE(LUOUT,*) '  Time Equl C    M/s  Air T'
      WRITE(*,*) ' To select default enter any letter.  '
      AM=0.15
      WRITE(*,*) ' Enter thermal air mass, default=0.15 AM= '
      READ (*,'(F10.0)',ERR=1) DAM
        AM=DAM
        WRITE(*,*)'AM=',AM
    1 CONTINUE
      DTH=16.0
      DTL=5.0
      DV=10.0
      WRITE(*,*) ' Enter TIME of MAXimum temp. for previous day'
      WRITE(*,*) ' default=',DTH,'  DTH (hrs) = '
      READ (*,'(F10.0)',ERR=2) TH
        DTH=TH
    2 CONTINUE
      TH=DTH
      WRITE(*,*) ' Input temperatures in degrees F'
      WRITE(*,*) ' Enter MAXimum TEMPerature for the previous day,',
     #           ' TAH= '
      READ (*,*) TAH
      WRITE(*,*) ' Enter TIME of MINimum temperature for the day'
      WRITE(*,*) ' default=',DTL,'  TL (hrs) = '
      READ (*,'(F10.0)',ERR=3) TL
        DTL=TL
    3 CONTINUE
      TL=DTL
      WRITE(*,*) ' Time of MIN temp =',TL,' hrs.'
      WRITE(*,*) ' Enter MINimum TEMPerature for the day'
      WRITE(*,*) '   TAL (deg F) = '
      READ(*,*) TAL
    5 CONTINUE
        WRITE(*,*) ' Enter wind speed for the day, default=',DV
        WRITE(*,*) ' V (mph)= '
        READ (*,'(F10.0)',ERR=6) V
          DV=V
    6   CONTINUE
        V=DV
        WRITE(*,*) ' Wind speed, mph =',V
C
C       Compute values for the first part of the day
        JE=IFIX(TL/DT)
        DO 10 J=1,JE
          T=DT*(FLOAT(J)-0.5)
          PH=((T+(24.0-TH))/(TL+24.0-TH))*P+P*0.5
          TA=(TAH-TAL)*0.5*SIN(PH)+(TAH+TAL)*0.5
          TE=TA+(TAH-TAL)*0.5*(P/(TL+24.0-TH))*COS(PH)/AM
          TO=5.0*(TE-32.0)/9.0
          VO=V*0.447
          WRITE(LUOUT,2000)T,TO,VO,TA
   10   CONTINUE
C
C       Compute values for center part of the day
        WRITE(*,*) ' Enter TIME of MAXimum temperature for day'
        WRITE(*,*) ' default=',DTH,'TH (hrs) = '
        READ (*,'(F10.0)',ERR=11) TH
          DTH=TH
   11   CONTINUE
        TH=DTH
        WRITE(*,*) ' Enter MAXimum TEMPerature for the day, TAH= '
        READ (*,*) TAH
        JS=JE+1
        JE=IFIX(TH/DT)
        DO 12 J=JS,JE
          T=DT*(FLOAT(J)-0.5)
          PH=((T-TL)/(TH-TL))*P+P*1.5
          TA=(TAH-TAL)*0.5*SIN(PH)+(TAH+TAL)*0.5
          TE=TA+(TAH-TAL)*0.5*(P/(TH-TL))*COS(PH)/AM
          TO=5.0*(TE-32.0)/9.0
          VO=V*0.447
          WRITE(LUOUT,2000)T,TO,VO,TA
   12   CONTINUE
C
C       Compute values for last part of the day
        WRITE(*,*) ' Enter TIME of MINimum temperature for the next day'
        WRITE(*,*) ' default=',DTL,'TL (hrs) = '
        READ (*,'(F10.0)',ERR=13) TL
          DTL=TL
   13   CONTINUE
        TL=DTL
        WRITE(*,*) ' Enter MINimum TEMPerature for the next day'
        WRITE(*,*) ' TAL (deg F) = '
        READ (*,*) TAL
        JS=JE+1
        JE=IFIX(24.0/DT)
        DO 14 J=JS,JE
          T=DT*(FLOAT(J)-0.5)
          PH=((T-TH)/(TL+24.0-TH))*P+P*0.5
          TA=(TAH-TAL)*0.5*SIN(PH)+(TAH+TAL)*0.5
          TE=TA+(TAH-TAL)*0.5*(P/(TL+24.0-TH))*COS(PH)/AM
          TO=5.0*(TE-32.0)/9.0
          VO=V*0.447
          WRITE(LUOUT,2000)T,TO,VO,TA
   14   CONTINUE
        WRITE(*,*) ' Do you want another day? (0=no, y=yes) '
      READ (*,'(I10)',ERR=5) JE
C
C     close files
      ENDFILE ( LUOUT )
      CLOSE   ( LUOUT )
      WRITE(*,*) ' The output was written to the file "out"'
C
   99 CONTINUE
C
      STOP
      END
