C
C
C
      SUBROUTINE   FINK
     ?                 (K,MX,J,NEQ,PV,PT,
     ?                  Q,A,W,XK,S,CR,IRC,NNN,LUOUT)
C
C     + + + PURPOSE + + +
C     Get decay coefficients from user.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOBR   - Maximum number of branches allowed in model
C     NOSC   - Maximum number of cross sections (grids) allowed in branch
C     NOPR   - Maximum number of parcels allowed in branch
C              (NOPR should be at least 20 + 2 times NOSC)
C     NOCO   - Maximum number of constituents allowed
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   K, MX, J, NEQ, IRC, NNN, LUOUT
      REAL      PV, PT(NOCO,NOPR), Q, A, W, XK(NOCO,NOCO), S(NOCO),
     #          CR(NOCO,NOCO)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     K      - Parcel number
C     MX     - Grid just upstream of parcel's upstream boundary
C     J      - Time step
C     NEQ    - Number of equations (constituents)
C     PV(K)  - Volume of parcel
C     PT(L,K)- Concentration of constituent L in parcel K, mg/l
C     Q      - Discharge, cfs or cms
C     A      - Average area of the subreach (sq m or sq ft)
C     W      - Average top width of the subreach, m or ft
C     XK(L,M)- Exchange coefficient for constituent L due to the
C              concentration of constituent M
C     S(L)   - Source flux of constituent L, per hour
C     CR(M,L)- Reference (equilibrium) concentration in equation M
C              for constituent L
C     IRC    - Code to set read for new data each time step
C              1=read data, 0=no read
C     NNN    - Branch number
C
C     + + + COMMONS + + +
      INTEGER   NXSEC(NOBR), NBRCH, IENG
      COMMON  /CMBLTM/  NXSEC, NBRCH, IENG
C
C     + + + COMMON DEFINTIONS + + +
C     IENG   - Input units: 0=metric, 1=English
C     NBRCH  - Number of branches
C     NXSEC(N) Number of subreaches in branch N
C
C     + + + SAVES + + +
      REAL    AK(NOBR,NOSC,NOCO)
      SAVE    AK

C     + + + SAVE DEFINITIONS + + +
C     AK(N,I,L)- Decay coefficient for branch N, subreach I, conc L
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      I,I1,N,L
      CHARACTER*64 VERSN
C
C     + + + OUTPUT FORMATS + + +
 1000 FORMAT(3I6,12X,G10.3)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)FINK - last modified May 5, 1994, kmf'
      VERSN = '@(#)FINK - last modified May 3, 1996, kmf'
C
      IF (IRC .EQ. 1) THEN
        IF (J .EQ. 1) THEN
          WRITE(LUOUT,*) 'Branch  Grid  Const No.  Decay cof in per hr.'
          WRITE(*,*) ' Enter all decay coefficients in per hour'
          DO 1 N=1,NBRCH
            WRITE(*,*) ' Enter decay coefficients for Branch- ',N
            I1=NXSEC(N)-1
            DO 1 I=1,I1
              WRITE(*,*) ' For subreach- ',I
              DO 1 L=1,NEQ
                WRITE(*,*) ' Enter decay coefficient for constituent',
     #                     L,' - '
                READ (*,*) AK(N,I,L)
                WRITE(LUOUT,1000)N,I,L,AK(N,I,L)
    1     CONTINUE
        END IF
      END IF
      DO 3 L=1,NEQ
        XK(L,L)=-AK(NNN,MX,L)
    3 CONTINUE
      IRC=0
C
      RETURN
      END
