C
C     ************ OPTIMIATION PROGRAM FOR DAFLOW MODEL ******************
C
      PROGRAM   FLWOPT
C
C     + + + PURPOSE + + +
C     Program is designed to optimize flow hydraulic coefficients
C     ******This program uses a variable DF=Q^(1-W2)/(2 W1 S)********
C     This program assumes A=AO+A1(Q**A2).
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C     NOBR   - Maximum number of branches allowed in model
C     NOSC   - Maximum number of cross sections (grids) allowed in branch
C     NOSH   - Maximum number of shocks allowed in branch
C              (NOSH should be at least 4 times NOSC)
C     NOTS   - Maximum number of time steps
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I,IDBG,IENG,IERR,IERRR,II,INX,IOUT(NOSC,NOBR),J,
     #            JCD(NOBR),JN,JNCD(NOBR),JNCU(NOBR),K,LUIN,LUOUT,
     #            LUOBS,N,NB,NBC,NBRCH,NCD(NOBR),NG,NHR,NIT,NJNCT,NN,
     #            NOBS,NRMS,NS(NOBR),NXSEC(NOBR),N1
      REAL        AA,AO(NOSC,NOBR),AQ(NOSC,NOBR),A1(NOSC,NOBR),
     #            A1R(NOSC,NOBR),A2(NOSC,NOBR),A2R(NOSC,NOBR),BC(NOSC,
     #            NOBR,NOTS),DT,DTS,F(NOSH,NOBR),FI(NOSC,NOBR),
     #            PF(NOBR),PRC,PX(NOSH,NOBR),QI,QJ,QOLD,QOBS(NOTS),QP,
     #            QR,RMSL,RMSR,SL(NOSC,NOBR),SR,SRR,
     #            TF(NOSC,NOBR),TIME,TOBS(NOTS),TRB(NOSC,NOBR),
     #            V(NOSC,NOBR),VI,VO,W1(NOSC,NOBR),W1R(NOSC,NOBR),
     #            W2(NOSC,NOBR),W2R(NOSC,NOBR),X(NOSC,NOBR),XFACT,XL,XR
      CHARACTER*64 VERSN
      CHARACTER*80 TITLE
C
C     + + + LOCAL DEFINITIONS + + +
C     AO(I,N)  Cross sectional area at zero flow for subreach I, branch N
C     AQ(I,N)  Discharge at grid I, branch N (time step averaged)
C     A1(I,N)  Cross sectional area at a flow of 1.0.
C     A1R(I,N) Reference A1
C     A2(I,N)  Exponent of area equation A=AO+A1(Q**A2).
C     A2R(I,N) Reference A2
C     BC(I,N,J) Boundary condition for grid, branch, time step.
C     DT       Time step size; DT (hours), DTS (seconds)
C     F(K,N)   Steady flow in shock K of branch N
C     FI(I,N)  Initial flow in subreach I of branch N
C     IDBG     Debugg code (0 or N = no, 1 = write debug code)
C     IENG     Input units: 0=metric , 1=English
C     IERR     Error code IERR<20 for warning, IERR>20 faital
C              (1=DL too large, 21=too many shocks)
C     IERRR    DL too large write at end of itteration
C     INX      NXSEC-1
C     IOUT(I,N) Flag (1 = output) for grid I in branch N
C     J        Time step
C     JCD(M)   Code for junction mixing (0=mixed, 2=not mixed,
C                  1=not all inflows known)
C     JN       junction in question
C     JNCD(N)  d/s junction no. for branch N (number interior first)
C     JNCU(N)  u/s junction no. for branch N (number interior first)
C     K        Shock number
C     NB       Branch where fit is obtained
C     NBC      Number of boundary conditions
C     NBRCH    Number of branches
C     NCD(N)   Branch code (0=routed, 1=not routed)
C     NG       Grid where fit is obtained
C     NHR      Number of time steps to be modeled
C     NIT      Number of combinations (1, 9)
C     NJNCT    Number of interior junctions
C     NOBS     Number of observed data
C     NRMS     Number of errors computed
C     NS(N)    Number of shocks in branch N
C     NXSEC(N) Number of grids in branch N
C     PF(N)    Portion of flow at junction to enter branch N
C     PRC      Precision of optimization (0.1=10%)
C     PX(K,N)  Location of u/s boundary of shock K in branch N
C     QOLD     Computed flow at optimization point for last time step
C     QOBS(J)  Observed flow
C     QP       Maximum discharge of interest
C     QI       Insignificant discharge (QP/100000.0)
C     QJ       Flow at junction M
C     QR       Reference discharge, where C & W held const as A2/W2 change
C     RMSL     RMS for variable decreased by the precision step
C     RMSR     RMS for variable increased by the precision step
C     SL(I,N)  Slope, wave dispersion coefficient=Q/(2*SL*W)
C     SR       Sum of errors
C     SRR      Sum of square of errors
C     TITLE    Title of program (80 characters max)
C     TF(I,N)  Flow  in tributary at grid I of branch N
C                (tributaries can not occur at first or last grids of branch)
C     TOBS(J)  Observed time 
C     TRB(I,N) New flow in tributary at grid I of branch N
C     V(I,N)   Volume of water in subreach I of branch N
C     VI       Insignificant volume (QI*DTS)
C     W1(I,N)  Coefficient in W=W1(Q)**W2 for subreach I branch N
C     W1R(I,N) Reference W1
C     W2(I,N)  Coefficient in W=W1(Q)**W2 for subreach I branch N
C     W2R(I,N) Reference W2
C     X(I,N)   Distance of grid I in branch N downstream of reference point
C     XFACT    Conversion from miles to feet or meters, depending on IENG
C
C
C     + + + INTRINSICS + + +
      INTRINSIC  FLOAT
C
C     + + + EXTERNALS + + +
      EXTERNAL   ROUTE, FKAI, FVOL
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (A)
 1010 FORMAT (20X,I10)
 1020 FORMAT (18X,I3)
 1030 FORMAT (20X,F10.3)
 1040 FORMAT (13X,I3,16X,F5.2,16X,I3,8X,I3)
 1050 FORMAT (I3,G11.4,I2,G11.4,4G10.3,F7.1,F6.3)
 1060 FORMAT (10X,I3,5X,I3,3X,G14.5)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (' The',I3,' Branch Model with',I3,' Internal Junctions',
     $        ' is run',I5,' Time Steps each',F5.2,' hours long.')
 2010 FORMAT (/,28X,'* * * OPTIMUM PARAMETERS CONDITIONS  * * *')
 2020 FORMAT ('Branch',I3,' has',I3,' xsects & routes',F5.2,
     $       ' of flow at',' JNCT',I3,' To JNCT',I3)
 2030 FORMAT ('Grd R Mile  IOUT  Disch        A1        A2        ',
     $        'AO         SL         W1    W2')
 4000 FORMAT ('TOO MANY WAVES IN BRANCH',I3,' AT J=',I5)
 4010 FORMAT ('Computed negative flow of',G14.3,' at',I4,
     $        ' branch',I3,' grid',I3)
 4020 FORMAT (' Warning DL>C DT, Increase DT to smooth.')
 4030 FORMAT ('Precision =',F7.3,' Reference RMS = ',G14.6)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)FLWOPT - last modified Apr 04, 1996, hej & kmf'
C
C     set unit numbers and open required data files
      LUIN=15
      LUOUT=16
      LUOBS=17
      OPEN (LUIN,FILE='flow.in')
      OPEN (LUOUT,FILE='optw.out')
      OPEN (LUOBS,FILE='obs')
C
      WRITE(*,*)' This program reads "flow.in"'
      WRITE(*,*)' It writes the files "opt.out" '
      WRITE(*,*)' This program is dimensioned for',NOBR, ' branches.'
      WRITE(*,*)'          For',NOSC, ' cross sections per branch.'
      WRITE(*,*)'          For',NOSH, ' shocks per branch.'
      WRITE(*,*)'          For',NOTS, ' time steps.'
C
C     zero arrays and priliminaries
      DO 10 N=1,NOBR
        DO 10 I=1,NOSC
          IOUT(I,N)=0
          X(I,N)=0.0
          TF(I,N)=0.0
          TRB(I,N)=0.0
   10   CONTINUE
C
C     read common input
      READ(LUIN,1000)   TITLE
      WRITE(LUOUT,1000) TITLE
      READ(LUIN,1010) NBRCH
      READ(LUIN,1010) NJNCT
      READ(LUIN,1010) NHR
      READ(LUIN,1010) N
      READ(LUIN,1010) N
      READ(LUIN,1010) IENG
      IF (IENG .EQ. 0) THEN
C       metric units
        XFACT=1609.34
      ELSE
C       english units
        XFACT=5280.00
      END IF
      READ(LUIN,1030) DT
      READ(LUIN,1030) QP
      WRITE(LUOUT,2000) NBRCH,NJNCT,NHR,DT
      DTS=DT*3600.0
      QI=QP/100000.0
      VI=QI*DTS
C
C     read data for each branch
      DO 20 N=1,NBRCH
        READ(LUIN,1040) NXSEC(N),PF(N),JNCU(N),JNCD(N)
        READ(LUIN,1000)TITLE
        WRITE(LUOUT,1000)TITLE
        DO 20 I=1,NXSEC(N)
          READ(LUIN,1050) II,X(I,N),IOUT(I,N),FI(I,N),A1R(I,N),A2R(I,N),
     #                    AO(I,N),SL(I,N),W1R(I,N),W2R(I,N)
          WRITE(LUOUT,1050) II,X(I,N),IOUT(I,N),FI(I,N),A1R(I,N),
     #                      A2R(I,N),AO(I,N),SL(I,N),W1R(I,N),W2R(I,N)
          X(I,N)=X(I,N)*XFACT
   20 CONTINUE
C
C     read boundary conditions
      DO 50 J=1,NHR
        DO 30 N=1,NBRCH
          DO 30 I=1,NXSEC(N)
            BC(I,N,J)=0.0
            IF(J.GT.1)BC(I,N,J)=BC(I,N,J-1)
   30   CONTINUE
        READ(LUIN,1020)NBC
        IF(NBC.EQ.0)GO TO 50
          DO 40 II=1,NBC
            READ(LUIN,1060)N,I,BC(I,N,J)
   40   CONTINUE
   50   CONTINUE
C
C     make preliminary computations
      WRITE(LUOUT,2010)
C
C     ****** read optimization data ******
      WRITE(*,*)' Enter branch number where fit is desired  '
      READ(*,*)NB
      WRITE(*,*)' Enter grid number where fit is desired  '
      READ(*,*)NG
      READ(LUOBS,1000,END=450)TITLE
      WRITE(LUOUT,1000)TITLE
      READ(LUOBS,1000,END=450)TITLE
      NOBS=0
   60 READ(LUOBS,*,END=70)XL,AA
        NOBS=NOBS+1
        TOBS(NOBS)=XL
        QOBS(NOBS)=AA
      GO TO 60
   70 IDBG=0
      CLOSE(LUOBS)
      WRITE(LUOUT,*)' No of observed data =',NOBS
C     WRITE(*,*)' Do you want debug output? 0 or N = no, 1 = yes - '
C     READ(*,'(I10)',ERR=80)IDBG
      IDBG=I
   80 WRITE(*,*)' Enter discharge where Celerity & Width remain const,',
     $          ' QR = '
      READ(*,*)QR
      PRC=0.1
C
C     **** start itteration loop
  100 CONTINUE
      WRITE(*,*)' Enter precision level (0.1=10%, 0.01=1%)' 
      WRITE(*,*)'       Default=',PRC,'   PRC = '
      READ(*,'(F10.0)',ERR=110)AA
        PRC=AA
  110 DO 420 NIT=1,9
        IERRR=0
        SR=0.0
        SRR=0.0
        NRMS=1
        QOLD=FI(NG,NB)
C       *************** Set hydraulic geometry coefficients ****
        DO 120 N=1,NBRCH
          NS(N)=NXSEC(N)-1
          DO 120 I=1,NS(N)
            A1(I,N)=A1R(I,N)
            A2(I,N)=A2R(I,N)
            W1(I,N)=W1R(I,N)
            W2(I,N)=W2R(I,N)
            IF(NIT.EQ.2) THEN
              A1(I,N)=A1R(I,N)*(1.0+PRC)
            ELSE  IF(NIT.EQ.3) THEN
              A1(I,N)=A1R(I,N)*(1.0-PRC)
            ELSE IF(NIT.EQ.4)THEN
              AA=(1.0+PRC)*A2R(I,N)/(1.0-A2R(I,N))
              A2(I,N)=AA/(1.0+AA)
              AA=(QR**(1.0-A2R(I,N)))/(A1R(I,N)*A2R(I,N))
              A1(I,N)=(QR**(1.0-A2(I,N)))/(AA*A2(I,N))
            ELSE IF(NIT.EQ.5)THEN
              AA=(1.0-PRC)*A2R(I,N)/(1.0-A2R(I,N))
              A2(I,N)=AA/(1.0+AA)
              AA=(QR**(1.0-A2R(I,N)))/(A1R(I,N)*A2R(I,N))
              A1(I,N)=(QR**(1.0-A2(I,N)))/(AA*A2(I,N))
            ELSE IF(NIT.EQ.6) THEN
              W1(I,N)=W1R(I,N)*(1.0+PRC)
            ELSE IF(NIT.EQ.7) THEN
              W1(I,N)=W1R(I,N)*(1.0-PRC)
            ELSE IF(NIT.EQ.8)THEN
              AA=(1.0+PRC)*W2R(I,N)/(1.0-W2R(I,N))
              W2(I,N)=AA/(1.0+AA)
              W1(I,N)=(W1R(I,N)*QR**W2R(I,N))/(QR**W2(I,N))
            ELSE IF(NIT.EQ.9)THEN
              AA=(1.0-PRC)*W2R(I,N)/(1.0-W2R(I,N))
              W2(I,N)=AA/(1.0+AA)
              W1(I,N)=(W1R(I,N)*QR**W2R(I,N))/(QR**W2(I,N))
            END IF
            PX(I,N)=X(I,N)
            F(I,N)=FI(I,N)
            AA=AO(I,N)+A1(I,N)*(FI(I,N)**A2(I,N))
            V(I,N)=AA*(X(I+1,N)-X(I,N))
            IF(I.GT.1)TF(I,N)=FI(I,N)-FI(I-1,N)
  120   CONTINUE
C
C       start time loop
        DO 400 J=1,NHR
C         set boundary and junction values
          DO 130 N=1,NBRCH
            NCD(N)=1
            DO 130 I=1,NXSEC(N)
              TRB(I,N)=BC(I,N,J)
  130     CONTINUE
          DO 140 N=1,NJNCT
            JCD(N)=2
  140     CONTINUE
C
C         route branches
          DO 200 NN=1,NBRCH
            N=0
            IERR=0
  150       CONTINUE
C             looking for something to route?
              N=N+1
              IF(JCD(JNCU(N)).EQ.1)GO TO 150
              IF(NCD(N).EQ.0)GO TO 150
            CALL ROUTE
     I      (AO(1,N),A1(1,N),A2(1,N),SL(1,N),DTS,IDBG,IERR,J,LUOUT,N,
     M      NXSEC(N),NS(N),F(1,N),PX(1,N),QI,TRB(1,N),TF(1,N),
     N      VI,W1(1,N),W2(1,N),X(1,N))
            IF(IERR.GT.20)GO TO 440
            IF(IERR.EQ.1)IERRR=1
C
C           update junction flows and codes
            NCD(N)=0
            JN=JNCD(N)
            JCD(JN)=0
            QJ=0.0
            AQ(1,N)=TF(1,N)
            INX=NXSEC(N)-1
            DO 160 I=1,INX
              XL=X(I,N)
              XR=X(I+1,N)
              VO=V(I,N)
              CALL FKAI(K,NS(N),PX(1,N),X(I,N))
              CALL FVOL(AO(1,N),A1(1,N),A2(1,N),K,NS(N),NXSEC(N),F(1,N),
     #                  PX(1,N),TF(1,N),V(I,N),X(1,N),XL,XR)
              AQ(I+1,N)=AQ(I,N)+TF(I+1,N)+(VO-V(I,N))/DTS
              IF(IDBG.EQ.1) THEN
C               debut output
                WRITE(LUOUT,*)'VO,V(I,N),AQ',VO,V(I,N),AQ(I+1,N)
              END IF
              IF(AQ(I+1,N).LT.0.0)THEN
C               can't have negagive flow
                II=I+1
                AQ(II,N)=0.0
                WRITE(LUOUT,4010)AQ(II,N),J,NN,II
              END IF
  160       CONTINUE
            DO 170 N1=1,NBRCH
              IF(JNCD(N1).EQ.JN) THEN
                IF(NCD(N1).NE.0) JCD(JN)=1
                QJ=QJ+AQ(NXSEC(N1),N1)
              END IF
  170       CONTINUE
            IF(JCD(JN).EQ.1)GO TO 200
              DO 180 N1=1,NBRCH
                IF(JNCU(N1).NE.JN) GO TO 180
                TRB(1,N1)=QJ*PF(N1)
  180         CONTINUE
  200     CONTINUE
C
          TIME=FLOAT(J)*DT-DT/2.0
          AA=TIME-DT
          IF(TIME.GE.TOBS(NRMS).AND.AA.LT.TOBS(NRMS))THEN
C           ****** Compute error ***********
            AA=QOLD+(AQ(NG,NB)-QOLD)*(TOBS(NRMS)-(TIME-DT))/DT
            AA=QOBS(NRMS)-AA
C           WRITE(LUOUT,*)'N,COMP, OBS, ERR ',NRMS,AA,QOBS(NRMS),AA
            NRMS=NRMS+1
            SR=SR+AA
            SRR=SRR+AA*AA
            IF((NRMS).GT.NOBS)GO TO 410
          END IF
          QOLD=AQ(NG,NB)
  400   CONTINUE

  410   SRR=SQRT(SRR/(FLOAT(NRMS)))
        IF(IERRR.EQ.1)WRITE(*,4020)
        IF(NIT.EQ.1)THEN
          WRITE(*,4030)PRC,SRR
          WRITE(*,*)'                           RMS error caused by'
          WRITE(*,*)'Coefficient =            decrease       increase '
        ELSE IF(NIT.EQ.2)THEN
          RMSR=SRR
        ELSE IF(NIT.EQ.3)THEN
          RMSL=SRR
          WRITE(*,*)'A1(1,1) = ',A1R(1,1),RMSL,RMSR
        ELSE IF(NIT.EQ.4)THEN
          RMSR=SRR
        ELSE IF(NIT.EQ.5)THEN
          RMSL=SRR
          WRITE(*,*)'A2(1,1) = ',A2R(1,1),RMSL,RMSR
        ELSE IF(NIT.EQ.6)THEN
          RMSR=SRR
        ELSE IF(NIT.EQ.7)THEN
          RMSL=SRR
          WRITE(*,*)'W1(1,1) = ',W1R(1,1),RMSL,RMSR
        ELSE IF(NIT.EQ.8)THEN
          RMSR=SRR
        ELSE IF(NIT.EQ.9)THEN
          RMSL=SRR
          WRITE(*,*)'W2(1,1) = ',W2R(1,1),RMSL,RMSR
        END IF
  420 CONTINUE
C
      WRITE(*,*)' Enter variable to change (1=A1,2=A2,3=W1,4=W2,',
     $          ' or q to Quit - '
      READ(*,'(I10)',ERR=450)NN
        WRITE(*,*)' Enter percent change (+10%=1.1, -10%=0.9), CH = '
        READ(*,*)XL
        DO 430 N=1,NBRCH
          INX=NXSEC(N)-1
          DO 430 I=1,INX
            IF(NN.EQ.1)THEN
              A1R(I,N)=A1R(I,N)*XL
            ELSE IF(NN.EQ.2)THEN
              XR=(QR**(1.0-A2R(I,N)))/(A1R(I,N)*A2R(I,N))
              AA=XL*A2R(I,N)/(1.0-A2R(I,N))
              A2R(I,N)=AA/(1.0+AA)
              A1R(I,N)=(QR**(1.0-A2R(I,N)))/(XR*A2R(I,N))
            ELSE IF(NN.EQ.3)THEN
              W1R(I,N)=W1R(I,N)*XL
            ELSE IF(NN.EQ.4)THEN
              XR=W1R(I,N)*(QR**W2R(I,N))
              AA=XL*W2R(I,N)/(1.0-W2R(I,N))
              W2R(I,N)=AA/(1.0+AA)
              W1R(I,N)=XR/(QR**W2R(I,N))
            END IF
  430   CONTINUE
      GO TO 100
C
  440 CONTINUE
      IF(IERR.EQ.21)WRITE(LUOUT,4000)N,J
C
  450 DO 460 N=1,NBRCH
        WRITE(LUOUT,2020)N,NXSEC(N),PF(N),JNCU(N),JNCD(N)
        WRITE(LUOUT,2030)
        DO 460 I=1,NXSEC(N)
          AA=X(I,N)/XFACT
          IF(I.LT.NXSEC(N))THEN
            WRITE(LUOUT,1050)I,AA,IOUT(I,N),FI(I,N),A1R(I,N),
     #                       A2R(I,N),AO(I,N),SL(I,N),W1R(I,N),W2R(I,N)
          ELSE
            WRITE(LUOUT,1050)I,AA,IOUT(I,N)
          END IF
  460 CONTINUE
C
C     close files
      CLOSE(LUIN)
      ENDFILE(LUOUT)
      CLOSE(LUOUT)
C
      STOP
      END
