C
C
C
      PROGRAM   FLWPLT
C
C     + + + PURPOSE + + +
C     Build a file which can be imported to plot flow data.
C     Can also estimate un-gaged inflow and plot.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER I,IC,IENG,II,IO,K,J,LUIN,LUOBS,LUOUT,M,MP,
     &        N,NB,NG,NP,IO1,LOS(2),NC
      REAL    AA,A1,A2,CEL,DST,DT,DX,DY,ER,FLOW(4),SR,
     &        SRR,XC,X(4000),XMAX,XMIN,XO(4000),Y(4000),YMAX,
     &        YMIN,YO(4000)
      CHARACTER*20 LEGEND
      CHARACTER*40 YLABS(4,2), YLAB, XLAB, MLAB
      CHARACTER*64 VERSN
      CHARACTER*80 BLK
C
C     + + + LOCAL VARIABLE DEFINITIONS + + +
C     AA     - Scratch value
C     A1     - Coefficient in area vs discharge equation
C     A2     - Exponent in area vs discharge equation
C     BLK    - Alpha numeric data
C     CEL    - Wave celerity
C     DST    - Distance of ungaged inflow upstream of plot
C     DT     - time step size in hours
C     DX     - Tic mark increment on X
C     DY     - Tic mark increment on Y
C     ER     - Error in computed value
C     FLOW   - array containing flow field where:
C              (1)  for discharge 
C              (2)  for area 
C              (3)  for top width
C              (4)  for tributary inflow
C     IC     - observed flow indicator flag: 0-don't plot,1-plot
C     II     - ungaged inflow indicator flag: 0-don't compute,1-compute
C     IENG   - units code: 1-english, 0-metric
C     IO       Number of observed points
C     IO1    - Number of observed points within computational period
C     LOS    - line type: 1-line, 2-symbol
C              (1) - line 1, simulated
C              (2) - optional line 2, observed
C     MP     - flow variable to be plotted: 1-Q,2-area,3-width,4-inflow
C     NB     - branch number
C     NC     - number of curves to be plotted
C     NG     - grid number where flow is desired
C     NP     - number of points to be plotted
C     SR     - Sum of errors
C     SRR    - Sum of squares of errors
C     X(NP)  - x plotting position for point NP
C     XC     - scratch value
C     XMAX   - maximum value of time
C     XMIN   - minimum value of time
C     XO     - observed time
C     Y(NP)  - y plotting position for point NP
C     YLABS  - contains y-axis titles, the selected title depends
C              on flow field M and english or metric units (M,EoM)
C     YMAX   - maximum value of ordinate
C     YMIN   - minimum value of ordinate
C     YO     - observe y value
C
C     + + + INTRINSICS + + +
      INTRINSIC   FLOAT, SQRT, ABS
C
C     + + + EXTERNALS + + +
      EXTERNAL   DRWLIN, DRWSET, DRWCRV, DRWDON
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  LOS   / 1, 2 /
      DATA  YLABS / 'Discharge, in cu ft/s                   ',
     $              'Area, in sq feet                        ',
     $              'Top Width, in feet                      ',
     $              'Tributary discharge, in cu ft/s         ',
     $              'Discharge, in cu m/s                    ',
     $              'Area, in sq meters                      ',
     $              'Top width, in meters                    ',
     $              'Tributary discharge, in cu m/s          ' /
C
C     + + + INPUT FORMATS + + +
 1700 FORMAT (A)
 1701 FORMAT (I5,I5,I5,4E18.5)
C
C     + + + OUTPUT FORMATS + + +
 1600 FORMAT ('Flow in branch',I3,' grid',I3,' DT=',F7.3)
 1601 FORMAT ('Area in branch',I3,' subreach',I3,' DT=',F7.3)
 1602 FORMAT ('Top width in branch',I3,' subreach',I3,' DT=',F7.3)
 1603 FORMAT ('Tributary inflow to branch',I3,' grid',I3,' DT=',F7.3)
 1604 FORMAT (1X,A)
C
C     + + + INPUT/OUTPUT FORMATS + + +
 1605 FORMAT(4G12.4)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)FLWPLT - modified May  5, 1994, kmf'
      VERSN = '@(#)FLWPLT - modified Sep 19, 1995, kmf'
      VERSN = '@(#)FLWPLT - modified Apr  4, 1996, kmf'
      VERSN = '@(#)FLWPLT - modified Jul 24, 1996, hej'
C
C     set unit numbers and open required data files
      LUIN  = 35
      LUOUT = 36
      LUOBS = 37
      OPEN (LUIN,  FILE = 'bltm.flw' )
      OPEN (LUOUT, FILE = 'plt.out'  )
C
C     notice to user
      WRITE(*,*) ' This program reads "bltm.flw".'
      WRITE(*,*) ' Optional observed data read from "obs"'
      WRITE(*,*) ' It writes the file "plt.out"'
      WRITE(*,*) ' '
C
C     get branch #, grid #, time step, plot option, and units
      WRITE(*,*) ' Enter branch number where plot is desired  '
      READ (*,*) NB
      WRITE(*,*) ' Enter grid number where plot is desired  '
      READ (*,*) NG
      WRITE(*,*) ' Enter time step size, in hours  '
      READ (*,*) DT
      WRITE(*,*) ' Enter 1 to plot discharge at grid'
      WRITE(*,*) ' Enter 2 to plot cross-sectional area in subreach'
      WRITE(*,*) ' Enter 3 to plot top width in subreach'
      WRITE(*,*) ' Enter 4 to plot tributary inflow at grid   '
      READ (*,*) MP
      WRITE(*,*) ' Enter 1 for English units, 0 for metric.  '
      READ (*,*) IENG
C
C     start building plt.out file
      IF (MP.EQ.1) THEN
C       discharge at grid
        WRITE(LUOUT,1600) NB, NG, DT
      ELSE IF (MP.EQ.2) THEN
C       cross-sectional area in subreach
        WRITE(LUOUT,1601) NB, NG, DT
      ELSE IF (MP.EQ.3) THEN
C       top width in subreach
        WRITE(LUOUT,1602) NB, NG, DT
      ELSE IF (MP.EQ.4) THEN
C       tributary inflow at grid
        WRITE(LUOUT,1603) NB, NG, DT
      END IF
      WRITE(LUOUT,*)'   Time    Computed'
C
C     get simulated from bltm.flw and add to plt.out file
      NP = 0
      X(1)=0.0
  100 CONTINUE
        READ (LUIN,1701,END=110) J, N, I, (FLOW(M) , M = 1, 4)
          IF (N .EQ. NB .AND. I .EQ. NG) THEN
            IF (J .EQ. 0) THEN
C             initialize limits
              YMAX = FLOW(MP)
              YMIN = FLOW(MP)
            END IF
            NP = NP+1
            IF (J .GT. 0) X(NP) = DT * (FLOAT(J) - 0.5)
            Y(NP) = FLOW(MP)
            IF (YMAX .LT. Y(NP)) YMAX = Y(NP)
            IF (YMIN .GT. Y(NP)) YMIN = Y(NP)
            WRITE(LUOUT,1605) X(NP), Y(NP)
          END IF
      GO TO 100
  110 CONTINUE
      XMIN = X(1)
      XMAX = X(NP)
C
C     is observed data to be included?
      IC = 0
      NC = 1
      WRITE(*,*)' Enter 1 to plot observed, q otherwise  '
      READ (*,'(I10)',ERR=210) I
        IC = I
  210 CONTINUE
      IF(IC .EQ. 1) THEN
C       observed data
        NC = 2
        OPEN (LUOBS,FILE='obs')
        IO = 0
        II = 0
        WRITE(*,*) ' Enter 1 to estimate un-gaged inflow, q otherwise '
        READ (*,'(I10)',ERR=220) I
C         estimate ungaged inflow
          II = I
          WRITE(*,*) ' Enter distance upstream to ungaged inflow',
     $               ' (miles)  '
          READ (*,*) DST
          WRITE(*,*) ' Enter A1  '
          READ (*,*) A1
          WRITE(*,*) ' Enter A2  '
          READ (*,*) A2
  220   CONTINUE
        READ (LUOBS,1700) BLK
        WRITE(LUOUT,1604) BLK
        READ (LUOBS,1700) BLK
        WRITE(LUOUT,1604) BLK
        IF (II .EQ. 1) THEN
          WRITE(LUOUT,*) ' Time in    un-gaged    Flow Obs    ',
     $                   'Time Obs'
        ELSE
          WRITE(LUOUT,*) '   Time    Flow Obs      error'
        END IF
  230   CONTINUE
          READ (LUOBS,*,END=240) XC, AA
          IO = IO + 1
          XO(IO) = XC
          IF (XMAX .LT. XC) XMAX = XC
          YO(IO) = AA
          IF (YMAX .LT. AA) YMAX = AA
          IF (YMIN .GT. AA) YMIN = AA
        GO TO 230
  240   CONTINUE
C
C       compute the RMS
        SR  = 0.0
        SRR = 0.0
        IO1 = 0
        DO 320 I = 1, IO
          N = 1
  310     N = N+1
          IF (N .LE. NP) THEN
C           still room for another point data value to be included
            IF (X(N) .LT. XO(I)) GO TO 310
            IO1 = IO1 + 1 
            AA  = Y(N-1)+(Y(N)-Y(N-1))*(XO(I)-X(N-1))/(X(N)-X(N-1))
            ER  = YO(I) - AA
            IF (II .EQ. 0) THEN
              WRITE(LUOUT,1605) XO(I), YO(I), ER
            ELSE
              CEL = (YO(I) ** (1.0 - A2)) / (A1 * A2)
              IF (IENG .EQ. 0) CEL = CEL / 0.3048
              AA = DST * 5280.0 / (CEL * 3600.0)
              AA = XO(I) - AA
              WRITE(LUOUT,1605) AA, ER, YO(I), XO(I)
            END IF
            SR  = SR + ER
            SRR = SRR + ER * ER
          END IF
  320   CONTINUE
        SR  = SR / ( FLOAT ( IO1 ) )
        SRR = SQRT( SRR / ( FLOAT( IO1 ) ) )
        WRITE(LUOUT,*) ' Mean Error=',SR,' RMS=',SRR,
     $                 ' based on',IO1,' Points'
      END IF
C
C     get axes limits from user
      CALL DRWLIM ( XMIN, DX, XMAX, YMIN, DY, YMAX )
C
C     draw and label the axes
      XLAB = 'Time, in hours'
      IF (IENG .EQ. 1) THEN
C       english units
        YLAB = YLABS(MP,1)
      ELSE
C       assume metric units
        YLAB = YLABS(MP,2)
      END IF
      MLAB = 'Grid'
      CALL DRWSET ( XLAB, XMIN, DX, XMAX,
     I              YLAB, YMIN, DY, YMAX,
     I              MLAB, NG, NC )
C
C     simulated curve (solid line, no symbol, black)
      LEGEND = 'Simulated'
      CALL DRWCRV (X,XMAX,XMIN,Y,YMAX,YMIN,NP, LEGEND, 1, 1, 1 )
      IF (IC .EQ. 1) THEN
C       observed curve (no line, o symbol, black)
        LEGEND = 'Observed'
        CALL DRWCRV (XO,XMAX,XMIN,YO,YMAX,YMIN,IO, LEGEND, 2, 2, 1 )
      END IF
C
C     close plot
      CALL DRWDON
C
C     close output files
      CLOSE   ( LUIN  )
      ENDFILE ( LUOUT )
      CLOSE   ( LUOUT )
      IF (IC .EQ. 1) CLOSE ( LUOBS )
C
      STOP
      END
