C
C
C
      SUBROUTINE   DRWLIM
     M                    ( XMIN, DX, XMAX, YMIN, DY, YMAX )
C
C     + + + PURPOSE + + +
C     Let user modify the limits and tic interval for the x and y axis.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL   XMIN, DX, XMAX, YMIN, DY, YMAX
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XMIN   - minimum value for x-axis
C     DX     - tic mark interval for x-axis
C     XMAX   - maximum value for x-axis
C     YMIN   - minimum value for y-axis
C     DY     - tic mark interval for y-axis
C     YMAX   - maximum value for y-axis
C
C     + + + INTRINSICS + + +
      INTRINSIC   ABS
C
C     + + + END SPECIFICATIONS + + +
C
C     get x-axes limits from user
      WRITE (*,*) ' The maximum x is ',XMAX
      WRITE (*,*) ' Enter Xmax - '
      READ  (*,*) XMAX
      WRITE (*,*) ' The minimum x is ',XMIN
      WRITE (*,*) ' Enter Xmin - '
      READ  (*,*) XMIN
      WRITE (*,*) ' Enter DX - '
      READ  (*,*) DX
C
C     get y-axes limits from user
      WRITE (*,*) ' The maximum ordinate is ',YMAX
      WRITE (*,*) ' Enter Ymax - '
      READ  (*,*) YMAX
      WRITE (*,*) ' The minimum ordinate is ',YMIN
      WRITE (*,*) ' Enter Ymin - '
      READ  (*,*) YMIN
      WRITE (*,*) ' Enter DY - '
      READ  (*,*) DY
C     have had problems with very small y going to negative
      IF (ABS(YMIN) .LT. 0.00001) YMIN = 0.0000001
C
      RETURN
      END
C
C
C
      SUBROUTINE   DRWSET
     I                    ( XLAB, XMIN, DX, XMAX,
     I                      YLAB, YMIN, DY, YMAX,
     I                      MLAB, NG, NC )
C
C     + + + PURPOSE + + +
C     Ask user for graphics device.  Draw and label the axes.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       NG, NC
      REAL          DX, DY, XMIN, XMAX, YMIN, YMAX
      CHARACTER*(*) XLAB, YLAB, MLAB
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XLAB   - label for x-axis
C     XMIN   - minimum value for x-axis
C     DX     - tic mark interval for x-axis
C     XMAX   - maximum value for x-axis
C     YLAB   - label for y-axis
C     YMIN   - minimum value for y-axis
C     DY     - tic mark interval for y-axis
C     YMAX   - maximum value for y-axis
C     MLAB   - plot title
C     NG     - identification number
C     NC     - number of curves to be plotted
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       AGAIN, IOUT, IXARG, IBUF(16), JBUF(16), NDX
      CHARACTER*100 LAB
C
C     + + + EQUIVALENCE + + +
      CHARACTER*30 PSFILE
      EQUIVALENCE  (JBUF(1), PSFILE)
C
C     + + + INTRINSICS + + +
      INTRINSIC  INDEX
C
C     + + + EXTERNALS + + +
      EXTERNAL  XWNDOW, TEKALL, PSCRPT, IOMGR
      EXTERNAL  RESET,  PAGE,   AREA2D
      EXTERNAL  XNAME,  YNAME,  MESSAG, INTNO, GRAF
C
C     + + + FORMATS + + +
    5 FORMAT ( A30 )
C
C     + + + END SPECIFICATIONS + + +
C
      AGAIN = 0
  100 CONTINUE
        WRITE(*,*) ' Select output device, ',
     #             '1=DG screen, 2=postscript, 3=Mac screen - '
        READ(*,*) IOUT
        IF (IOUT .EQ. 1) THEN
C         x window
          CALL XWNDOW ( 1, IXARG, 10 )
        ELSE IF (IOUT .EQ. 3) THEN
C         tektronix window
          CALL TEKALL ( 4014, 960, 0, 1, 0 )
        ELSE IF (IOUT .EQ. 2) THEN
C         postscript file
          IBUF(1)=5
          CALL IOMGR(IBUF,-102)
          WRITE (*,*) ' ENTER POSTSCRIPT FILE NAME:'
          READ(5,5) PSFILE
          CALL IOMGR(JBUF,-103)
          IBUF(1)=1
          CALL IOMGR(IBUF,-104)
          WRITE (*,*) ' WRITING POSTSCRIPT FILE'
          CALL PSCRPT(0,0,0)
        ELSE
C         invalid response
          WRITE (*,*) ' Invalid device, reenter'
          AGAIN = 1
        END IF
C       if wrong device type, reenter
      IF (AGAIN .EQ. 1) GO TO 100
C
C     set up page
      CALL RESET  ( 'ALL' )
      CALL PAGE   ( 11.0, 8.5 )
      CALL AREA2D ( 10.0, 7.0 )
C
C     x-axis title
      NDX = INDEX ( XLAB, '   ' )
      LAB = XLAB(1:NDX)//'$'
      CALL XNAME  ( LAB, 100 )
C
c     y-axis title
      NDX = INDEX ( YLAB, '   ' )
      LAB = YLAB(1:NDX)//'$'
      CALL YNAME  ( LAB, 100 )
C
C     draw axes
      CALL GRAF   ( XMIN, DX, XMAX, YMIN, DY, YMAX )
C
C     plot title
      NDX = INDEX ( MLAB, '   ' )
      LAB = MLAB(1:NDX)//'$'
      CALL MESSAG ( LAB, 100, 7.0, 6.0 )
      CALL INTNO  ( NG, 'ABUT', 'ABUT' )
C
      RETURN
      END
C
C
C
      SUBROUTINE   DRWCRV
     I                    ( X, XMAX, XMIN, Y, YMAX, YMIN, NPTS,
     I                      LEGEND, LNTYPE, LNSYMB, LNCOLR )
C
C     + + + PURPOSE + + +
C     Add a curve to the plot.
C     NOTE:  The symbol type and line color (LNSYMB and LNCOLR) are
C     not currently supported in the display version of drwcrv, but
C     our included for consistency with other versions of drwcrv.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      NPTS, LNTYPE, LNSYMB, LNCOLR
      REAL         X(NPTS), XMAX, XMIN, Y(NPTS), YMAX, YMIN
      CHARACTER*20 LEGEND
C
C     + + + ARGUMENT DEFINITIONS + + +
C     X      - x coordinates of curve
C     XMAX   - maximum for x-axis
C     XMIN   - minimum for x-axis
C     Y      - y coordinates of curve
C     YMAX   - maximum for y-axis
C     YMIN   - minimum for y-axis
C     NPTS   - number of points to be drawn
C     LNTYPE - line type
C              1 - connect points with solid line
C              2 - don't connect points
C     LNSYMB - symbol type
C              1 - no symbol
C              2 - symbol
C     LNCOLR - line and symbol color
C              1 - black
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   LINE(2)
C
C     + + + LOCAL DEFINITIONS + + +
C     LINE   - array of appropriate indexes for requested line type
C              (1) solid line
C              (2) no line
C
C     + + + EXTERNALS + + +
      EXTERNAL   CURVE
C
C     + + + DATA INITIALIZATIONS + + +
C        line type: line symbol
      DATA   LINE /   0,   -1  /
C
C     + + + END SPECIFICATOINS + + +
C
C     draw the curve
      CALL CURVE ( X, Y, NPTS, LINE(LNTYPE) )
C
      RETURN
      END
C
C
C
      SUBROUTINE   DRWDON
C
C     + + + PURPOSE + + +
C     Adds title identifying the plot.  The integer NG is appended
C     to the end of the text string.  Ends and closes plot.  The
C     'end' of the text string is assumed to occur after the last
C     character in the string or after the occurence of 3 blanks.
C
C     + + + EXTERNALS + + +
      EXTERNAL   ENDPL, DONEPL
C
C     + + + END SPECIFICATION + + +
C
      CALL ENDPL  ( 0 )
      CALL DONEPL
C
      RETURN
      END
