C
C
C
      SUBROUTINE   DRWLIM
     M                    ( XMIN, DX, XMAX, YMIN, DY, YMAX )
C
C     + + + PURPOSE + + +
C     Let user modify the limits and tic interval for the x and y axis.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL   XMIN, DX, XMAX, YMIN, DY, YMAX
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XMIN   - minimum value for x-axis
C     DX     - tic mark interval for x-axis
C     XMAX   - maximum value for x-axis
C     YMIN   - minimum value for y-axis
C     DY     - tic mark interval for y-axis
C     YMAX   - maximum value for y-axis
C
C     + + + INTRINSICS + + +
      INTRINSIC   ABS
C
C     + + + END SPECIFICATIONS + + +
C
C     get x-axes limits from user
      WRITE (*,*) ' The maximum x is ',XMAX
      WRITE (*,*) ' Enter Xmax - '
      READ  (*,*) XMAX
      WRITE (*,*) ' The minimum x is ',XMIN
      WRITE (*,*) ' Enter Xmin - '
      READ  (*,*) XMIN
      WRITE (*,*) ' Enter DX - '
      READ  (*,*) DX
C
C     get y-axes limits from user
      WRITE (*,*) ' The maximum ordinate is ',YMAX
      WRITE (*,*) ' Enter Ymax - '
      READ  (*,*) YMAX
      WRITE (*,*) ' The minimum ordinate is ',YMIN
      WRITE (*,*) ' Enter Ymin - '
      READ  (*,*) YMIN
      WRITE (*,*) ' Enter DY - '
      READ  (*,*) DY
C     have had problems with very small y going to negative
      IF (ABS(YMIN) .LT. 0.00001) YMIN = 0.0000001
C
      RETURN
      END
C
C
C
      SUBROUTINE   DRWSET
     I                    ( XLAB, XMIN, DX, XMAX,
     I                      YLAB, YMIN, DY, YMAX,
     I                      MLAB, NG, NC )
C
C     + + + PURPOSE + + +
C     Ask user for graphics device.  Set up axes.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       NG, NC
      REAL          DX, DY, XMIN, XMAX, YMIN, YMAX
      CHARACTER*(*) XLAB, YLAB, MLAB
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XLAB   - label for x-axis
C     XMIN   - minimum value for x-axis
C     DX     - tic mark interval for x-axis
C     XMAX   - maximum value for x-axis
C     YLAB   - label for y-axis
C     YMIN   - minimum value for y-axis
C     DY     - tic mark interval for y-axis
C     YMAX   - maximum value for y-axis
C     MLAB   - plot title
C     NG     - identification number
C     NC     - number of curves to be drawn
C
C     + + + COMMONS + + +
      INCLUDE   'cdrwcv.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       PSFILE, IOS, DEVTYP(5), DEVCD(5),
     $              L3, L240, LN, JUST, XTYP, YTYP(2)
      REAL          ALEN
      CHARACTER*64  PSNAME
C
C     + + + FUNCTIONS + + +
      INTEGER    LENSTR
C
C     + + + EXTERNALS + + +
      EXTERNAL   LENSTR, INTCHR
      EXTERNAL   GBOPEN, GPINIT, GPMTFL, GPDEVC,  GPLBXB, GPLABL
C
C     + + + DATA INITIALIZATIONS + + +
C                             postscript  ------- cgm ---------
C                    xwindow  b&w  color  clear_txt bin_encoded
      DATA  DEVTYP /     1,     2,   2,       4,       4 /
      DATA  DEVCD  /  1100,   102,  101,     25,      24 /
      DATA  L3, L240, JUST, XTYP, YTYP, ALEN
     $     / 3,  240,    1,    1,  1,0,  0.0 /
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT ( A64 )
C
C     + + + END SPECIFICATIONS + + +
C
 100  CONTINUE
        WRITE (*,*) ' Available output devices:'
        WRITE (*,*)
        WRITE (*,*) '    1 - unix screen'
        WRITE (*,*) '    2 - postscript, black & white'
        WRITE (*,*) '    3 - postscript, color'
        WRITE (*,*) '    4 - cgm file, clear text'
        WRITE (*,*) '    5 - cgm file, binary encoded'
        WRITE (*,*)
        WRITE (*,*) ' Enter number for the plot device - '
        READ (*,*) IOUT
C       if invalid device, try again
      IF (IOUT .LT. 1  .OR.  IOUT .GT. 5) GO TO 100
C
C     open GKS and error file for GKS
      CALL GBOPEN ( FE )
C
C     initialize graphics common block
      CALL GPINIT
C
      IF (IOUT .GT. 1) THEN
C       need to open an output file and pass unit to common block
        IF (IOUT .EQ. 2  .OR.  IOUT .EQ. 3) THEN
C         postscript file
          WRITE (*,*) ' Enter postscript file name:'
        ELSE IF (IOUT .EQ. 4  .OR.  IOUT .EQ. 5) THEN
C         cgm file
          WRITE (*,*) ' Enter cgm file name:'
        END IF
        READ (*,1000) PSNAME
        PSFILE = 97
        OPEN ( UNIT = PSFILE,
     $         FILE = PSNAME,
     $       ACCESS = 'SEQUENTIAL',
     $       STATUS = 'UNKNOWN',
     $         FORM = 'FORMATTED',
     $       IOSTAT = IOS )
        CALL GPMTFL ( PSFILE )
      END IF
C
C     set up graphics device
      CALL GPDEVC ( DEVTYP(IOUT), DEVCD(IOUT) )
C
C     initially, fake it out with MAX/2 curves
      NCRV = NC
      NVAR = NC * 2
      CALL GPNCRV ( NCRV, NVAR )
C
C     set labels and titles
      XLABB = XLAB
      YLABL = YLAB
      YLABR = ' '
      YLABA = ' '
      TITLE = MLAB
      LN = LENSTR ( L240, TITLE )
      CALL INTCHR ( NG, L3, JUST, LN, TITLE(LN+2:LN+5) )
      CALL GPLBXB ( XLABB )
      CALL GPLABL ( XTYP, YTYP, ALEN, YLABL, YLABR, YLABA, TITLE )
C
C     set scale min & max, tics, and option of off scale values
      IF (ABS(YMIN) .LT. .000001) YMIN = 0.0
      TICS(1) = (YMAX - YMIN) / DY
      TICS(4) = (XMAX - XMIN) / DX
      PLMN(1) = YMIN
      PLMX(1) = YMAX
      PLMN(4) = XMIN
      PLMX(4) = XMAX
C
      RETURN
      END
C
C     
C
      SUBROUTINE   DRWCRV
     I                   ( X, XMAX, XMIN, Y, YMAX, YMIN, NPTS,
     I                     LEGEND, LNTYP, LNSYMB, LNCOLR )
C
C     + + + PURPOSE + + +
C     Add a curve to the plot.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      NPTS, LNTYP, LNSYMB, LNCOLR
      REAL         X(NPTS), XMAX, XMIN, Y(NPTS), YMAX, YMIN
      CHARACTER*20 LEGEND
C
C     + + + ARGUMENT DEFINITIONS + + +
C     X      - x coordinates of curve
C     XMAX   - maximum for x-axis (not used, included for consistency)
C     XMIN   - minimum for x-axis (not used, included for consistency)
C     Y      - y coordinates of curve
C     YMAX   - maximum for y-axis (not used, included for consistency)
C     YMIN   - minimum for y-axis (not used, included for consistency)
C     NPTS   - number of points to be drawn
C     LEGEND - curve legend
C     LNTYP  - line type
C              1 - solid                4 - dot
C              2 - none                 5 - dash
C              3 - dash
C     LNSYMB - symbol type
C              1 - none                 4 - . (dot)
C              2 - o (circle)           5 - + (plus)
C              3 - x (X)                6 - * (star)
C     LNCOLR - line and symbol color
C              1 - black                5 - cyan
C              2 - red                  6 - magenta
C              3 - green                7 - yellow
C              4 - blue
C
C     + + + COMMONS + + +
      INCLUDE   'cdrwcv.inc'
C
C     + + + SAVES + + +
      INTEGER   COUNT, INDX1, INDX2
      SAVE      COUNT, INDX1, INDX2
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   RETC, LINES(5), SMBLS(6), COLRS(7,5)
C
C     + + + LOCAL DEFINITIONS + + +
C     INDX1  - index number of the y array
C     INDX2  - index number of the x array
C     NCRV   - curve number for the line defined by x and y
C
C     + + + FUNCTIONS + + +
      REAL      SRMIN, SRMAX
C
C     + + + EXTERNALS + + +
      EXTERNAL   SRMIN,  SRMAX
      EXTERNAL   GPDATA, GPWCXY
C
C     + + + DATA INITIALIZATIONS + + +
      DATA    COUNT, INDX1, INDX2
     $      /     0,     -1,    0 /
      DATA    LINES,      SMBLS
     $      / 1,0,2,3,4,  0,4,5,1,2,3 /
C                                  postscript files       cg,
C                   screen,        b&w  color          clear-text    binary
      DATA  COLRS / 1,2,3,4,5,6,7, 7*1, 1,2,3,4,5,6,7, 0,2,3,4,5,6,7,
     $              0,2,3,4,5,6,7  /
C
C     + + + END SPECIFICATIONS + + +
C
C     set data for curve
      COUNT = COUNT + 1
      INDX1 = INDX1 + 2
      INDX2 = INDX2 + 2
      CALL GPDATA ( INDX1, NPTS, Y, RETC)
      CALL GPDATA ( INDX2, NPTS, X, RETC)
      CALL GPWCXY ( COUNT, INDX1, INDX2 )
C
C     set parameters for y-array of data
      LGNVAR(INDX1)= 'Y-axis              '
      WHICH(INDX1) = 1
      VARMAX(INDX1)= SRMAX ( NPTS, Y )
      VARMIN(INDX1)= SRMIN ( NPTS, Y )
C
C     set parameters for x-array of data
      LGNVAR(INDX2)= 'X-axis              '
      WHICH(INDX2) = 4
      VARMAX(INDX2)= SRMAX ( NPTS, X )
      VARMIN(INDX2)= SRMIN ( NPTS, X )
C
C     set parameters for x-y curve
      LGNCRV(COUNT) = LEGEND
      LNTYPE(COUNT) = LINES(LNTYP)
      COLOR(COUNT)  = COLRS(LNCOLR,IOUT)
      SYMBL(COUNT)  = SMBLS(LNSYMB)
C
      RETURN
      END
C
C
C
      SUBROUTINE   DRWDON

C     + + + PURPOSE + + +
C     Draw the plot and close it out
C
C     + + + COMMONS + + +
      INCLUDE   'cdrwcv.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   RETC, CMPTYP, PREC, FONT(4), RETC,
     $          CVTYP(MAX), PATTRN(MAX), TRNTYP(MAX),
     $          OFFSCL(4), WSID, ICLOS, IWAIT, META, FG
      REAL      CHXP, CHSP, RR, RED(2), GREEN(2), BLUE(2), 
     $          SIZEL, ALEN, XLEN, XPAGE, XPHYS, YLEN, YPAGE, YPHYS
C
C     + + + LOCAL DEFINITIONS + + +
C     CMPTYP - comuter type: 1-pc,2-prime,3-dec,4-sun,5-aviion
C     PREC   - precision code for test
C     FONT   - font for text
C     CHXP   - characer expansion factor
C     CHSP   - character spacing
C     RED    - fraction of red for background
C     GREEN  - fraction of green for background
C     BLUE   - fraction of blue for background
C     RR     - symbol size ratio
C     TRNTYP - transformation type
C              1 - arithmetic (none)
C              2 - logarithmetic
C     OFFSCL - action for off scale values
C              (1) left y, (2) right y, (3) aux, (4) x
C              1 - clip, plot at point going off scale
C              2 - skip, leave a blank if line drawn
C              3 - arrow pointing off scale, lines not connected
C              4 - ignore, lines connect good values
C     XTYP   - type of x-axis: 0-time,1-arithmetic,2-logarithmic
C     YTYP   - type of y-axis:
C              (1) left axis: 0-none, 1-arithmetic, 2-logarithmetic
C     ALEN   - length of auxiliary axis 0.0-no aux axis
C     
C     + + + EXTERNALS + + + 
      EXTERNAL   GPVAR,  GPCURV, GPSCLE, GPLABL, GPSIZE
      EXTERNAL   GSETUP, PLTONE, DNPLTW, GPCLOS
      EXTERNAL   GSCR
C
C     + + + DATA INITIALIZATIONS + + +
C                    xwindow, ps-b&w, mac, cgm
      DATA  FONT   /     1,  -2808,     1,   1 /
      DATA  CMPTYP, META, PREC, CHXP, CHSP, RR, WSID, ICLOS, IWAIT
     $     /     5,    0,    2,  1.0,  0.0, 1.0,   1,     1,     2 /
      DATA   FG, RED,     GREEN,   BLUE
     $     /  1, 0.0,1.0, 0.0,1.0, 0.0,1.0 /
      DATA  PATTRN, CVTYP, TRNTYP, OFFSCL
     $     / MAX*0, MAX*6,  MAX*1,    4*1 /
      DATA  ALEN, SIZEL, XPAGE, YPAGE, XPHYS, YPHYS, XLEN, YLEN
     $     / 0.0,   0.1,   8.0,   6.4,   1.5,   1.5,  4.4,  4.2 /
C
C     + + + END SPECIFICATIONS + + +
C
C     set min/max and which axis
      CALL GPVAR ( VARMIN, VARMAX, WHICH, TRNTYP, LGNVAR )
C
C     set curve parameters
      CALL GPCURV ( CVTYP, LNTYPE, SYMBL, COLOR, PATTRN, LGNCRV )
C
C     set scale min & max, tics, and option of off scale values
      CALL GPSCLE ( PLMN, PLMX, TICS, OFFSCL )
C
C     set up relative physical plot size
      CALL GPSIZE ( SIZEL, XPAGE, YPAGE, XPHYS, YPHYS,
     I              XLEN, YLEN, ALEN )
C
C     set up plotting area and open graphics device
      CALL GSETUP ( CMPTYP, META, PREC, FONT(IOUT), CHXP, CHSP,
     I              RED, GREEN, BLUE, RR,
     O              RETC )
C
      IF (IOUT .EQ. 4  .OR.  IOUT .EQ. 5) THEN
C       set foreground to black for cgm file
        CALL GSCR ( WSID, FG, RED(1), GREEN(1), BLUE(1) )
      END IF
C
C     make the plot
      CALL PLTONE
C
C     close the plot
      WSID  = 1
      ICLOS = 1
      IWAIT = 2
      CALL DNPLTW ( IWAIT, WSID, ICLOS, CMPTYP, META )
C
C     close out GKS
      CALL GPCLOS ( FE )
C
      RETURN
      END
