
C     **************** SOLAR RADIATION MODEL ******************
C
      PROGRAM   SOLAR
C
C     + + + PURPOSE + + +
C     Compute the solar radiation from empirical equations.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       IC,IIC,IP,IIP,LUOUT,J,JM
      REAL          ALAT, ALON, DEL, DJLN, PI, QE, R, T,
     $              TLON, DT,P,QC,QE,QO,AX,
     $              AP,APP,APW,ASD,ASR,ASS,BPW,BSD,CC,DIN,ELEV,
     $              HA1,HA2,OAM,PWS,QS,SALT,SD,SDM,SR,SS,
     $              T1,T2,TD,TLHA,TPI,TSR,TSS
      CHARACTER*64  VERSN
      CHARACTER*80  TITLE
C
C     + + + LOCAL DEFINITIONS + + +
C     ALAT   - Latitude of point in question (degrees)
C     ALON   - Longitude of point n question (degrees)
C     DEL    - Declination of the sun, in radians
C     DJLN   - Julian day
C     DT     - Time step size in model, in hoursI
C     HA     - Local hour angle (angle from true solar noon)
C     PI     - 3.14159
C     QE     - Solar constant
C     R      - dimensionless radius vector of earth-sun distance
C     T      - Time, in hours
C     TLON   - Longitude of local time meridian (degrees)
C     TITLE  - Title to placed on data file
C
C     + + + INTRINSICS + + +
      INTRINSIC   ACOS, ASIN, COS, EXP, IFIX, SIN
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT ( A80 )
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT(11F7.2)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)SOLAR - last modified MAY  3, 1996, kmf'
C
C     open external files
      LUOUT=16
      OPEN(LUOUT,FILE='out')
C
C     zero arrays and priliminaries
      PI=3.14159
C
C     read input data
      WRITE(*,*) ' Enter time step size, in hours  '
      READ (*,*) DT
      WRITE(*,*) ' Enter title for data file (80 character max)  '
      READ(*,1000) TITLE
      WRITE(LUOUT,1000) TITLE
      WRITE(LUOUT,*)'  Time   Lang  Clear  Extra'
      WRITE(*,*) ' To select default enter any letter.  '
      QE=2.0
      WRITE(*,*) ' Enter solar constant in cal/(sq cm min)'
      WRITE(*,*) ' Default=',QE,'  QE (cal/(sq cm min)) = '
      READ(*,'(F9.0)',ERR=1)DIN
        QE=DIN
    1 CONTINUE
      WRITE(*,*) ' Enter julian day of start of model - '
      READ (*,*) DJLN
      WRITE(*,*) ' Enter longitude of point on in question',
     $           ' (xx.xx deg) - '
      READ (*,*) ALON
      WRITE(*,*) ' Enter longitude of local time meridian',
     $           ' (xx.xx deg) -'
      READ (*,*) TLON
      WRITE(*,*) ' Enter latitude of point in question',
     $           ' (xx.xx degrees) - '
      READ (*,*) ALAT
      ALAT=ALAT*2.0*PI/360.0
      ASR=0.0
      WRITE(*,*) ' Enter altitude of sunrise horizon (degrees)'
      WRITE(*,*) ' Default = 0 degrees, ASR = '
      READ (*,'(F9.0)',ERR=2) DIN
        ASR=DIN*PI/180.0
    2 CONTINUE
      ASS=0.0
      WRITE(*,*) ' Enter altitude of sunset horizon, default=0, ASS= '
      READ (*,'(F9.0)',ERR=3) DIN
        ASS=DIN*PI/180.0
    3 CONTINUE
      IP=0
      WRITE(*,*) ' Enter any letter to input elevation, or'
      WRITE(*,*) ' Enter 1 to input atmospheric pressure. '
      READ (*,'(I9.0)',ERR=4) IIP
        IP=IIP
    4 CONTINUE
      IF(IP.NE.0)GO TO 5
        WRITE(*,*) ' Enter elevation of river in feet - '
        READ (*,*) ELEV
        ELEV=ELEV*0.3048
    5 CONTINUE
      WRITE(*,*) '     Constants in equation for precipitable water'
      WRITE(*,*) '          Station                         a         b'
      WRITE(*,*) '    Greensboro, North Carolina       -0.975    0.0345'
      WRITE(*,*) '    Lake Charles, Louisiana          -0.930    0.0328'
      WRITE(*,*) '    Oklahoma City, Oklahoma          -0.956    0.0343'
      WRITE(*,*) '    Tampa, Flordia                   -1.378    0.0396'
      WRITE(*,*) '    Washington, D. C.                -0.948    0.0345'
      WRITE(*,*) '    Mean of 15 stations in US        -0.981    0.0342'
      APW=-0.981
      WRITE(*,*) ' Enter value of a, Default=',APW,' a= '
      READ (*,'(F9.0)',ERR=51) DIN
        APW=DIN
   51 CONTINUE
      BPW=0.0342
      WRITE(*,*) ' Enter value of b, Default=',BPW,' b= '
      READ (*,'(F9.0)',ERR=52) DIN
        BPW=DIN
   52 CONTINUE
      IC=0
      WRITE(*,*) ' Enter any letter to input cloud cover, or'
      WRITE(*,*) ' Enter 1 to input sunshine duration. '
      READ (*,'(I9.0)',ERR=6) IIC
        IC=IIC
    6 CONTINUE
      ASD=0.22
      BSD=0.78
      P=29.93
      TD=50.0
      CC=0.0
      SD=12.0
    8 CONTINUE
        R=1.0+0.017*COS(2.0*PI*(186.0-DJLN)/365.0)
        DEL=(23.45*PI*COS(2.0*PI*(172.0-DJLN)/365.0))/180.0
C       Compute time of sunrise and sunset.
        SR=ACOS((SIN(ASR)-SIN(ALAT)*SIN(DEL))/(COS(ALAT)*COS(DEL)))
        TSR=12.0-SR*12.0/PI+(ALON-TLON)/15.0
        SS=ACOS((SIN(ASS)-SIN(ALAT)*SIN(DEL))/(COS(ALAT)*COS(DEL)))
        TSS=12.0+SS*12.0/PI+(ALON-TLON)/15.0
        IF (IP .NE. 1) GO TO 9
           WRITE(*,*) ' Enter barometric pressure, in inches of mercury'
           WRITE(*,*) '   Default=',P,', P= '
           READ (*,'(F9.0)',ERR=9) DIN
             P=DIN
    9   CONTINUE
        WRITE(*,*) ' Enter dew point, in degree F '
        WRITE(*,*) '   Default=',TD,', TD= '
        READ (*,'(F9.0)',ERR=10) DIN
          TD=DIN
   10   CONTINUE
        IF (IC .EQ. 0) THEN
          WRITE(*,*) ' Enter cloud cover in tenths of sky covered - '
          WRITE(*,*) ' (1.0>CC>0.0) Default=',CC,', CC= '
          READ (*,'(F9.0)',ERR=11) DIN
            CC=DIN
   11     CONTINUE
        ELSE
          WRITE(*,*) ' Enter sunshine duration, in hours for the day - '
          WRITE(*,*) '   Default=',SD,', SD= '
          READ (*,'(F9.0)',ERR=12) DIN
            SD=DIN
   12     CONTINUE
          SDM=TSS-TSR
          WRITE(*,*) ' Enter maximum possible sunshine duration',
     $               ' for day - '
          WRITE(*,*) '   Default=',SDM,', SDM= '
          READ (*,'(F9.0)',ERR=13) DIN
            SDM=DIN
   13     CONTINUE
        END IF
        T=-DT/2.0
        JM=IFIX(24.0/DT)
        DO 15 J=1,JM
          T=T+DT
          T1=T-DT/2.0
          T2=T+DT/2.0
          QO=0.0
          QS=0.0
          QC=0.0
          IF (T2 .LE. TSR) GO TO 14
            IF (T1 .GE. TSS) GO TO 14
C             Compute extra-terrestrial solar radiation
              IF (T1 .LT. TSR) T1=TSR
              IF (T2 .GT. TSS) T2=TSS
              HA1=PI*(T1+12.0+(TLON-ALON)/15.0)/12.0
              TPI=2.0*PI
              IF (HA1 .GT. TPI) HA1=HA1-TPI
              HA2=PI*(T2+12.0+(TLON-ALON)/15.0)/12.0
              IF (HA2 .GT. TPI) HA2=HA2-TPI
              QO=12.0*COS(ALAT)*COS(DEL)*(SIN(HA2)-SIN(HA1))/PI
              QO=QE*(QO+(T2-T1)*SIN(ALAT)*SIN(DEL))/(R*R*DT)
C             Compute clear sky radiation
              TLHA=((T1+T2)*0.5+12.0+(TLON-ALON)/15.0)*PI/12.0
              SALT=ASIN(SIN(ALAT)*SIN(DEL)+COS(ALAT)*COS(DEL)*COS(TLHA))
              OAM=1.0/(SIN(SALT)+0.150*(SALT*180.0/PI+3.885)**(-1.253))
              IF (IP .EQ. 0) THEN
                OAM=OAM*(((288.0-0.0065*ELEV)/288.0)**5.256)
              ELSE
                OAM=OAM*P/29.93
              END IF
C             Compute precipitable water in the atmosphere
              PWS=EXP(APW+BPW**TD)
C             Compute A' and A"
              AX =0.465+0.134*PWS
              AP =EXP(-(AX)*(0.129+0.171*EXP(-0.88*OAM))*OAM)
              APP=EXP(-(AX)*(0.179+0.421*EXP(-0.721*OAM))*OAM)
              QS=QO*(APP+0.5-0.5*AP)
C             Compute radiation under cloudy sky
              IF (IC .EQ. 0) THEN
                QC=QS*(1.0-0.71*CC*CC)
              ELSE
                QC=QS*(ASD+BSD*((SD/SDM)**0.66667))
              END IF
   14     CONTINUE
          WRITE(LUOUT,2000)T,QC,QS,QO
   15   CONTINUE
        DJLN=DJLN+1
        WRITE(*,*)'Do you want another day? (0=no, y=yes) '
      READ(*,'(I9)',ERR=8)IIP
C
C     finished, close files
      ENDFILE ( LUOUT )
      CLOSE   (LUOUT  )
      WRITE(*,*)' Results writen in file "out"'
C
      STOP
      END
