C
C
C
      SUBROUTINE   FINK
     ?                 (K,MX,J,NEQ,PV,PT,
     ?                  Q,A,W,XK,S,CR,IRC,NNN,LUOUT)
C
C     + + + PURPOSE + + +
C     Simplified temperature model to calculate the constituent
C     coefficients.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOBR   - Maximum number of branches allowed in model
C     NOSC   - Maximum number of cross sections (grids) allowed in branch
C     NOPR   - Maximum number of parcels allowed in branch
C              (NOPR should be at least 20 + 2 times NOSC)
C     NOCO   - Maximum number of constituents allowed
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   K, MX, J, NEQ, IRC, NNN, LUOUT
      REAL      PV, PT(NOCO,NOPR), Q, A, W, XK(NOCO,NOCO), S(NOCO),
     #          CR(NOCO,NOCO)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     K      - Parcel number
C     MX     - Grid just upstream of parcel's upstream boundary
C     J      - Time step
C     NEQ    - Number of equations (constituents)
C     PV(K)  - Volume of parcel
C     PT(L,K)- Concentration of constituent L in parcel K, mg/l
C     Q      - Discharge, cfs or cms
C     A      - Average area of the subreach (sq m or sq ft)
C     W      - Average top width of the subreach, m or ft
C     XK(L,LL) Exchange coefficient for constituent L due to the
C              concentration of constituent LL
C     S(L)   - Source flux of constituent L, (units/hour)
C     CR(M,L)- Reference (equilibrium) concentration in equation M
C              for constituent L
C     IRC    - Code to set read for new data each time step
C              1=read data, 0=no read
C     NNN    - Branch number
C     LUOUT  - Fortran unit number for output
C
C     + + + COMMONS + + +
      COMMON  /CMBLTM/ NXSEC,NBRCH,IENG
      INTEGER   NXSEC(NOBR), NBRCH, IENG
      SAVE    /CMBLTM/
C
C     + + + COMMON DEFINITIONS + + +
C     NXSEC(N) Number of subreaches in branch N
C     NBRCH  - Number of branches N
C     IENG   - Input units: 0=metric, 1=English
C
C     + + + SAVES + + +
      REAL      AW,BW,CPR,SIG,TE,V
      SAVE      AW,BW,CPR,SIG,TE,V
C
C     + + + SAVE DEFINTITIONS + + +
C     AW     - Constant in wind function (mm/day Kpa)
C     BW     - Mass transfer coefficient (mm/day Kpa (m/s)
C     CPR    - Conversion constant, gm/(sq cm per unit depth)
C     SIG    - Stefan Boltzman constant for black body radiation
C     TE     - Equilibrium temperatue (Celsius)
C     V      - Wind speed (entered in mi/hr converted to M/s)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      LUQ2
      REAL         AA,AL,DAW,DBW,DFT ,PSI,TEB,XKX
      CHARACTER*64 VERSN
      CHARACTER*80 TITLE
C
C     + + + LOCAL DEFINITIONS + + +
C     AL     - Latent heat of vaporization, cal/gm
C     CR(M,L)- Equilibrium concentration in equation M for constituent L
C     DFT    - Slope of saturation vapor pressure curve, Kpa/C
C     IRC    - Code to set read for new data each time step
C     NNN    - Branch number
C     PSI    - Wind function, cm/hour Kpa = gm/hr Kpa
C     PT(L,K)- Concentration of constituent L in pracel K
C     TEB    - Absolute temperature, Celsius
C     U      - Average water velocity  in subreach
C     XKX    - Heat loss per unit area, time, degree C, cal/(sq cm hr C)
C
C     + + + INTRINSICS + + +
      INTRINSIC   EXP
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (A)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (/,30X,'Reaction Kinetics')
 2001 FORMAT (/,' TEMPERATURE')
 2002 FORMAT ('     Wind function =',F6.2,'+',F6.2,
     $        'V [mm/day Kpa when V in m/s].')
C
C     + + + END SPECIFICATIONS + + +
C
      VERSN = '@(#)TEMP FINK- last modified May  9, 1996, kmf'
C
      IF (IRC .EQ. 1) THEN
C       read data
        IF (J .EQ. 1) THEN
C         first time, open input file
          LUQ2 = 19
          OPEN (LUQ2,FILE='temp.in')
          WRITE(LUOUT,2000)
          WRITE(*,*) ' To select default enter any letter.  '
          READ (LUQ2,1000) TITLE
          READ (LUQ2,1000) TITLE
          AW = 3.01
          WRITE(*,*) ' Enter constant in wind function, '
          WRITE(*,*) ' Default=3.01 (mm/(day Kpa)) value = '
          READ (*,'(F10.0)',ERR=110) DAW
            AW=DAW
  110     CONTINUE
          BW = 1.13
          WRITE(*,*) ' Enter mass transfer coefficient,'
          WRITE(*,*) ' Default=1.13 (mm/(day Kpa(m/s))), value = '
          READ (*,'(F10.0)',ERR=120) DBW
            BW=DBW
  120     CONTINUE
          WRITE(LUOUT,2001)
          WRITE(LUOUT,2002) AW,BW
          IF (IENG .EQ. 0) THEN
            CPR=100.0
          ELSE IF (IENG .EQ. 1) THEN
            CPR=30.48
          END IF
          SIG= 1.171E-7/24.0
        END IF
        READ (LUQ2,*)AA,TE,V
      END IF
C
C     temperature (simplified temperature algorithm)
      AL=595.9-0.545*PT(1,K)
      PSI=(AW+BW*V)/(24.0*10.0)
      TEB=PT(1,K)+273.16
      DFT=1.1532E11*EXP(-4271.1/(PT(1,K)+242.63))
     #             /((PT(1,K)+242.63)**2)
      XKX=4.0*0.97*SIG*(TEB**3)+AL*PSI*(DFT+0.06)
      XK(1,1)=-XKX*W/(A*CPR)
      CR(1,1)=TE
C
      IRC=0
C
      RETURN
      END
