C
C
C
      PROGRAM   UNIT
C
C     + + + PURPOSE + + +
C     This program reads data from a unit values flat file written by ADAPS.
C     It is assumed that stage is written first then discharge.
C     The program allows stage or discharge to be used.
C     The input file is assumed to contain four lines of text at the top.
C     The rating table is assumed to be in a file called RT
C     The first entry must be at time zero for the model
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      ID,IDO,IM,MIN,MINO,IY,LH,LT,I,LHM,
     $             LUIN, LUPC, LUOBS
      REAL         DT,Q,RQ(99),RS(99),S,TIME,A1, A2
      CHARACTER*64 VERSN
      CHARACTER*80 BLK
C
C     + + + LOCAL DEFINITIONS + + +
C     DT     - change in time between data points in hours
C     IY     - year
C     IM     - month
C     ID     - day
C     IDO    - old day
C     MIN    - minute
C     MINO   - old minute
C     LH     - break point in rating table below observed stage
C     LT     - code for stage (LT=1) or discharge (LT=0)
C     Q      - discharge
C     RS     - stage break in rating table
C     RQ     - discharge break in ration table
C     S      - stage
C     TIME   - time since start of data in hours
C
C     + + + INTRINSICS + + +
      INTRINSIC   ALOG, EXP, FLOAT
C
C     + + + INPUT/OUTPUT FORMATS + + +
 3501 FORMAT(A)
C
C     + + + OUTPUT FORMATS + + +
 3600  FORMAT(3G12.4)
C
C     + + + END SPECIFICATIONS + + +
C
C     unix what information
      INCLUDE 'versn.inc'
      VERSN = '@(#)UNIT - last modified Mar 29, 1996, hej'
C
C     notice to users
      WRITE (*,*) ' This program reads "unit.in" & "rt" and',
     $            ' writes to "out"'
      WRITE (*,*) ' Enter q to quit, 1 to continue '
      READ  (*,'(I10)',ERR=900) I
C
C     set unit numbers and open required data files
      LUIN  = 35
      LUPC  = 36
      LUOBS = 37
      OPEN (LUIN, FILE='unit.in')
      OPEN (LUPC, FILE='out')
C
      READ  (LUIN,3501) BLK
      WRITE (LUPC,3501) BLK
      READ  (LUIN,3501) BLK
      WRITE (LUPC,3501) BLK
      READ  (LUIN,3501) BLK
      READ  (LUIN,3501) BLK
      WRITE (*,*) ' Be sure that the first entry in "unit.in" is',
     $            ' at 0 hours for the model.'
      WRITE (*,*) ' Enter 1 to read stage and compute the discharge',
     $            ' from a rating table'
      WRITE (*,*) '       0 or r to read discharge directly -  '
      READ  (*,'(I10)',ERR=150) LT
        IF (LT .NE. 0) THEN
C         read stage from rating table
          OPEN (LUOBS, FILE='rt')
          READ  (LUOBS,3501) BLK
          READ  (LUOBS,3501) BLK
          I=0
  110     CONTINUE
            I = I + 1
            READ  (LUOBS,*,END=120) RQ(I), RS(I)
          GO TO 110
  120     CONTINUE
          CLOSE (LUOBS)
          LHM=I-1
        END IF
  150 CONTINUE
C
      IDO = 0
  200 CONTINUE
        READ  (LUIN,*,END=300) IY, IM, ID, MIN, S, Q
        IF (IDO .EQ. 0) THEN
          TIME = 0.0
        ELSE
          DT = (FLOAT(MIN - MINO)) / 60.0
          IF (ID .NE. IDO) DT = DT + 24.0
          TIME = TIME + DT
        END IF
        MINO = MIN
        IDO  = ID
        IF (LT .EQ. 1) THEN
C         compute discharge from rating table
          LH = 1
  220     CONTINUE
            LH = LH + 1
            IF (LH .GT. LHM) THEN
              LH = LHM
              GO TO 240
            END IF
          IF (S .GT. RS(LH)) GO TO 220
  240     CONTINUE
          IF (LH .EQ. 2) THEN
            Q=RQ(LH-1)+(S-RS(LH-1))*(RQ(LH)-RQ(LH-1))/(RS(LH)-RS(LH-1))
          ELSE
            A1=(ALOG(S)-ALOG(RS(LH-1)))*(ALOG(RQ(LH))-ALOG(RQ(LH-1)))
            A2=ALOG(RS(LH))-ALOG(RS(LH-1))
            Q=EXP(ALOG(RQ(LH-1))+A1/A2) 
          END IF
        END IF
        WRITE (LUPC,3600)TIME,Q,S
      GO TO 200
C
C     close files
  300 CONTINUE
      CLOSE (LUIN)
      ENDFILE (LUPC)
      CLOSE (LUPC)
C
  900 CONTINUE
C
      STOP
      END
