#!/bin/sh
#
#    check.sh:  Compare (using diff) 'original' versions of files to
#               files in current directory.  Used most often to compare
#               expected files to actual test output.
#   arguments:  DirO - directory containing original, expected versions
#                      defaults to ../data
#
#     history:  96/04/10 kmflynn  modified version of check.sh

  DirO=${1-../data}             # by default, data in parallel directory

# define range of tests and sufixes to be considered
  Tests='1 _F1 _F3 _D1 _W1 _T1 _Q2a _Q2b _Q2c'
  Sufix='in lg fl ot pl ps bc bl'
  Steps='1 2 3 4 5 6 7 8 9'

# delete old file and write heading to file
  if [ -f check.out ] ; then rm check.out ; fi
  echo "Comparing original output files and new output files" | tee check.out
  echo `date`                                                 | tee -a check.out
  echo "Original files in " $DirO                             | tee -a check.out
  echo "  "                                                   | tee -a check.out

  for No in $Tests ; do
    for Stp in $Steps ; do
      for Sfx in $Sufix ; do
        if [ -f $DirO/test$No.$Sfx$Stp -a -f test$No.$Sfx$Stp ] ; then
          # both original and new output files exist, compare
          echo "_________________________________________________________ " \
                test$No.$Sfx$Stp | tee -a check.out
          if diff -b $DirO/test$No.$Sfx$Stp test$No.$Sfx$Stp >> check.out ; then
            echo FILES ARE IDENTICAL | tee -a check.out
          else
            echo FILES DIFFER:  see file check.out for differences
          fi
        fi
      done
    done
  done

# end of shell
