#!/bin/sh
# test.sh -- run daflow and bltm test data sets
#
# Usage: test.sh [start [stop]]
#
#        where: start = starting test number
#                stop = ending test number (may be same as start)
#
# History: 96/05/16 kmflynn

# pathname
WrdA=/usr/opt/wrdapp

#*******************************************************************
#***** You should not need to modify anything below this line. *****
#*******************************************************************

# pathnames and other variables
  Prgm=$WrdA/bltm1.2
  Data=$Prgm/data
  Chck=$Prgm/test
  Anne=$WrdA/annie2.2

  exec 2>&1                                # stderr shows up in .out file
  Start=${1-1}                             # by default, start at 1
  Stop=${2-6}                              # by default, stop at 6
                                           # 1 - Problem F1, steps 1 - 4
                                           # 2 - Problem F3, steps 1 - 4
                                           # 3 - Problem D1, steps 1 - 3
                                           # 4 - Problem W1
                                           # 5 - Problem T1, steps 1 - 9
                                           # 6 - Problem Q2, steps a1-c5

# begin test runs
  echo
  echo _________________________________________________________________
  echo " Begin processing daflow and  bltm test runs $Start to $Stop"
  echo "       programs from: $Prgm"
  echo "      test data from: $Data"
  echo `date`

  if [ $Start -le 1 -a $Stop -ge 1 ] ; then
#   run programs associated with test F1

#   step 1 - compute coefficients for area and width
    if [ -f test_F1.lg1 ] ; then rm test_F1.lg1 ; fi
    $Prgm/bin/cel < $Data/test_F1.cm1 | tee test_F1.lg1

#   step 2 - build the input file for daflow
    if [ -f test_F1.lg2 ] ; then rm test_F1.lg2 ; fi
    if [ -f test_F1.in2 ] ; then rm test_F1.in2 ; fi
    if [ -f flow.in ]     ; then rm flow.in     ; fi
    if [ -f bc ]          ; then rm bc          ; fi
    ln -s $Data/hinton.q bc
    $Prgm/bin/bdaflow < $Data/test_F1.cm2 | tee test_F1.lg2
    cp -p flow.in test_F1.in2

#   step 3 - compute average flow at the grids
    if [ -f test_F1.lg3 ] ; then rm test_F1.lg3 ; fi
    if [ -f test_F1.ot3 ] ; then rm test_F1.ot3 ; fi
    if [ -f test_F1.fl3 ] ; then rm test_F1.fl3 ; fi
    if [ -f flow.out ]    ; then rm flow.out    ; fi
    if [ -f bltm.flw ]    ; then rm bltm.flw    ; fi
    $Prgm/bin/daflow | tee test_F1.lg3
    cp -p bltm.flw test_F1.fl3
    cp -p flow.out test_F1.ot3

#   step 4 - plot the results
    if [ -f test_F1.lg4 ] ; then rm test_F1.lg4 ; fi
    if [ -f test_F1.pl4 ] ; then rm test_F1.pl4 ; fi
    if [ -f test_F1.ps4 ] ; then rm test_F1.ps4 ; fi
    if [ -f plt.out ]     ; then rm plt.out     ; fi
    if [ -f obs ]         ; then rm obs         ; fi
    ln -s $Data/thurmond.q obs
    $Prgm/bin/flwplt < $Data/test_F1.cm4 | tee test_F1.lg4
     cp -p plt.out test_F1.pl4

#   clean up
    for Fil in bc obs flow.in bltm.flw flow.out plt.out ; do
      if [ -f $Fil ] ; then rm $Fil ; fi
    done
  fi


  if [ "$Start" -le 2 -a "$Stop" -ge 2 ] ; then
#   run programs associated with test F3

#   step 1 - read adaps file unit.in (and optional stage discharge
#            rating table file rt) and output file out.
#            Note:  first time in unit.in must be 0.
    if [ -f test_F3.lg1 ] ; then rm test_F3.lg1 ; fi
    if [ -f test_F3.ot1 ] ; then rm test_F3.ot1 ; fi
    if [ -f unit.in ]     ; then rm unit.in     ; fi
    if [ -f rt ]          ; then rm rt          ; fi
    if [ -f out ]         ; then rm out         ; fi
    ln -s $Data/hinton.rt rt
    cp -p $Data/hinton.unt unit.in
    # first time in unit must be zero hours
    ed -s unit.in <<-E-O-T
	/---/+1 t /---/+1
	.-ls/30/00/
	w
	q
	E-O-T
    $Prgm/bin/unit < $Data/test_F3.cm1 | tee test_F3.lg1
    cp -p out test_F3.ot1

#   step 2 - interpolate data to an even interval
    if [ -f test_F3.lg2 ] ; then rm test_F3.lg2 ; fi
    if [ -f test_F3.bc2 ] ; then rm test_F3.bc2 ; fi
    if [ -f in ]          ; then rm in          ; fi
    if [ -f bc ]          ; then rm bc          ; fi
    mv out in
    $Prgm/bin/intrp < $Data/test_F3.cm2 | tee test_F3.lg2
    cp -p bc test_F3.bc2

#   step 3 - build input file for daflow
#            note:  changes made to flow.in from test 1
    if [ -f test_F3.lg3 ] ; then rm test_F3.lg3 ; fi
    if [ -f test_F3.in3 ] ; then rm test_F3.in3 ; fi
    if [ -f old ]         ; then rm old         ; fi
    if [ -f flow.in ]     ; then rm flow.in     ; fi
    cp $Data/test_F1.in2 old
    # change test id, time step, and Disch
    ed -s old <<-E-O-T
	/F1/s//F3/
	/312/s//168/
	g/2356/s//3860
	w
	q
	E-O-T
    $Prgm/bin/bdaflow < $Data/test_F3.cm3 | tee test_F3.lg3
    cp -p flow.in test_F3.in3

#   step 4 - compute average flow at the grids
    if [ -f test_F3.lg4 ] ; then rm test_F3.lg4 ; fi
    if [ -f test_F3.fl4 ] ; then rm test_F3.fl4 ; fi
    if [ -f test_F3.ot4 ] ; then rm test_F3.ot4 ; fi
    if [ -f flow.out ]    ; then rm flow.out    ; fi
    if [ -f bltm.flw ]    ; then rm bltm.flw    ; fi
    $Prgm/bin/daflow | tee test_F3.lg4
    cp -p bltm.flw test_F3.fl4
    cp -p flow.out test_F3.ot4

#   clean up
    for Fil in bc rt in old obs flow.in bltm.flw flow.out ; do
      if [ -f $Fil ] ; then rm $Fil ; fi
    done

  fi

  if [ $Start -le 3 -a $Stop -ge 3 ] ; then
#   run programs associated with test D1

#   step 1 - interpolate data to an even interval
#            note:  changes made for flow.in and out from test 2
    if [ -f test_D1.lg1 ] ; then rm test_D1.lg1 ; fi
    if [ -f test_D1.bc1 ] ; then rm test_D1.bc1 ; fi
    if [ -f in ]        ; then rm in        ; fi
    if [ -f bc ]        ; then rm bc        ; fi
    cp -p $Data/test_D1.in1 in
    $Prgm/bin/intrp < $Data/test_D1.cm1 | tee test_D1.lg1
    cp -p bc test_D1.bc1

#   step 2 - build the input file for daflow
    if [ -f test_D1.lg2 ] ; then rm test_D1.lg2 ; fi
    if [ -f test_D1.in2 ] ; then rm test_D1.in2 ; fi
    if [ -f flow.in   ] ; then rm flow.in   ; fi
    if [ -f old       ] ; then rm old       ; fi
    cp -p $Data/test_F3.in3 old
    ed -s old <<-E-O-T
	/F3/s//D1/
	/168/s//112/
	/1.000/s//0.250/
	/ 6 /s// 5 /
	/ 1 /d
	.-1
	/ 2 /s// 1 /
	/ 3 /s// 2 /
	/ 4 /s// 3 /
	/ 5 /s// 4 /
	/ 6 /s// 5 /
	g/3860. /s//9600. /
	w
	q
	E-O-T
    $Prgm/bin/bdaflow < $Data/test_D1.cm2 | tee test_D1.lg2
    cp -p flow.in test_D1.in2

#   Step 3 - compute average flow at the grids
    if [ -f test_D1.lg3 ] ; then rm test_D1.lg3 ; fi
    if [ -f test_D1.ot3 ] ; then rm test_D1.ot3 ; fi
    if [ -f test_D1.fl3 ] ; then rm test_D1.fl3 ; fi
    if [ -f bltm.flw  ] ; then rm bltm.flw  ; fi
    if [ -f flow.out  ] ; then rm flow.out  ; fi
    $Prgm/bin/daflow | tee test_D1.lg3
    cp -p bltm.flw test_D1.fl3
    cp -p flow.out test_D1.ot3

  fi

  if [ $Start -le 4 -a $Stop -ge 4 ] ; then
#   run programs associated with test W1

#   step 1 - get or build wdm file
    if [ -f $Data/test_W1.wdm ] ; then
#     copy existing wdm file from data directory
      echo " Copying test_W1.wdm from $Data"
      if [ -f test_W1.wdm ] ; then rm test_W1.wdm ; fi
      cp -p $Data/test_W1.wdm test_W1.wdm
    else
#     build wdm file
      echo " Building the test_W1.wdm file using annie"
      rm -f ERROR.FIL ANNIE.LOG
      ln -s $Data/test_W1.exp test_W1.exp
      $Anne/bin/annie <<-EOT
	@$Data/test_W1.cm1
	EOT
      rm -f ERROR.FIL ANNIE.LOG XPAD.DAT
      rm -f test_W1.exp
    fi

#   step 2 - run wdaflo with wdm file input and output
    if [ -f test_W1.lg2 ] ; then rm test_W1.lg2 ; fi
    if [ -f test_W1.ot2 ] ; then rm test_W1.ot2 ; fi
    if [ -f test_W1.fl2 ] ; then rm test_W1.fl2 ; fi
    if [ -f bltm.flw  ] ; then rm bltm.flw  ; fi
    if [ -f flow.out  ] ; then rm flow.out  ; fi
    cp -p $Data/test_W1.in2 flow.in
    $Prgm/bin/wdaflo | tee test_W1.lg2
    cp -p bltm.flw test_W1.fl2
    cp -p flow.out test_W1.ot2

#   step 3 - run wdaflo without wdm file
    if [ -f test_W1.lg3 ] ; then rm test_W1.lg3 ; fi
    if [ -f test_W1.ot3 ] ; then rm test_W1.ot3 ; fi
    if [ -f test_W1.fl3 ] ; then rm test_W1.fl3 ; fi
    if [ -f bltm.flw  ] ; then rm bltm.flw  ; fi
    if [ -f flow.out  ] ; then rm flow.out  ; fi
    cp -p $Data/test_W1.in3 flow.in
    $Prgm/bin/wdaflo | tee test_W1.lg3
    cp -p bltm.flw test_W1.fl3
    cp -p flow.out test_W1.ot3

  fi

  if [ $Start -le 5 -a $Stop -ge 5 ] ; then
#   run programs associated with testing temperature model

#   step 1 - interpolate data to an even interval
    echo
    echo '   test T1, step 1, intrp'
    if [ -f test_T1.lg1 ] ; then rm test_T1.lg1 ; fi
    if [ -f test_T1.bc1 ] ; then rm test_T1.bc1 ; fi
    if [ -f in ]        ; then rm in        ; fi
    if [ -f bc ]        ; then rm bc        ; fi
    cp -p $Data/atlanta.flw in
    $Prgm/bin/intrp < $Data/test_T1.cm1 | tee test_T1.lg1
    cp -p bc test_T1.bc1

#   step 2 - build the input file for daflow
    echo
    echo '   test T1, step 2, bdaflow'
    if [ -f test_T1.lg2 ] ; then rm test_T1.lg2 ; fi
    if [ -f test_T1.in2 ] ; then rm test_T1.in2 ; fi
    if [ -f flow.in   ] ; then rm flow.in   ; fi
    $Prgm/bin/bdaflow < $Data/test_T1.cm2 | tee test_T1.lg2
    cp -p flow.in test_T1.in2

#   Step 3 - compute average flow at the grids
    echo
    echo '   test T1, step 3, daflow'
    if [ -f test_T1.lg3 ] ; then rm test_T1.lg3 ; fi
    if [ -f test_T1.ot3 ] ; then rm test_T1.ot3 ; fi
    if [ -f test_T1.fl3 ] ; then rm test_T1.fl3 ; fi
    if [ -f bltm.flw  ] ; then rm bltm.flw  ; fi
    if [ -f flow.out  ] ; then rm flow.out  ; fi
    $Prgm/bin/daflow | tee test_T1.lg3
    cp -p bltm.flw test_T1.fl3
    cp -p flow.out test_T1.ot3

#   Step 4 - flwplt
    echo
    echo '   test T1, step 4, flwplt'
    if [ -f test_T1.lg4 ] ; then rm test_T1.lg4 ; fi
    if [ -f test_T1.ps4 ] ; then rm test_T1.ps4 ; fi
    if [ -f plt.out   ] ; then rm plt.out   ; fi
    if [ -f bc ]        ; then rm bc        ; fi
    $Prgm/bin/flwplt < $Data/test_T1.cm4 | tee test_T1.lg4
    cp -p plt.out test_T1.pl4
    mv plt.out bc
    if [ -f ERROR.FIL ] ; then rm ERROR.FIL ; fi

#   Step 5 - bdaflow
    echo
    echo '   test T1, step 5, bdaflow'
    if [ -f test_T1.lg5 ] ; then rm test_T1.lg5 ; fi
    if [ -f test_T1.in5 ] ; then rm test_T1.in5 ; fi
    if [ -f flow.in   ] ; then rm flow.in   ; fi
    if [ -f old       ] ; then rm old       ; fi
    cp -p $Data/test_T1.ol5 old
    $Prgm/bin/bdaflow < $Data/test_T1.cm5 | tee test_T1.lg5
    cp -p flow.in test_T1.in5

#   Step 5a - daflow
    echo
    echo '   test T1, step 5a, daflow'
    if [ -f test_T1.ot5 ] ; then rm test_T1.ot5 ; fi
    if [ -f test_T1.fl5 ] ; then rm test_T1.fl5 ; fi
    if [ -f bltm.flw  ] ; then rm bltm.flw  ; fi
    if [ -f flow.out  ] ; then rm flow.out  ; fi
    $Prgm/bin/daflow | tee -a test_T1.lg5
    cp -p bltm.flw test_T1.fl5
    cp -p flow.out test_T1.ot5

#   Step 6 - intrp
    echo
    echo '   test T1, step 6, intrp'
    if [ -f test_T1.lg6 ] ; then rm test_T1.lg6 ; fi
    if [ -f in ]        ; then rm in        ; fi
    if [ -f bc ]        ; then rm bc        ; fi
    cp -p $Data/rte280.tmp in
    $Prgm/bin/intrp < $Data/test_T1.cm6 | tee test_T1.lg6
    cp -p bc test_T1.bc6

#   Step 7 - bbltm
    echo
    echo '   test T1, step 7, bltm'
    if [ -f test_T1.lg7 ] ; then rm test_T1.lg7 ; fi
    if [ -f test_T1.in7 ] ; then rm test_T1.in7 ; fi
    $Prgm/bin/bbltm < $Data/test_T1.cm7 | tee test_T1.lg7
    cp -p bltm.in test_T1.bl7

#   Step 8 - compute equilibrium temperature
    echo
    echo '   test T1, step 8, equltmp'
    if [ -f test_T1.lg8 ] ; then rm test_T1.lg8 ; fi
    if [ -f out       ] ; then rm out       ; fi
    $Prgm/bin/equltmp < $Data/test_T1.cm8 | tee test_T1.lg8
    cp -p out test_T1.ot8
#   all data before 6:30 deleted from out should be same as temp.in

#   Step 9 - bltmt
    echo
    echo '   test T1, step 9, bltmt'
    if [ -f test_T1.lg9 ] ; then rm test_T1.lg9 ; fi
    if [ -f temp.in   ] ; then rm temp.in   ; fi
    cp -p $Data/test_T1.in9 temp.in
    $Prgm/bin/tbltm < $Data/test_T1.cm9 | tee test_T1.lg9
    cp -p bltm.out test_T1.bl9
    cp -p parcel.out test_T1.ot9

  fi

  if [ $Start -le 6 -a $Stop -ge 6 ] ; then
#   run programs associated with testing qual2e model

#   Step a1 - interpolate data to an even interval
    echo
    echo '   test Q2, step a 1, intrp'
    if [ -f test_Q2a.lg1 ] ; then rm test_Q2a.lg1 ; fi
    if [ -f test_Q2a.bc1 ] ; then rm test_Q2a.bc1 ; fi
    if [ -f in  ]          ; then rm in           ; fi
    if [ -f old ]          ; then rm old          ; fi
    if [ -f bc  ]          ; then rm bc           ; fi
    cp -p $Data/rte280.flw in
    $Prgm/bin/intrp < $Data/test_Q2a.cm1 | tee test_Q2a.lg1
    cp -p bc test_Q2a.bc1

#   step 2 - start building the input file for daflow
    echo
    echo '   test Q2, step a 2, bdaflow'
    if [ -f test_Q2a.lg2 ] ; then rm test_Q2a.lg2 ; fi
    if [ -f test_Q2a.in2 ] ; then rm test_Q2a.in2 ; fi
    if [ -f old     ]      ; then rm old          ; fi
    if [ -f flow.in ]      ; then rm flow.in      ; fi
    cp -p $Data/test_Q2a.ol2 old
    $Prgm/bin/bdaflow < $Data/test_Q2a.cm2 | tee test_Q2a.lg2
    cp -p flow.in test_Q2a.in2
    cp -p flow.in old

#   step 3 - finish building the input file for daflow
    echo
    echo '   test Q2, step a 3, bdaflow'
    if [ -f test_Q2a.lg3 ] ; then rm test_Q2a.lg3 ; fi
    if [ -f test_Q2a.in3 ] ; then rm test_Q2a.in3 ; fi
    if [ -f bc      ]      ; then rm bc           ; fi
    if [ -f flow.in ]      ; then rm flow.in      ; fi
    cp -p $Data/sweetwater.flw bc
    $Prgm/bin/bdaflow < $Data/test_Q2a.cm3 | tee test_Q2a.lg3
    cp -p flow.in test_Q2a.in3

#   Step 4 - compute average flow at the grids
    echo
    echo '   test Q2, step a 4, daflow'
    if [ -f test_Q2a.lg4 ] ; then rm test_Q2a.lg4 ; fi
    if [ -f test_Q2a.ot4 ] ; then rm test_Q2a.ot4 ; fi
    if [ -f test_Q2a.fl4 ] ; then rm test_Q2a.fl4 ; fi
    if [ -f flow.out  ] ; then rm flow.out  ; fi
    if [ -f bltm.flw  ] ; then rm bltm.flw  ; fi
    $Prgm/bin/daflow | tee test_Q2a.lg4
    cp -p flow.out test_Q2a.ot4
    cp -p bltm.flw test_Q2a.fl4

#   Step 5 - flwplt
    echo
    echo '   test Q2, step a 5, flwplt'
    if [ -f test_Q2a.lg5 ] ; then rm test_Q2a.lg5 ; fi
    if [ -f test_Q2a.ps5 ] ; then rm test_Q2a.ps5 ; fi
    if [ -f test_Q2a.pl5 ] ; then rm test_Q2a.pl5 ; fi
    if [ -f obs     ]      ; then rm obs          ; fi
    if [ -f plt.out ]      ; then rm plt.out      ; fi
    cp -p $Data/whitesburg.flw obs
    $Prgm/bin/flwplt < $Data/test_Q2a.cm5 | tee test_Q2a.lg5
    cp -p plt.out test_Q2a.pl5
    cp -p plt.out bc
    if [ -f ERROR.FIL ] ; then rm ERROR.FIL ; fi

#   Step 6 - intrp
    echo
    echo '   test Q2, step a 6, intrp'
    if [ -f test_Q2a.lg6 ] ; then rm test_Q2a.lg6 ; fi
    if [ -f in ]           ; then rm in           ; fi
    if [ -f bc ]           ; then rm bc           ; fi
    cp -p $Data/test_Q2a.pl6 in
    $Prgm/bin/intrp < $Data/test_Q2a.cm6 | tee test_Q2a.lg6
    cp -p bc test_Q2a.bc6

#   Step 7 - bdaflow
    echo
    echo '   test Q2, step a 7, bdaflow'
    if [ -f test_Q2a.lg7 ] ; then rm test_Q2a.lg7 ; fi
    if [ -f test_Q2a.in7 ] ; then rm test_Q2a.in7 ; fi
    if [ -f old       ] ; then rm old       ; fi
    if [ -f flow.in   ] ; then rm flow.in   ; fi
    cp -p $Data/test_Q2a.ol7 old
    $Prgm/bin/bdaflow < $Data/test_Q2a.cm7 | tee test_Q2a.lg7
    cp -p flow.in test_Q2a.in7

#   Step 8 - daflow
    echo
    echo '   test Q2, step a 8, daflow'
    if [ -f test_Q2a.lg8 ] ; then rm test_Q2a.lg8 ; fi
    if [ -f test_Q2a.ot8 ] ; then rm test_Q2a.ot8 ; fi
    if [ -f test_Q2a.fl8 ] ; then rm test_Q2a.fl8 ; fi
    if [ -f flow.out ]     ; then rm flow.out     ; fi
    if [ -f bltm.flw ]     ; then rm bltm.flw     ; fi
    $Prgm/bin/daflow | tee test_Q2a.lg8
    cp -p flow.out test_Q2a.ot8
    cp -p bltm.flw test_Q2a.fl8

#   Step b1 - interpolate data to an even interval
    echo
    echo '   test Q2, step b 1, intrp'
    if [ -f test_Q2b.lg1 ] ; then rm test_Q2b.lg1 ; fi
    if [ -f test_Q2b.bc1 ] ; then rm test_Q2b.bc1 ; fi
    if [ -f in  ]          ; then rm in           ; fi
    if [ -f bc  ]          ; then rm bc           ; fi
    cp -p $Data/rte280.in1 in
    $Prgm/bin/intrp < $Data/test_Q2b.cm1 | tee test_Q2b.lg1
    cp -p bc test_Q2b.bc1
    cp -p bc in

#   Step b2 - merge data
    echo
    echo '   test Q2, step b 2, mrg'
    if [ -f test_Q2b.lg2 ] ; then rm test_Q2b.lg2 ; fi
    if [ -f test_Q2b.bc2 ] ; then rm test_Q2b.bc2 ; fi
    if [ -f old.bc ]       ; then rm old.bc       ; fi
    if [ -f bc  ]          ; then rm bc           ; fi
    cp -p $Data/rte280.bc old.bc
    $Prgm/bin/mrg < $Data/test_Q2b.cm2 | tee test_Q2b.lg2
    cp -p bc test_Q2b.bc2
    cp -p bc old.bc

#   Step b3 - interpolate data to an even interval
    echo
    echo '   test Q2, step b 3, intrp'
    if [ -f test_Q2b.lg3 ] ; then rm test_Q2b.lg3 ; fi
    if [ -f test_Q2b.bc3 ] ; then rm test_Q2b.bc3 ; fi
    if [ -f in  ]          ; then rm in           ; fi
    if [ -f bc  ]          ; then rm bc           ; fi
    cp -p $Data/rte280.in3 in
    $Prgm/bin/intrp < $Data/test_Q2b.cm3 | tee test_Q2b.lg3
    cp -p bc test_Q2b.bc3
    cp -p bc in

#   Step b4 - merge data
    echo
    echo '   test Q2, step b 4, mrg'
    if [ -f test_Q2b.lg4 ] ; then rm test_Q2b.lg4 ; fi
    if [ -f test_Q2b.bc4 ] ; then rm test_Q2b.bc4 ; fi
    if [ -f bc     ]       ; then rm bc           ; fi
    $Prgm/bin/mrg < $Data/test_Q2b.cm4 | tee test_Q2b.lg4
    cp -p bc test_Q2b.bc4

#   Step b5 - start building bltm input
    echo
    echo '   test Q2, step b 5, bltm'
    if [ -f test_Q2b.lg5 ] ; then rm test_Q2b.lg5 ; fi
    if [ -f test_Q2b.in5 ] ; then rm test_Q2b.in5 ; fi
    if [ -f test_Q2b.bl5 ] ; then rm test_Q2b.bl5 ; fi
    if [ -f bltm.in  ]     ; then rm bltm.in      ; fi
    $Prgm/bin/bbltm < $Data/test_Q2b.cm5 | tee test_Q2b.lg5
    cp -p bltm.in test_Q2b.bl5
    cp -p bltm.in old

#   Step b6 - continue building bltm input
    echo
    echo '   test Q2, step b 6, bltm'
    if [ -f test_Q2b.lg6 ] ; then rm test_Q2b.lg6 ; fi
    if [ -f test_Q2b.in6 ] ; then rm test_Q2b.in6 ; fi
    if [ -f test_Q2b.bl6 ] ; then rm test_Q2b.bl6 ; fi
    if [ -f bltm.in  ]     ; then rm bltm.in      ; fi
    cp -p $Data/grid3.bc bc
    $Prgm/bin/bbltm < $Data/test_Q2b.cm6 | tee test_Q2b.lg6
    cp -p bltm.in test_Q2b.bl6
    cp -p bltm.in old

#   Step b7 - finish building bltm input
    echo
    echo '   test Q2, step b 7, bltm'
    if [ -f test_Q2b.lg7 ] ; then rm test_Q2b.lg7 ; fi
    if [ -f test_Q2b.in7 ] ; then rm test_Q2b.in7 ; fi
    if [ -f test_Q2b.bl7 ] ; then rm test_Q2b.bl7 ; fi
    if [ -f bltm.in  ]     ; then rm bltm.in      ; fi
    cp -p $Data/grid5.bc bc
    $Prgm/bin/bbltm < $Data/test_Q2b.cm7 | tee test_Q2b.lg7
    cp -p bltm.in test_Q2b.bl7

#   Step c1 - compute equilibrium
    echo
    echo '   test Q1, step c 1, equltmp'
    if [ -f test_Q2c.lg1 ] ; then rm test_Q2c.lg1 ; fi
    if [ -f out ]          ; then rm out          ; fi
    $Prgm/bin/equltmp < $Data/test_Q2c.cm1 | tee test_Q2c.lg1
    cp -p out test_Q2c.ot1
    cp -p out old.bc

#   Step c2 - solar
    echo
    echo '   test Q1, step c 2 solar'
    if [ -f test_Q2c.lg2 ] ; then rm test_Q2c.lg2 ; fi
    if [ -f out ]          ; then rm out          ; fi
    $Prgm/bin/solar < $Data/test_Q2c.cm2 | tee test_Q2c.lg2
    cp -p out test_Q2c.ot2
    cp -p out in

#   Step c3 - merge data
    echo
    echo '   test Q2, step c 3, mrg'
    if [ -f test_Q2c.lg3 ] ; then rm test_Q2c.lg3 ; fi
    if [ -f test_Q2c.bc3 ] ; then rm test_Q2c.bc3 ; fi
    if [ -f bc     ]       ; then rm bc           ; fi
    $Prgm/bin/mrg < $Data/test_Q2c.cm3 | tee test_Q2c.lg3
    cp -p bc test_Q2c.bc3

#   Step c4 - build qual2e.in
    echo
    echo '   test Q2, step c 4 bqual2e'
    if [ -f test_Q2c.lg4 ] ; then rm test_Q2c.lg4 ; fi
    if [ -f test_Q2c.in4 ] ; then rm test_Q2c.in4 ; fi
    if [ -f qual2e.in ]    ; then rm qual2e.in    ; fi
    cp -p $Data/test_Q2c.bc4 bc
    $Prgm/bin/bqual2e < $Data/test_Q2c.cm4 | tee test_Q2c.lg4
    cp -p qual2e.in test_Q2c.in4

#   Step c5 - run bltm with  qual2e
    echo
    echo '   test Q2, step c 5 bltmq'
    if [ -f test_Q2c.lg5 ] ; then rm test_Q2c.lg5 ; fi
    if [ -f test_Q2c.bl5 ] ; then rm test_Q2c.bl5 ; fi
    if [ -f test_Q2c.ot5 ] ; then rm test_Q2c.ot5 ; fi
    if [ -f bltm.out   ]   ; then rm bltm.out     ; fi
    if [ -f parcel.out ]   ; then rm parcel.out   ; fi
    $Prgm/bin/qbltm < $Data/test_Q2c.cm5 | tee test_Q2c.lg5
    cp -p bltm.out test_Q2c.bl5
    cp -p parcel.out test_Q2c.ot5

  fi

  echo
  echo
  echo _________________________________________________________________
  echo "Completed bltm test runs $Start to $Stop"
  echo
# check output against original output in data directory
  $Chck/check.sh $Data

# end of shell
